@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_PARAM__HPP
 #define CORELIB___NCBI_PARAM__HPP
 
-/*  $Id: ncbi_param.hpp 208067 2010-10-13 18:26:36Z grichenk $
+/*  $Id: ncbi_param.hpp 445853 2014-09-09 13:27:06Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -89,6 +89,7 @@ BEGIN_NCBI_SCOPE
 ///
 
 /// Get string configuration value.
+/// NOTE: The function is not MT-safe.
 ///
 /// @param section
 ///   Check application configuration named section first if not null.
@@ -112,6 +113,7 @@ string NCBI_XNCBI_EXPORT g_GetConfigString(const char* section,
                                            const char* default_value);
 
 /// Get integer configuration value.
+/// NOTE: The function is not MT-safe.
 ///
 /// @param section
 ///   Check application configuration named section first if not null.
@@ -135,6 +137,7 @@ int NCBI_XNCBI_EXPORT g_GetConfigInt(const char* section,
                                      int         default_value);
 
 /// Get boolean configuration value.
+/// NOTE: The function is not MT-safe.
 ///
 /// @param section
 ///   Check application configuration named section first if not null.
@@ -158,6 +161,30 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
                                        bool        default_value);
 
 
+/// Get double configuration value.
+/// NOTE: The function is not MT-safe.
+///
+/// @param section
+///   Check application configuration named section first if not null.
+/// @param variable
+///   Variable name within application section.
+///   If no value found in configuration file, environment variable with
+///   name NCBI_CONFIG__section__variable or NCBI_CONFIG__variable will be
+///   checked, depending on wether section is null.
+/// @param env_var_name
+///   If not empty, overrides the default NCBI_CONFIG__section__name
+///   name of the environment variable.
+/// @param default_value
+///   If no value found neither in configuration file nor in environment,
+///   this value will be returned.
+/// @return
+///   double configuration value.
+/// @sa g_GetConfigString(), g_GetConfigInt()
+double NCBI_XNCBI_EXPORT g_GetConfigDouble(const char* section,
+                                           const char* variable,
+                                           const char* env_var_name,
+                                           double  default_value);
+
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// Parameter declaration and definition macros
@@ -178,36 +205,40 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
 
 // Common part of the param description structure. 'desctype' can be
 // SParamDescription or SParamEnumDescription.
-#define X_NCBI_PARAM_DESC_DECL(type, desctype)                              \
+#define X_NCBI_PARAM_DESC_DECL(type, desctype, tagtype)                     \
     {                                                                       \
+        typedef tagtype TTagType;                                           \
         typedef type TValueType;                                            \
         typedef desctype<TValueType> TDescription;                          \
+        typedef TDescription::TStaticValue TStaticValue;                    \
         typedef CStaticTls< type > TTls;                                    \
         static TDescription sm_ParamDescription;                            \
-        static TValueType sm_Default;                                       \
+        static TStaticValue sm_Default;                                     \
         static bool sm_DefaultInitialized;                                  \
         static TTls sm_ValueTls;                                            \
         static CParamBase::EParamState sm_State;                            \
     }
 
 // Common definitions related to enum parser.
-#define X_NCBI_PARAM_ENUM_PARSER_DECL(type)                                 \
+// 'ptype' is the final parameter type used to make the parser unique even
+// if the same enum is used by another parameter.
+#define X_NCBI_PARAM_ENUM_PARSER_DECL(type, ptype)                          \
     EMPTY_TEMPLATE inline                                                   \
-    CParamParser< SParamEnumDescription< type > >::TValueType               \
-    CParamParser< SParamEnumDescription< type > >::                         \
+    CParamParser< SParamEnumDescription< type >, ptype >::TValueType        \
+    CParamParser< SParamEnumDescription< type >, ptype >::                  \
     StringToValue(const string&     str,                                    \
                   const TParamDesc& descr)                                  \
-    { return CEnumParser< type >::StringToEnum(str, descr); }               \
+    { return CEnumParser< type, ptype >::StringToEnum(str, descr); }        \
     EMPTY_TEMPLATE inline string                                            \
-    CParamParser< SParamEnumDescription< type > >::                         \
+    CParamParser< SParamEnumDescription< type >, ptype >::                  \
     ValueToString(const TValueType& val,                                    \
                   const TParamDesc& descr)                                  \
-    { return CEnumParser< type >::EnumToString(val, descr); }
+    { return CEnumParser< type, ptype >::EnumToString(val, descr); }
 
 // Defenition of SNcbiParamDesc_XXXX static members common for normal
 // and enum parameters.
-#define X_NCBI_PARAM_STATIC_DEF(type, descname, defval)                     \
-    type descname::sm_Default = defval;                                     \
+#define X_NCBI_PARAM_STATIC_DEF(descname, defval)                           \
+    descname::TStaticValue descname::sm_Default = defval;                   \
     bool descname::sm_DefaultInitialized = false;                           \
     descname::TTls descname::sm_ValueTls;                                   \
     CParamBase::EParamState descname::sm_State = CParamBase::eState_NotSet  \
@@ -224,33 +255,37 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
 /// @sa NCBI_PARAM_DEF
 #define NCBI_PARAM_DECL(type, section, name)                                \
     struct X_NCBI_PARAM_DECLNAME(section, name)                             \
-    X_NCBI_PARAM_DESC_DECL(type, SParamDescription)
+    X_NCBI_PARAM_DESC_DECL(type, SParamDescription, type)
 
 
 /// Same as NCBI_PARAM_DECL but with export specifier (e.g. NCBI_XNCBI_EXPORT)
 /// @sa NCBI_PARAM_DECL
 #define NCBI_PARAM_DECL_EXPORT(expname, type, section, name)                \
     struct expname X_NCBI_PARAM_DECLNAME(section, name)                     \
-    X_NCBI_PARAM_DESC_DECL(type, SParamDescription)
+    X_NCBI_PARAM_DESC_DECL(type, SParamDescription, type)
 
 
 /// Enum parameter declaration. In addition to NCBI_PARAM_DECL also
-/// specializes CParamParser<type> to convert between strings and
+/// specializes CParamParser<type, ptype> to convert between strings and
 /// enum values.
 /// @sa NCBI_PARAM_ENUM_ARRAY
 /// @sa NCBI_PARAM_ENUM_DEF
-#define NCBI_PARAM_ENUM_DECL(type, section, name)                           \
-    X_NCBI_PARAM_ENUM_PARSER_DECL(type)                                     \
-    struct X_NCBI_PARAM_DECLNAME(section, name)                             \
-    X_NCBI_PARAM_DESC_DECL(type, SParamEnumDescription)
+#define NCBI_PARAM_ENUM_DECL(type, section, name)                             \
+    struct X_NCBI_PARAM_DECLNAME(section, name);                              \
+    X_NCBI_PARAM_ENUM_PARSER_DECL(type, X_NCBI_PARAM_DECLNAME(section, name)) \
+    struct X_NCBI_PARAM_DECLNAME(section, name)                               \
+    X_NCBI_PARAM_DESC_DECL(type, SParamEnumDescription,                       \
+        X_NCBI_PARAM_DECLNAME(section, name))
 
 
 /// Same as NCBI_PARAM_ENUM_DECL but with export specifier (e.g. NCBI_XNCBI_EXPORT)
 /// @sa NCBI_PARAM_ENUM_DECL
-#define NCBI_PARAM_ENUM_DECL_EXPORT(expname, type, section, name)           \
-    X_NCBI_PARAM_ENUM_PARSER_DECL(type)                                     \
-    struct expname X_NCBI_PARAM_DECLNAME(section, name)                     \
-    X_NCBI_PARAM_DESC_DECL(type, SParamEnumDescription)
+#define NCBI_PARAM_ENUM_DECL_EXPORT(expname, type, section, name)             \
+    struct expname X_NCBI_PARAM_DECLNAME(section, name);                      \
+    X_NCBI_PARAM_ENUM_PARSER_DECL(type, X_NCBI_PARAM_DECLNAME(section, name)) \
+    struct expname X_NCBI_PARAM_DECLNAME(section, name)                       \
+    X_NCBI_PARAM_DESC_DECL(type, SParamEnumDescription,                       \
+        X_NCBI_PARAM_DECLNAME(section, name))
 
 
 /// Parameter definition. "value" is used to set the initial parameter
@@ -260,8 +295,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
     SParamDescription< type >                                               \
     X_NCBI_PARAM_DECLNAME(section, name)::sm_ParamDescription =             \
         { #section, #name, 0, default_value, NULL, eParam_Default };        \
-    X_NCBI_PARAM_STATIC_DEF(type,                                           \
-        X_NCBI_PARAM_DECLNAME(section, name),                               \
+    X_NCBI_PARAM_STATIC_DEF(X_NCBI_PARAM_DECLNAME(section, name),           \
         default_value)
 
 
@@ -272,8 +306,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
     SParamDescription< type >                                               \
     X_NCBI_PARAM_DECLNAME(section, name)::sm_ParamDescription =             \
         { #section, #name, 0, default_value, init, eParam_Default };        \
-    X_NCBI_PARAM_STATIC_DEF(type,                                           \
-        X_NCBI_PARAM_DECLNAME(section, name),                               \
+    X_NCBI_PARAM_STATIC_DEF(X_NCBI_PARAM_DECLNAME(section, name),           \
         default_value)
 
 
@@ -284,8 +317,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
     SParamDescription< type >                                               \
     X_NCBI_PARAM_DECLNAME(section, name)::sm_ParamDescription =             \
         { #section, #name, #env, default_value, NULL, flags };              \
-    X_NCBI_PARAM_STATIC_DEF(type,                                           \
-        X_NCBI_PARAM_DECLNAME(section, name),                               \
+    X_NCBI_PARAM_STATIC_DEF(X_NCBI_PARAM_DECLNAME(section, name),           \
         default_value)
 
 
@@ -297,8 +329,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
     SParamDescription< type >                                               \
     X_NCBI_PARAM_DECLNAME(section, name)::sm_ParamDescription =             \
         { #section, #name, #env, def_value, init, flags };                  \
-    X_NCBI_PARAM_STATIC_DEF(type,                                           \
-        X_NCBI_PARAM_DECLNAME(section, name),                               \
+    X_NCBI_PARAM_STATIC_DEF(X_NCBI_PARAM_DECLNAME(section, name),           \
         def_value)
 
 
@@ -309,7 +340,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
     SParamDescription< type >                                                \
     X_NCBI_PARAM_DECLNAME_SCOPE(scope, section, name)::sm_ParamDescription = \
         { #section, #name, 0, default_value, NULL, eParam_Default };         \
-    X_NCBI_PARAM_STATIC_DEF(type,                                            \
+    X_NCBI_PARAM_STATIC_DEF(                                                 \
         X_NCBI_PARAM_DECLNAME_SCOPE(scope, section, name),                   \
         default_value)
 
@@ -327,8 +358,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
         { #section, #name, 0, default_value, NULL, eParam_Default,          \
           X_NCBI_PARAM_ENUMNAME(section, name),                             \
           ArraySize(X_NCBI_PARAM_ENUMNAME(section, name)) };                \
-    X_NCBI_PARAM_STATIC_DEF(type,                                           \
-        X_NCBI_PARAM_DECLNAME(section, name),                               \
+    X_NCBI_PARAM_STATIC_DEF(X_NCBI_PARAM_DECLNAME(section, name),           \
         default_value)
 
 
@@ -342,8 +372,7 @@ bool NCBI_XNCBI_EXPORT g_GetConfigFlag(const char* section,
         { #section, #name, #env, default_value, NULL, flags,                \
           X_NCBI_PARAM_ENUMNAME(section, name),                             \
           ArraySize(X_NCBI_PARAM_ENUMNAME(section, name)) };                \
-    X_NCBI_PARAM_STATIC_DEF(type,                                           \
-        X_NCBI_PARAM_DECLNAME(section, name),                               \
+    X_NCBI_PARAM_STATIC_DEF(X_NCBI_PARAM_DECLNAME(section, name),           \
         default_value)
 
 
@@ -385,7 +414,7 @@ public:
 ///
 
 
-template<class TDescription>
+template<class TDescription, class TParam>
 class CParamParser
 {
 public:
@@ -411,19 +440,16 @@ class NCBI_XNCBI_EXPORT CParamBase
 public:
     /// Current param state flag - indicates which possible sources for
     /// the param have been checked. This flag does not indicate where
-    /// does the current value originate from (except eState_User).
-    /// It just shows the stage of parameter loading process.
+    /// does the current value originate from. It just shows the stage
+    /// of parameter loading process.
     enum EParamState {
         eState_NotSet = 0, ///< The param's value has not been set yet
         eState_InFunc = 1, ///< The initialization function is being executed
         eState_Func   = 2, ///< Initialized using FParamInit function
-        eState_EnvVar = 3, ///< The environment variable has been checked
-        eState_Config = 4, ///< The app. config file has been checked
-        eState_User   = 5  ///< Value has been set by user
+        eState_User   = 3, ///< Value has been set by user
+        eState_EnvVar = 4, ///< The environment variable has been checked
+        eState_Config = 5  ///< The app. config file has been checked
     };
-
-protected:
-    static SSystemMutex& s_GetLock(void);
 };
 
 
@@ -467,11 +493,12 @@ template<class TDescription>
 class CParam : public CParamBase
 {
 public:
-    typedef CParam<TDescription>                   TParam;
-    typedef typename TDescription::TDescription    TParamDescription;
-    typedef typename TParamDescription::TValueType TValueType;
-    typedef CParamParser<TParamDescription>        TParamParser;
-    typedef typename TDescription::TTls            TTls;
+    typedef CParam<TDescription>                        TParam;
+    typedef typename TDescription::TTagType             TTagType;
+    typedef typename TDescription::TDescription         TParamDescription;
+    typedef typename TParamDescription::TValueType      TValueType;
+    typedef CParamParser<TParamDescription, TTagType>   TParamParser;
+    typedef typename TDescription::TTls                 TTls;
 
     /// Create parameter with the thread default or global default value.
     /// Changing defaults does not affect the existing parameter objects.
@@ -490,12 +517,17 @@ public:
     /// Get current state of the param.
     static EParamState GetState(void);
 
-    /// Get current parameter value.
+    /// Get current parameter value. It is safe to get value from
+    /// multiple threads, but setting a new value is not MT-safe.
     TValueType Get(void) const;
     /// Set new parameter value (this instance only).
+    /// @note The method is not MT-safe. Use an additional external lock
+    /// if it's necessary for multiple threads to use the same variable.
     void Set(const TValueType& val);
     /// Reset value as if it has not been initialized yet. Next call to
     /// Get() will cache the thread default (or global default) value.
+    /// @note The method is not MT-safe. Use an additional external lock
+    /// if it's necessary for multiple threads to use the same variable.
     void Reset(void);
 
     /// Get global default value. If not yet set, attempts to load the value
@@ -518,6 +550,8 @@ public:
     static void ResetThreadDefault(void);
 
 private:
+    static SSystemMutex& s_GetLock(void);
+
     static TValueType& sx_GetDefault(bool force_reset = false);
     static TTls&       sx_GetTls    (void);
     static EParamState& sx_GetState(void);
