@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___DBAPI_DRIVER_CONVERT__HPP
 #define DBAPI_DRIVER___DBAPI_DRIVER_CONVERT__HPP
 
-/* $Id: dbapi_driver_convert.hpp 169152 2009-08-26 16:47:29Z ivanovp $
+/* $Id: dbapi_driver_convert.hpp 498375 2016-04-15 17:20:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NTOICE
@@ -65,7 +65,7 @@ public:
         CDB_Bit db_obj;
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL to bool");
         }
         return db_obj.Value() != 0;
     }
@@ -98,9 +98,9 @@ public:
         CDB_VarChar db_obj;
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL to string");
         }
-        return string(db_obj.Value(), db_obj.Size());
+        return string(db_obj.Data(), db_obj.Size());
     }
     operator CTime(void) const
     {
@@ -114,7 +114,7 @@ private:
         FROM db_obj;
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL");
         }
         return MakeCP<CP>(db_obj.Value());
     }
@@ -157,7 +157,9 @@ public:
                 return bool();
             }
 
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONVERSION_ERROR("non-NULL CDB_Int ("
+                                         << db_obj_int.Value()
+                                         << ") to bool");
         }
 
         CDB_Bit db_obj;
@@ -217,7 +219,9 @@ public:
                 return string();
             }
 
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONVERSION_ERROR("non-NULL CDB_Int("
+                                         << db_obj_int.Value()
+                                         << ") to string");
         }
 
         CDB_VarChar db_obj;
@@ -225,7 +229,7 @@ public:
         if (db_obj.IsNULL()) {
             return string();
         }
-        return string(db_obj.Value(), db_obj.Size());
+        return string(db_obj.Data(), db_obj.Size());
     }
     operator CTime(void) const
     {
@@ -250,7 +254,9 @@ private:
                 return TO();
             }
 
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONVERSION_ERROR("non-NULL CDB_Int("
+                                         << db_obj_int.Value()
+                                         << ')');
         }
 
         FROM db_obj;
@@ -302,7 +308,7 @@ private:
 
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (obj_wrapper.is_null()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL");
         }
 
         // return MakeCP<SRunTimeCP>(obj_wrapper.get_value());
@@ -317,10 +323,10 @@ private:
 
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL");
         }
 
-        return Convert(db_obj.Value());
+        return NCBI_CONVERT_TO(Convert(db_obj.Value()), TO);
     }
 
     template <typename TO, typename FROM>
@@ -330,10 +336,11 @@ private:
 
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL string");
         }
 
-        return Convert(string(static_cast<const char*>(db_obj.Value()), db_obj.Size()));
+        return Convert(string(static_cast<const char*>(db_obj.Data()),
+                              db_obj.Size()));
     }
 
     template <typename TO, typename FROM>
@@ -343,10 +350,11 @@ private:
 
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL fixed-length string");
         }
 
-        return Convert(string(static_cast<const char*>(db_obj.Value()), db_obj.Size()));
+        return Convert(string(static_cast<const char*>(db_obj.Data()),
+                              db_obj.Size()));
     }
 
     template <typename TO, typename FROM>
@@ -357,11 +365,11 @@ private:
 
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL BLOB");
         }
 
         result.resize(db_obj.Size());
-        db_obj.Read(const_cast<void*>(static_cast<const void*>(result.c_str())),
+        db_obj.Read(const_cast<void*>(static_cast<const void*>(result.data())),
                 db_obj.Size()
                 );
 
@@ -375,7 +383,7 @@ private:
 
         const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
         if (db_obj.IsNULL()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("NULL to time");
         }
 
         return CTime(time_t(Convert(db_obj.Value())));
@@ -428,6 +436,12 @@ private:
             case eDB_Image:
                 value = ConvertFromLOB<TO, CDB_Image>();
                 break;
+            case eDB_VarCharMax:
+                value = ConvertFromLOB<TO, CDB_VarCharMax>();
+                break;
+            case eDB_VarBinaryMax:
+                value = ConvertFromLOB<TO, CDB_VarBinaryMax>();
+                break;
             case eDB_Bit:
                 value = ConvertFrom<TO, CDB_Bit>();
                 break;
@@ -443,14 +457,16 @@ private:
 
                     const_cast<CDB_Result*>(m_Value)->GetItem(&db_obj);
                     if (db_obj.IsNULL()) {
-                        throw CInvalidConversionException();
+                        NCBI_REPORT_CONSTANT_CONVERSION_ERROR
+                            ("NULL long binary");
                     }
 
                     value = Convert(string(static_cast<const char*>(db_obj.Value()), db_obj.DataSize()));
                 }
                 break;
             default:
-                throw CInvalidConversionException();
+                NCBI_REPORT_CONVERSION_ERROR("CDB_Object of unsupported type "
+                                             << db_type);
         }
     }
 
@@ -493,7 +509,7 @@ private:
             return TO();
         }
 
-        return Convert(db_obj.Value());
+        return NCBI_CONVERT_TO(Convert(db_obj.Value()), TO);
     }
 
     template <typename TO, typename FROM>
@@ -506,7 +522,8 @@ private:
             return TO();
         }
 
-        return Convert(string(static_cast<const char*>(db_obj.Value()), db_obj.Size()));
+        return Convert(string(static_cast<const char*>(db_obj.Data()),
+                              db_obj.Size()));
     }
 
     template <typename TO, typename FROM>
@@ -519,7 +536,8 @@ private:
             return TO();
         }
 
-        return Convert(string(static_cast<const char*>(db_obj.Value()), db_obj.Size()));
+        return Convert(string(static_cast<const char*>(db_obj.Data()),
+                              db_obj.Size()));
     }
 
     template <typename TO, typename FROM>
@@ -534,7 +552,7 @@ private:
         }
 
         result.resize(db_obj.Size());
-        db_obj.Read(const_cast<void*>(static_cast<const void*>(result.c_str())),
+        db_obj.Read(const_cast<void*>(static_cast<const void*>(result.data())),
                 db_obj.Size()
                 );
 
@@ -623,7 +641,8 @@ private:
                 }
                 break;
             default:
-                throw CInvalidConversionException();
+                NCBI_REPORT_CONVERSION_ERROR("CDB_Object of unsupported type "
+                                             << db_type);
         }
     }
 
@@ -862,7 +881,9 @@ public:
     static void Convert(const auto_ptr<CDB_Result>& rs, TValue& value)
     {
         if (rs->Fetch()) {
-             value = CMakeObject<CP, TValue, CDB_Result>::Make(*rs);
+            value = CMakeObject<CP, TValue, CDB_Result>::Make(*rs);
+        } else {
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("valueless result");
         }
     }
 };
@@ -955,7 +976,7 @@ public:
     : m_Stmt(&value)
     {
         if (!m_Stmt->Send()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("unsendable CDB_LangCmd");
         }
     }
     ~CValueConvert(void)
@@ -985,10 +1006,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        NCBI_REPORT_CONSTANT_CONVERSION_ERROR("failed CDB_LangCmd");
     }
 
 private:
@@ -1009,7 +1031,7 @@ public:
     : m_Stmt(value)
     {
         if (!m_Stmt->Send()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("unsendable CDB_LangCmd");
         }
     }
     ~CValueConvert(void)
@@ -1039,10 +1061,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        NCBI_REPORT_CONSTANT_CONVERSION_ERROR("failed CDB_LangCmd");
     }
 
 private:
@@ -1064,7 +1087,7 @@ public:
     : m_Stmt(&value)
     {
         if (!m_Stmt->Send()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("unsendable CDB_RPCCmd");
         }
     }
     ~CValueConvert(void)
@@ -1094,10 +1117,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        NCBI_REPORT_CONSTANT_CONVERSION_ERROR("failed CDB_RPCCmd");
     }
 
 private:
@@ -1118,7 +1142,7 @@ public:
     : m_Stmt(value)
     {
         if (!m_Stmt->Send()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("unsendable CDB_RPCCmd");
         }
     }
     ~CValueConvert(void)
@@ -1148,10 +1172,11 @@ public:
 
             CConvertTO<CP, TO>::Convert(rs, result);
 
-            break;
+            return result;
         }
 
-        return result;
+        // return TO();
+        NCBI_REPORT_CONSTANT_CONVERSION_ERROR("failed CDB_RPCCmd");
     }
 
 private:
@@ -1175,7 +1200,7 @@ public:
     , m_RS(value.Open())
     {
         if (!m_RS.get()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("failed CDB_CursorCmd");
         }
     }
     ~CValueConvert(void)
@@ -1195,7 +1220,7 @@ public:
 
 private:
     TObj* m_Stmt;
-    auto_ptr<CDB_Result> m_RS;
+    mutable auto_ptr<CDB_Result> m_RS;
 };
 
 template <typename CP>
@@ -1214,7 +1239,7 @@ public:
     , m_RS(value->Open())
     {
         if (!m_RS.get()) {
-            throw CInvalidConversionException();
+            NCBI_REPORT_CONSTANT_CONVERSION_ERROR("failed CDB_CursorCmd");
         }
     }
     ~CValueConvert(void)
