@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___PUBLIC__HPP
 #define DBAPI_DRIVER___PUBLIC__HPP
 
-/* $Id: public.hpp 440628 2014-07-15 18:57:56Z ucko $
+/* $Id: public.hpp 498375 2016-04-15 17:20:50Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -54,6 +54,7 @@ namespace impl
 }
 
 
+class CAutoTrans;
 template <class I> class CInterfaceHook;
 
 
@@ -112,10 +113,10 @@ public:
     ///   Newly created send-data object.
     ///
     /// @sa SendData
-    virtual CDB_SendDataCmd* SendDataCmd(I_ITDescriptor& desc,
-                                         size_t          data_size,
-                                         bool            log_it = true,
-                                         bool            discard_results = true);
+    virtual CDB_SendDataCmd* SendDataCmd(I_BlobDescriptor& desc,
+                                         size_t            data_size,
+                                         bool              log_it = true,
+                                         bool              discard_results = true);
 
     /// @brief 
     ///   Shortcut to send text and image to the server without using the
@@ -124,7 +125,7 @@ public:
     /// @param desc 
     ///   Lob descriptor.
     /// @param lob 
-    ///   Text or Image object.
+    ///   Large object -- [N]TEXT, IMAGE, [N]VARCHAR(MAX), or VARBINARY(MAX).
     /// @param log_it 
     ///   Log LOB operation if this value is set to true.
     /// 
@@ -133,7 +134,7 @@ public:
     ///
     /// @sa
     ///   SendDataCmd
-    virtual bool SendData(I_ITDescriptor& desc, CDB_Stream& lob,
+    virtual bool SendData(I_BlobDescriptor& desc, CDB_Stream& lob,
                           bool log_it = true);
 
     /// @brief 
@@ -206,6 +207,10 @@ public:
     ///   - true if this connection is a reusable one.
     virtual bool IsReusable() const;
 
+    /// Indicate how many times (if at all) this connection has been
+    /// previously used.
+    unsigned int GetReuseCount() const;
+    
     /// @brief 
     ///   Find out which connection pool this connection belongs to
     /// 
@@ -295,11 +300,22 @@ public:
     /// Set timeout for command cancellation and connection closing
     virtual void SetCancelTimeout(size_t nof_secs);
 
+    /// Get connection timeout.    
+    size_t GetTimeout(void) const;
+
+    /// Get timeout for command cancellation and connection closing
+    size_t GetCancelTimeout(void) const;
+    
     /// Get interface for extra features that could be implemented in the driver.
     virtual I_ConnectionExtra& GetExtraFeatures(void);
 
     virtual string GetDriverName(void) const;
 
+    bool HasTransaction(void) const
+    {
+        return m_HasTransaction;
+    }
+    
 public:
     // Deprecated legacy methods.
     // CXX-601
@@ -344,6 +360,7 @@ public:
 
 private:
     impl::CConnection* m_ConnImpl;
+    bool               m_HasTransaction;
 
     CDB_Connection(impl::CConnection* c);
 
@@ -359,6 +376,7 @@ private:
 
     // The constructor should be called by "I_DriverContext" only!
     friend class impl::CDriverContext;
+    friend class CAutoTrans;
     friend class CInterfaceHook<CDB_Connection>;
 };
 
@@ -444,16 +462,17 @@ public:
     ///   If "item_buf" is not NULL, then use "*item_buf" (its type should be
     ///   compatible with the type of retrieved item!) to retrieve the item to;
     ///   otherwise allocate new "CDB_Object".
-	///   In case of "CDB_Image" and "CDB_Text" data types value will be *appended*
-	///   to the "item_buf" by default (policy == eAppendLOB).
+    ///   If the existing "item_buf" has a BLOB type (CDB_Image, CDB_Text,
+    ///   CDB_VarBinaryMax, or CDB_VarCharMax), the value will be *appended*
+    ///   to the "item_buf" by default (policy == eAppendLOB).
     /// 
-	/// @param policy
-	///   Data retrieval policy. If policy == eAppendLOB and "item_buf" is an
-	///   object of CDB_Image or CDB_Text type, then data will be *appended* to
-	///   the end of previously assigned data. If policy == eAssignLOB and "item_buf" is an
-	///   object of CDB_Image or CDB_Text type, then new value will be *assigned*
-	///   to the "item_buf" object.
-	///
+    /// @param policy
+    ///   Data retrieval policy. If policy == eAppendLOB *and* "item_buf" has a
+    ///   BLOB type (CDB_Image, CDB_Text, CDB_VarBinaryMax, or CDB_VarCharMax),
+    ///   data will be *appended* to the end of previously assigned data.
+    ///   Otherwise (policy == eAssignLOB or "item_buf" has some other type),
+    ///   the new value will be *assigned* to the "item_buf" object.
+    ///
     /// @return 
     ///   a result item
     ///
@@ -462,7 +481,7 @@ public:
     virtual CDB_Object* GetItem(CDB_Object* item_buf = 0, EGetItem policy = eAppendLOB);
 
     /// @brief 
-    ///   Read a result item body (for text/image mostly).
+    ///   Read a result item body (for BLOB columns, mostly).
     ///   Throw an exception on any error.
     /// 
     /// @param buffer 
@@ -481,15 +500,15 @@ public:
                             bool* is_null = 0);
 
     /// @brief 
-    ///   Get a descriptor for text/image column (for SendData).
+    ///   Get a descriptor for a BLOB column (for SendData).
     /// 
     /// @return 
-    ///   Return NULL if this result doesn't (or can't) have img/text descriptor.
+    ///   Return NULL if this result doesn't (or can't) have a BLOB descriptor.
     ///
     /// @note
     ///   You need to call ReadItem (maybe even with buffer_size == 0)
     ///   before calling this method!
-    virtual I_ITDescriptor* GetImageOrTextDescriptor();
+    virtual I_BlobDescriptor* GetBlobDescriptor();
 
     /// @brief 
     /// Skip result item
@@ -809,8 +828,8 @@ public:
     /// Update the last fetched row.
     /// NOTE: the cursor must be declared for update in CDB_Connection::Cursor()
     virtual bool Update(const string& table_name, const string& upd_query);
-    virtual bool UpdateTextImage(unsigned int item_num, CDB_Stream& data,
-                                 bool log_it = true);
+    virtual bool UpdateBlob(unsigned int item_num, CDB_Stream& data,
+                            bool log_it = true);
     /// Make "send-data" command
     /// @brief 
     ///   Create send-data command.
@@ -911,16 +930,16 @@ private:
 
 
 
-class NCBI_DBAPIDRIVER_EXPORT CDB_ITDescriptor : public I_ITDescriptor
+class NCBI_DBAPIDRIVER_EXPORT CDB_BlobDescriptor : public I_BlobDescriptor
 {
 public:
     enum ETDescriptorType {eUnknown, eText, eBinary};
 
-    CDB_ITDescriptor(const string& table_name,
-                     const string& column_name,
-                     const string& search_conditions,
-                     ETDescriptorType column_type = eUnknown);
-    virtual ~CDB_ITDescriptor(void);
+    CDB_BlobDescriptor(const string& table_name,
+                       const string& column_name,
+                       const string& search_conditions,
+                       ETDescriptorType column_type = eUnknown);
+    virtual ~CDB_BlobDescriptor(void);
 
     virtual int DescriptorType(void) const;
 
@@ -947,6 +966,8 @@ protected:
     ETDescriptorType    m_ColumnType;
 };
 
+// historical name
+typedef CDB_BlobDescriptor CDB_ITDescriptor;
 
 
 class NCBI_DBAPIDRIVER_EXPORT CDB_ResultProcessor
