@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___DATA_SOURCE__HPP
 #define OBJECTS_OBJMGR_IMPL___DATA_SOURCE__HPP
 
-/*  $Id: data_source.hpp 202812 2010-08-30 15:56:57Z vasilche $
+/*  $Id: data_source.hpp 494477 2016-03-07 19:25:35Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -138,7 +138,7 @@ public:
 
     typedef CDSAnnotLockReadGuard                   TAnnotLockReadGuard;
     typedef CDSAnnotLockWriteGuard                  TAnnotLockWriteGuard;
-    typedef CMutex TMainLock;
+    typedef CRWLock TMainLock;
     typedef CMutex TAnnotLock;
     typedef CMutex TCacheLock;
 
@@ -182,6 +182,7 @@ public:
     const CTSE_LockSet& GetStaticBlobs(void) const;
 
     CDataLoader* GetDataLoader(void) const;
+    void RevokeDataLoader(void);
     const CConstRef<CObject>& GetSharedObject(void) const;
     TTSE_Lock GetSharedTSE(void) const;
     bool CanBeEdited(void) const;
@@ -190,13 +191,25 @@ public:
     void UpdateAnnotIndex(const CSeq_entry_Info& entry_info);
     void UpdateAnnotIndex(const CSeq_annot_Info& annot_info);
 
+    // annotations for sequence that is not in this data source
     void GetTSESetWithOrphanAnnots(const TSeq_idSet& ids,
                                    TTSE_LockMatchSet& tse_set,
                                    const SAnnotSelector* sel);
+    // annotations for sequence that is provided by this data source
+    // external_only = true means to exclude TSE with the sequence
     void GetTSESetWithBioseqAnnots(const CBioseq_Info& bioseq,
                                    const TTSE_Lock& tse,
                                    TTSE_LockMatchSet& tse_set,
-                                   const SAnnotSelector* sel);
+                                   const SAnnotSelector* sel,
+                                   bool external_only = false);
+    // external annotations for sequence that is provided by this data source
+    void GetTSESetWithExternalAnnots(const CBioseq_Info& bioseq,
+                                     const TTSE_Lock& tse,
+                                     TTSE_LockMatchSet& tse_set,
+                                     const SAnnotSelector* sel)
+        {
+            GetTSESetWithBioseqAnnots(bioseq, tse, tse_set, sel, true);
+        }
 
     // Fill the set with bioseq handles for all sequences from a given TSE.
     // Return empty tse lock if the entry was not found or is not a TSE.
@@ -217,20 +230,48 @@ public:
 
     typedef vector<CSeq_id_Handle> TIds;
     void GetIds(const CSeq_id_Handle& idh, TIds& ids);
-    CSeq_id_Handle GetAccVer(const CSeq_id_Handle& idh);
-    int GetGi(const CSeq_id_Handle& idh);
+    // null CSeq_id_Handle if sequence doesn't have accession
+    typedef CDataLoader::SAccVerFound SAccVerFound;
+    SAccVerFound GetAccVer(const CSeq_id_Handle& idh);
+    // ZERO_GI if sequence doesn't have GI
+    typedef CDataLoader::SGiFound SGiFound;
+    SGiFound GetGi(const CSeq_id_Handle& idh);
+    // empty string if sequence not found
     string GetLabel(const CSeq_id_Handle& idh);
+    // -1 if sequence not found
+    // 0 if sequence doesn't have TaxID
     int GetTaxId(const CSeq_id_Handle& idh);
+    // kInvalidSeqPos if sequence not found
+    TSeqPos GetSequenceLength(const CSeq_id_Handle& idh);
+    typedef CDataLoader::STypeFound STypeFound;
+    STypeFound GetSequenceType(const CSeq_id_Handle& idh);
+    // CBioseq_Handle::fState_not_found if sequence not found
+    int GetSequenceState(const CSeq_id_Handle& idh);
+    typedef CDataLoader::SHashFound SHashFound;
+    SHashFound GetSequenceHash(const CSeq_id_Handle& idh);
 
     // bulk interface
     typedef vector<bool> TLoaded;
-    typedef vector<int> TGis;
+    typedef vector<TGi> TGis;
     typedef vector<string> TLabels;
     typedef vector<int> TTaxIds;
+    typedef vector<TSeqPos> TSequenceLengths;
+    typedef vector<CSeq_inst::TMol> TSequenceTypes;
+    typedef vector<int> TSequenceStates;
+    typedef vector<int> TSequenceHashes;
+    typedef vector<bool> THashKnown;
     void GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret);
     void GetGis(const TIds& ids, TLoaded& loaded, TGis& ret);
     void GetLabels(const TIds& ids, TLoaded& loaded, TLabels& ret);
     void GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret);
+    void GetSequenceLengths(const TIds& ids, TLoaded& loaded,
+                            TSequenceLengths& ret);
+    void GetSequenceTypes(const TIds& ids, TLoaded& loaded,
+                          TSequenceTypes& ret);
+    void GetSequenceStates(const TIds& ids, TLoaded& loaded,
+                           TSequenceStates& ret);
+    void GetSequenceHashes(const TIds& ids, TLoaded& loaded,
+                           TSequenceHashes& ret, THashKnown& known);
 
     typedef map<CSeq_id_Handle, SSeqMatch_DS>       TSeqMatchMap;
     void GetBlobs(TSeqMatchMap& match_map);
@@ -263,6 +304,8 @@ public:
     typedef pair<CConstRef<CBioseq_set_Info>, TTSE_Lock> TBioseq_set_Lock;
     typedef pair<CConstRef<CBioseq_Info>, TTSE_Lock> TBioseq_Lock;
 
+    TSeq_entry_Lock GetSeq_entry_Lock(const CBlobIdKey& blob_id);
+
     TTSE_Lock FindTSE_Lock(const CSeq_entry& entry,
                            const TTSE_LockSet& history) const;
     TSeq_entry_Lock FindSeq_entry_Lock(const CSeq_entry& entry,
@@ -440,6 +483,7 @@ private:
 
     TBlob_Map             m_Blob_Map;       // TBlobId -> CTSE_Info
     mutable TBlob_Cache   m_Blob_Cache;     // unlocked blobs
+    mutable size_t        m_Blob_Cache_Size;// list<>::size() is slow
 
     // Prefetching thread and lock, used when initializing the thread
     CRef<CPrefetchThreadOld> m_PrefetchThread;
