@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 #define OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 
-/*  $Id: tse_info.hpp 205152 2010-09-15 14:09:19Z vasilche $
+/*  $Id: tse_info.hpp 446824 2014-09-18 14:47:47Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -57,6 +57,7 @@ BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
 class CScope_Impl;
+class CTSE_ScopeInfo;
 class CBioseq_Info;
 class CSeq_entry_Info;
 class CSeq_annot_Info;
@@ -80,6 +81,7 @@ class CTSE_Default_Assigner;
 class IEditSaver;
 
 class CSeq_annot_Finder;
+class CMasterSeqSegments;
 
 ////////////////////////////////////////////////////////////////////
 //
@@ -167,6 +169,13 @@ private:
 
 class CTSEAnnotObjectMapper;
 
+
+class CBioseqUpdater : public CObject {
+public:
+    virtual void Update(CBioseq_Info& seq) = 0;
+};
+
+
 class NCBI_XOBJMGR_EXPORT CTSE_Info : public CSeq_entry_Info
 {
     typedef CSeq_entry_Info TParent;
@@ -255,7 +264,7 @@ public:
     bool OnlyGiAnnotIds(void) const;
 
     // indexes types
-    typedef map<int, CBioseq_set_Info*>                      TBioseq_sets;
+    typedef map<TIntId, CBioseq_set_Info*>                   TBioseq_sets;
     typedef CBioseq_Info*                                    TBioseqInfo;
     typedef map<CSeq_id_Handle, TBioseqInfo>                 TBioseqs;
 
@@ -276,18 +285,18 @@ public:
 
     struct SFeatIdInfo
     {
-        SFeatIdInfo(bool xref, CAnnotObject_Info* info)
-            : m_Xref(xref), m_IsChunk(false)
+        SFeatIdInfo(EFeatIdType type, CAnnotObject_Info* info)
+            : m_Type(type), m_IsChunk(false)
             {
                 m_Info = info;
             }
-        SFeatIdInfo(bool xref, TChunkId chunk_id)
-            : m_Xref(xref), m_IsChunk(true)
+        SFeatIdInfo(EFeatIdType type, TChunkId chunk_id)
+            : m_Type(type), m_IsChunk(true)
             {
                 m_ChunkId = chunk_id;
             }
 
-        bool               m_Xref;
+        Int1               m_Type;
         bool               m_IsChunk;
         union {
             CAnnotObject_Info* m_Info;
@@ -318,22 +327,33 @@ public:
     CConstRef<CBioseq_Info> FindBioseq(const CSeq_id_Handle& id) const;
 
     // find bioseq with matching id
-    bool ContainsMatchingBioseq(const CSeq_id_Handle& id) const;
+    // returns matching Seq-id handle
+    CSeq_id_Handle ContainsMatchingBioseq(const CSeq_id_Handle& id) const;
     CConstRef<CBioseq_Info> FindMatchingBioseq(const CSeq_id_Handle& id) const;
     SSeqMatch_TSE GetSeqMatch(const CSeq_id_Handle& id) const;
 
+    CConstRef<CBioseq_Info> GetSegSetMaster(void) const;
+    CConstRef<CMasterSeqSegments> GetMasterSeqSegments(void) const;
+
     // fill ids with all Bioseqs Seq-ids from this TSE
     // the result will be sorted and contain no duplicates
     virtual void GetBioseqsIds(TSeqIds& ids) const;
     // fill ids with all Annot Seq-ids from this TSE
     // the result will be sorted and contain no duplicates
     virtual void GetAnnotIds(TSeqIds& ids) const;
+
+    // patch loaded CBioseq objects now, and split CBioseq objects when loaded
+    void SetBioseqUpdater(CRef<CBioseqUpdater> updater);
     
     void UpdateAnnotIndex(const CSeq_id_Handle& id) const;
     void UpdateAnnotIndex(void) const;
     void UpdateAnnotIndex(const CTSE_Info_Object& object) const;
     void UpdateAnnotIndex(void);
     void UpdateAnnotIndex(CTSE_Info_Object& object);
+    void UpdateFeatIdIndex(CSeqFeatData::E_Choice type,
+                           EFeatIdType id_type) const;
+    void UpdateFeatIdIndex(CSeqFeatData::ESubtype subtype,
+                           EFeatIdType id_type) const;
 
     void x_UpdateAnnotIndexContents(CTSE_Info& tse);
 
@@ -348,6 +368,8 @@ public:
     void x_LoadChunks(const TChunkIds& chunk_ids) const;
 
     CTSE_Split_Info& GetSplitInfo(void);
+    const CTSE_Split_Info& GetSplitInfo(void) const;
+    bool HasSplitInfo(void) const;
     bool x_NeedsDelayedMainChunk(void) const;
 
     const CSeq_id_Handle& GetRequestedId(void) const;
@@ -369,47 +391,49 @@ public:
     SFeatIdIndex::TIndexInt& x_GetFeatIdIndexInt(CSeqFeatData::ESubtype type);
     SFeatIdIndex::TIndexStr& x_GetFeatIdIndexStr(CSeqFeatData::ESubtype type);
 
+    bool x_HasFeaturesWithId(CSeqFeatData::ESubtype subtype) const;
+
     typedef vector<CAnnotObject_Info*> TAnnotObjects;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            const SFeatIdIndex& index,
                            TFeatIdInt id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            CSeqFeatData::ESubtype subtype,
                            TFeatIdInt id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddAllFeaturesById(TAnnotObjects& objects,
                               TFeatIdInt id,
-                              bool xref) const;
+                              EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::E_Choice type,
                                     TFeatIdInt id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                                     TFeatIdInt id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            const SFeatIdIndex& index,
                            const TFeatIdStr& id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddFeaturesById(TAnnotObjects& objects,
                            CSeqFeatData::ESubtype subtype,
                            const TFeatIdStr& id,
-                           bool xref) const;
+                           EFeatIdType id_type) const;
     void x_AddAllFeaturesById(TAnnotObjects& objects,
                               const TFeatIdStr& id,
-                              bool xref) const;
+                              EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::E_Choice type,
                                     const TFeatIdStr& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                                     const TFeatIdStr& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::E_Choice type,
                                     const TFeatId& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesById(CSeqFeatData::ESubtype subtype,
                                     const TFeatId& id,
-                                    bool xref) const;
+                                    EFeatIdType id_type) const;
     TAnnotObjects x_GetFeaturesByLocus(const string& locus, bool tag) const;
 
     typedef pair<CConstRef<CSeq_annot_Info>, CTSE_Lock> TSeq_annot_Lock;
@@ -418,10 +442,13 @@ public:
                                   TSeqPos loc_pos,
                                   const CSeq_feat& feat) const;
 
+    virtual string GetDescription(void) const;
+
 private:
     friend class CTSE_Guard;
     friend class CDataSource;
     friend class CScope_Impl;
+    friend class CTSE_ScopeInfo;
     friend class CDataLoader;
     friend class CAnnot_Collector;
     friend class CSeq_entry_Info;
@@ -439,6 +466,7 @@ private:
 
 
     void x_Initialize(void);
+    void x_Reset(void); // can be called after incomplete loading
    
     void x_DSMapObject(CConstRef<TObject> obj, CDataSource& ds);
     void x_DSUnmapObject(CConstRef<TObject> obj, CDataSource& ds);
@@ -510,22 +538,22 @@ private:
 
     void x_MapFeatById(TFeatIdInt id,
                        CAnnotObject_Info& info,
-                       bool xref);
+                       EFeatIdType type);
     void x_UnmapFeatById(TFeatIdInt id,
                          CAnnotObject_Info& info,
-                         bool xref);
+                         EFeatIdType type);
     void x_MapFeatById(const TFeatIdStr& id,
                        CAnnotObject_Info& info,
-                       bool xref);
+                       EFeatIdType type);
     void x_UnmapFeatById(const TFeatIdStr& id,
                          CAnnotObject_Info& info,
-                         bool xref);
+                         EFeatIdType type);
     void x_MapFeatById(const TFeatId& id,
                        CAnnotObject_Info& info,
-                       bool xref);
+                       EFeatIdType type);
     void x_UnmapFeatById(const TFeatId& id,
                          CAnnotObject_Info& info,
-                         bool xref);
+                         EFeatIdType type);
     void x_MapFeatByLocus(const string& locus, bool tag,
                           CAnnotObject_Info& info);
     void x_UnmapFeatByLocus(const string& locus, bool tag,
@@ -538,42 +566,42 @@ private:
     void x_MapChunkByFeatType(const SAnnotTypeSelector& type,
                               TChunkId chunk_id);
 
-    void x_MapChunkByFeatId(TFeatIdInt feat_id,
+    void x_MapChunkByFeatId(TFeatIdInt id,
                             CSeqFeatData::ESubtype subtype,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(TFeatIdInt feat_id,
+                            EFeatIdType type);
+    void x_MapChunkByFeatId(TFeatIdInt id,
                             CSeqFeatData::E_Choice type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(TFeatIdInt feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(TFeatIdInt id,
                             const SAnnotTypeSelector& type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatIdStr& id,
                             CSeqFeatData::ESubtype subtype,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatIdStr& id,
                             CSeqFeatData::E_Choice type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatIdStr& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatIdStr& id,
                             const SAnnotTypeSelector& type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatId& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatId& id,
                             CSeqFeatData::ESubtype subtype,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatId& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatId& id,
                             CSeqFeatData::E_Choice type,
                             TChunkId chunk_id,
-                            bool xref);
-    void x_MapChunkByFeatId(const TFeatId& feat_id,
+                            EFeatIdType id_type);
+    void x_MapChunkByFeatId(const TFeatId& id,
                             const SAnnotTypeSelector& type,
                             TChunkId chunk_id,
-                            bool xref);
+                            EFeatIdType id_type);
 
     friend class CTSEAnnotObjectMapper;
 
@@ -689,6 +717,11 @@ private:
     typedef map<CBioObjectId, CTSE_Info_Object*> TBioObjects;
     TBioObjects m_BioObjects;
 
+    mutable bool m_MasterSeqSegmentsLoaded;
+    mutable CConstRef<CMasterSeqSegments> m_MasterSeqSegments;
+
+    CRef<CBioseqUpdater> m_BioseqUpdater;
+
 private:
     // Hide copy methods
     CTSE_Info(const CTSE_Info&);
