@@ -1,7 +1,7 @@
 #ifndef SEQ_FEAT_HANDLE__HPP
 #define SEQ_FEAT_HANDLE__HPP
 
-/*  $Id: seq_feat_handle.hpp 197370 2010-07-15 19:58:28Z vasilche $
+/*  $Id: seq_feat_handle.hpp 483571 2015-11-02 17:33:20Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -164,6 +164,8 @@ public:
     typedef SSNP_Info::TWeight TWeight;
 
     TSNPId GetSNPId(void) const;
+    const CSeq_id& GetSNPSeq_id(void) const;
+    NCBI_DEPRECATED
     CSeq_id::TGi GetSNPGi(void) const;
     bool IsSNPMinusStrand(void) const;
     TWeight GetSNPWeight(void) const;
@@ -182,12 +184,14 @@ public:
 
     /// Return true if this feature was removed already
     bool IsRemoved(void) const;
+
     /// Remove the feature from Seq-annot
-    /// The method is deprecated, use CSeq_feat_EditHandle
+    /// @deprecated  Use CSeq_feat_EditHandle
     NCBI_DEPRECATED void Remove(void) const;
+
     /// Replace the feature with new Seq-feat object.
     /// All indexes are updated correspondingly.
-    /// The method is deprecated, use CSeq_feat_EditHandle
+    /// @deprecated Use CSeq_feat_EditHandle
     NCBI_DEPRECATED void Replace(const CSeq_feat& new_feat) const;
 
 
@@ -201,16 +205,18 @@ public:
     /// Return a specified DB xref.  This will find the *first* item in the
     /// given referenced database.  If no item is found, an empty CConstRef<>
     /// is returned.
-    CConstRef<CDbtag> GetNamedDbxref(const string& db) const;
+    CConstRef<CDbtag> GetNamedDbxref(const CTempString& db) const;
 
     /// Return a named qualifier.  This will return the first item matching the
     /// qualifier name.  If no such qualifier is found, an empty string is
     /// returned.
-    const string& GetNamedQual(const string& qual_name) const;
+    const string& GetNamedQual(const CTempString& qual_name) const;
 
 protected:
     friend class CMappedFeat;
+    friend class CFeat_CI;
     friend class CCreatedFeat_Ref;
+    friend class CSeq_annot_Info;
     friend class CSeq_annot_Handle;
     friend class CSeq_annot_ftable_CI;
     friend class CSeq_annot_ftable_I;
@@ -245,6 +251,7 @@ protected:
 private:
     CSeq_annot_Handle              m_Seq_annot;
     TFeatIndex                     m_FeatIndex;
+    mutable CConstRef<CSeq_feat>   m_CreatedOriginalFeat;
     mutable CRef<CCreatedFeat_Ref> m_CreatedFeat;
 };
 
@@ -632,10 +639,46 @@ public:
     /// Add a qualifier to this feature
     void AddQualifier(const string& qual_name, const string& qual_val);
 
+    /// Remove all qualifiers with the given name on this feature.
+    void RemoveQualifier(const string& qual_name);
+
     /// add a DB xref to this feature
     void AddDbxref(const string& db_name, const string& db_key);
     void AddDbxref(const string& db_name, int db_key);
 
+    /// Add the given exception_text and set the except flag to true.
+    /// If the exception_text is already there, it just sets the except flag.
+    void AddExceptText(const string & exception_text);
+
+    /// Remove all instances of the given exception text in this feature,
+    /// and reset the except flag if there are no exception texts left.
+    void RemoveExceptText(const string & exception_text);
+
+    /// Add feature id
+    void AddFeatId(int id);
+    void AddFeatId(const string& id);
+    void AddFeatId(const CObject_id& id);
+    void AddFeatXref(int id);
+    void AddFeatXref(const string& id);
+    void AddFeatXref(const CObject_id& id);
+    
+    /// Remove feature id
+    void RemoveFeatId(int id);
+    void RemoveFeatId(const string& id);
+    void RemoveFeatId(const CObject_id& id);
+    void RemoveFeatXref(int id);
+    void RemoveFeatXref(const string& id);
+    void RemoveFeatXref(const CObject_id& id);
+    
+    /// Clear feature ids
+    void ClearFeatIds(void);
+    void ClearFeatXrefs(void);
+    
+    /// Set single feature id
+    void SetFeatId(int id);
+    void SetFeatId(const string& id);
+    void SetFeatId(const CObject_id& id);
+
 protected:
     friend class CSeq_annot_EditHandle;
 
