@@ -1,7 +1,7 @@
 #ifndef OBJOSTR__HPP
 #define OBJOSTR__HPP
 
-/*  $Id: objostr.hpp 207464 2010-10-06 13:23:15Z gouriano $
+/*  $Id: objostr.hpp 484578 2015-11-12 19:13:20Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -118,11 +118,15 @@ public:
     /// @param deleteOutStream
     ///   When eTakeOwnership, the output stream will be deleted automatically
     ///   when the writer is deleted
+    /// @param formatFlags
+    ///   Formatting flags (see ESerial_xxx_Flags)
     /// @return
     ///   Writer (created on heap)
+    /// @sa ESerial_AsnText_Flags, ESerial_Xml_Flags, ESerial_Json_Flags
     static CObjectOStream* Open(ESerialDataFormat format,
                                 CNcbiOstream& outStream,
-                                EOwnership deleteOutStream = eNoOwnership);
+                                EOwnership deleteOutStream = eNoOwnership,
+                                TSerial_Format_Flags formatFlags = 0);
 
     /// Create serial object writer and attach it to a file stream.
     ///
@@ -132,12 +136,15 @@ public:
     ///   Output file name
     /// @param openFlags
     ///   File open flags
+    /// @param formatFlags
+    ///   Formatting flags (see ESerial_xxx_Flags)
     /// @return
     ///   Writer (created on heap)
-    /// @sa ESerialOpenFlags
+    /// @sa ESerialOpenFlags, ESerial_AsnText_Flags, ESerial_Xml_Flags, ESerial_Json_Flags
     static CObjectOStream* Open(ESerialDataFormat format,
                                 const string& fileName,
-                                TSerialOpenFlags openFlags = 0);
+                                TSerialOpenFlags openFlags = 0,
+                                TSerial_Format_Flags formatFlags = 0);
 
     /// Create serial object writer and attach it to a file stream.
     ///
@@ -145,10 +152,14 @@ public:
     ///   Output file name
     /// @param format
     ///   Format of the output data
+    /// @param formatFlags
+    ///   Formatting flags (see ESerial_xxx_Flags)
     /// @return
     ///   Writer (created on heap)
+    /// @sa ESerial_AsnText_Flags, ESerial_Xml_Flags, ESerial_Json_Flags
     static CObjectOStream* Open(const string& fileName,
-                                ESerialDataFormat format);
+                                ESerialDataFormat format,
+                                TSerial_Format_Flags formatFlags = 0);
 
     /// Get data format
     ///
@@ -189,11 +200,24 @@ public:
     ///   Data verification parameter
     static  void SetVerifyDataGlobal(ESerialVerifyData verify);
 
-    virtual EFixNonPrint FixNonPrint(EFixNonPrint how)
+    EFixNonPrint FixNonPrint(EFixNonPrint how)
     {
-        return how;
+        EFixNonPrint tmp = m_FixMethod;
+        m_FixMethod = how == eFNP_Default ? x_GetFixCharsMethodDefault() : how;
+        return tmp;
     }
 
+    // Enforce explicit writing of values with default,
+    // even when they were never set
+    //
+    // This is dangerous, and contradicts specification
+    // but might be useful to make sure defaults are properly reported to recipient
+    // Use at your own risk
+    void EnforceWritingDefaultValues(bool enforce = true);
+
+    // Is explicit writing of values with default enforced
+    bool IsWritingDefaultValuesEnforced() const;
+
 //---------------------------------------------------------------------------
 // Formatting of the output
 
@@ -209,6 +233,19 @@ public:
     /// @return
     ///   TRUE or FALSE
     bool GetUseIndentation(void) const;
+
+    /// Set up end-of-line symbol usage in text streams.
+    /// 
+    /// @param set
+    ///   When TRUE, the writer puts end-of-line symbol where needed,
+    ///   otherwise, the output is a single line.
+    void SetUseEol(bool set);
+
+    /// Get end-of-line symbol usage in text streams.
+    /// 
+    /// @return
+    ///   TRUE or FALSE
+    bool GetUseEol(void) const;
     
     /// Set up writing named integers (in ANS.1 sense) by value only.
     ///
@@ -259,6 +296,12 @@ public:
     ///   TRUE or FALSE
     void SetAutoSeparator(bool value);
 
+    /// Set output formatting flags
+    ///
+    /// @param flags
+    ///   Formatting flag
+    virtual void SetFormattingFlags(TSerial_Format_Flags flags);
+
 //---------------------------------------------------------------------------
 // Stream state
 
@@ -326,6 +369,11 @@ public:
     ///   TRUE is there is no errors
     bool InGoodState(void);
 
+    /// Set cancellation check callback.
+    /// The stream will periodically check for a cancellation request and
+    /// throw an exception when requested.
+    void SetCanceledCallback(const ICanceled* callback);
+
     /// @deprecated
     ///   Use GetStreamPos() instead
     /// @sa GetStreamPos()
@@ -428,7 +476,7 @@ public:
     void WriteStd(const unsigned short& data);
     void WriteStd(const int& data);
     void WriteStd(const unsigned int& data);
-#if SIZEOF_LONG == 4
+#ifndef NCBI_INT8_IS_LONG
     void WriteStd(const long& data);
     void WriteStd(const unsigned long& data);
 #endif
@@ -444,13 +492,13 @@ public:
 
     // string
     void WriteStd(const string& data);
-    void WriteStd(const CStringUTF8& data);
 
     // C string; VisualAge can't cope with refs here.
     void WriteStd(const char* const data);
     void WriteStd(char* const data);
 
     void WriteStd(const CBitString& data);
+
     // primitive writers
     // bool
     virtual void WriteBool(bool data) = 0;
@@ -474,7 +522,8 @@ public:
     // string
     virtual void WriteString(const string& str,
                              EStringType type = eStringTypeVisible) = 0;
-    virtual void CopyString(CObjectIStream& in) = 0;
+    virtual void CopyString(CObjectIStream& in,
+                            EStringType type = eStringTypeVisible) = 0;
 
     // StringStore
     virtual void WriteStringStore(const string& data) = 0;
@@ -501,6 +550,14 @@ public:
 
     // delayed buffer
     virtual bool Write(CByteSource& source);
+    void Write(const char* data, size_t size);
+
+#ifdef NCBI_STRICT_GI
+    void WriteStd(const TGi& data);
+
+    virtual void WriteGi(const TGi& obj);
+    virtual void CopyGi(CObjectIStream& in);
+#endif
 
 //---------------------------------------------------------------------------
 // Internals
@@ -629,9 +686,6 @@ public:
                                   TConstObjectPtr memberPtr);
     MLIOVIR bool WriteClassMember(const CMemberId& memberId,
                                   const CDelayBuffer& buffer);
-    // choice
-    MLIOVIR void WriteChoice(const CChoiceTypeInfo* choiceType,
-                             TConstObjectPtr choicePtr);
     // alias
     MLIOVIR void WriteAlias(const CAliasTypeInfo* aliasType,
                             TConstObjectPtr aliasPtr);
@@ -697,9 +751,21 @@ public:
 
     void WritePointer(TConstObjectPtr object, TTypeInfo typeInfo);
 
+    enum ESpecialCaseWrite {
+        eWriteAsNormal  = 0,
+        eWriteAsDefault = 1,
+        eWriteAsNil     = 2
+    };
+    void  SetSpecialCaseWrite( ESpecialCaseWrite how) {
+        m_SpecialCaseWrite = how;
+    }
+    MLIOVIR void WriteClassMemberSpecialCase(
+        const CMemberId& memberId, TTypeInfo memberType,
+        TConstObjectPtr memberPtr, ESpecialCaseWrite how);
+
 protected:
     CObjectOStream(ESerialDataFormat format,
-                   CNcbiOstream& out, bool deleteOut = false);
+                   CNcbiOstream& out, EOwnership deleteOut = eNoOwnership);
 
     // low level writers
     typedef size_t TObjectIndex;
@@ -715,6 +781,10 @@ protected:
     void RegisterObject(TConstObjectPtr object, TTypeInfo typeInfo);
 
     void x_SetPathHooks(bool set);
+    EFixNonPrint x_GetFixCharsMethodDefault(void) const;
+    EFixNonPrint x_FixCharsMethod(void) const {
+        return m_FixMethod;
+    }
     // Write current separator to the stream
     virtual void WriteSeparator(void);
 
@@ -727,20 +797,23 @@ protected:
     ESerialDataFormat   m_DataFormat;
     bool  m_WriteNamedIntegersByValue;
     EDelayBufferParsing  m_ParseDelayBuffers;
+    bool  m_FastWriteDouble;
+    ESpecialCaseWrite m_SpecialCaseWrite;
+    bool  m_EnforceWritingDefaults;
 
 private:
     static CObjectOStream* OpenObjectOStreamAsn(CNcbiOstream& out,
-                                                bool deleteOut);
+                                                EOwnership deleteOut);
     static CObjectOStream* OpenObjectOStreamAsnBinary(CNcbiOstream& out,
-                                                      bool deleteOut);
+                                                      EOwnership deleteOut);
     static CObjectOStream* OpenObjectOStreamXml(CNcbiOstream& out,
-                                                bool deleteOut);
+                                                EOwnership deleteOut);
     static CObjectOStream* OpenObjectOStreamJson(CNcbiOstream& out,
-                                                bool deleteOut);
+                                                EOwnership deleteOut);
     static ESerialVerifyData x_GetVerifyDataDefault(void);
 
+    EFixNonPrint m_FixMethod; // method of fixing wrong (eg, non-printable) chars
     ESerialVerifyData   m_VerifyData;
-    static ESerialVerifyData ms_VerifyDataDefault;
     CStreamObjectPathHook<CWriteObjectHook*>                m_PathWriteObjectHooks;
     CStreamPathHook<CMemberInfo*, CWriteClassMemberHook*>   m_PathWriteMemberHooks;
     CStreamPathHook<CVariantInfo*,CWriteChoiceVariantHook*> m_PathWriteVariantHooks;
@@ -754,6 +827,15 @@ public:
     friend class CObjectStreamCopier;
 };
 
+inline void
+CObjectOStream::EnforceWritingDefaultValues(bool enforce) {
+    m_EnforceWritingDefaults = enforce;
+}
+inline bool
+CObjectOStream::IsWritingDefaultValuesEnforced() const {
+    return m_EnforceWritingDefaults;
+}
+
 
 /* @} */
 
