@@ -1,4 +1,4 @@
-/*  $Id: remote_blast.cpp 416632 2013-10-29 22:14:22Z rafanovi $ 
+/*  $Id: remote_blast.cpp 495288 2016-03-16 14:51:11Z ivanov $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -892,6 +892,14 @@ void CRemoteBlast::x_SetAlgoOpts(void)
     CBlast4_parameters * algo_opts =
         m_CBOH->SetOptions().GetBlast4AlgoOpts();
     
+    string user_ipv6_address;
+    CNcbiEnvironment env;
+    if( env.Get("HTTP_X_FORWARDED_FOR_IPV6") != kEmptyStr )
+	user_ipv6_address = env.Get("HTTP_X_FORWARDED_FOR_IPV6");
+    if( !user_ipv6_address.empty() ){
+	algo_opts->Add( string("HTTP_X_FORWARDED_FOR_IPV6"), user_ipv6_address );
+    }
+
     m_QSR->SetAlgorithm_options().Set() = *algo_opts;
 }
 
@@ -939,6 +947,21 @@ void CRemoteBlast::x_SetOneParam(objects::CBlast4Field & field,
     m_QSR->SetProgram_options().Set().push_back(p);
 }
 
+void CRemoteBlast::x_SetOneParam(objects::CBlast4Field & field,
+                                 const list<Int8> * x)
+{
+    CRef<CBlast4_value> v(new CBlast4_value);
+    v->SetBig_integer_list() = *x;
+
+    CRef<CBlast4_parameter> p(new CBlast4_parameter);
+    p->SetName(field.GetName());
+    p->SetValue(*v);
+    _ASSERT(field.Match(*p));
+
+    m_QSR->SetProgram_options().Set().push_back(p);
+}
+
+
 void CRemoteBlast::x_SetOneParam(objects::CBlast4Field & field,
                                  const char ** x)
 {
@@ -1384,7 +1407,7 @@ void CRemoteBlast::x_Init(CRef<CBlastOptionsHandle>   opts_handle,
     {{
         const CSearchDatabase::TGiList& tmplist = db.GetGiListLimitation();
         if ( !tmplist.empty() ) {
-            list<Int4> gilist;
+            list<TGi> gilist;
             copy(tmplist.begin(), tmplist.end(), back_inserter(gilist));
             SetGIList(gilist);
         }
@@ -1395,7 +1418,7 @@ void CRemoteBlast::x_Init(CRef<CBlastOptionsHandle>   opts_handle,
         const CSearchDatabase::TGiList& tmplist = 
             db.GetNegativeGiListLimitation();
         if ( !tmplist.empty() ) {
-            list<Int4> gilist;
+            list<TGi> gilist;
             copy(tmplist.begin(), tmplist.end(), back_inserter(gilist));
             SetNegativeGIList(gilist);
         }
@@ -1427,7 +1450,7 @@ CRemoteBlast::~CRemoteBlast()
 {
 }
 
-void CRemoteBlast::SetGIList(const list<Int4> & gi_list)
+void CRemoteBlast::SetGIList(const list<TGi> & gi_list)
 {
     if (gi_list.empty()) {
         return;
@@ -1435,8 +1458,16 @@ void CRemoteBlast::SetGIList(const list<Int4> & gi_list)
         NCBI_THROW(CBlastException, eNotSupported, 
            "Submitting gi lists remotely is currently not supported");
     }
-    x_SetOneParam(CBlast4Field::Get(eBlastOpt_GiList), & gi_list);
     
+	#ifdef NCBI_STRICT_GI
+        	list<int> int_gi_list;
+        	ITERATE ( list<TGi>, it, gi_list ) {
+        		int_gi_list.push_back(GI_TO( int, *it));
+        	}
+        x_SetOneParam(CBlast4Field::Get(eBlastOpt_GiList), & int_gi_list);
+	#else
+        x_SetOneParam(CBlast4Field::Get(eBlastOpt_GiList), & gi_list);
+	#endif
     m_GiList.clear();
     copy(gi_list.begin(), gi_list.end(), back_inserter(m_GiList));
 }
@@ -1472,7 +1503,7 @@ CRemoteBlast::GetSubjectMaskingType() const
     return m_SubjectMaskingType;
 }
 
-void CRemoteBlast::SetNegativeGIList(const list<Int4> & gi_list)
+void CRemoteBlast::SetNegativeGIList(const list<TGi> & gi_list)
 {
     if (gi_list.empty()) {
         return;
@@ -1480,7 +1511,15 @@ void CRemoteBlast::SetNegativeGIList(const list<Int4> & gi_list)
         NCBI_THROW(CBlastException, eNotSupported, 
            "Submitting negative gi lists remotely is currently not supported");
     }
-    x_SetOneParam(CBlast4Field::Get(eBlastOpt_NegativeGiList), & gi_list);
+	#ifdef NCBI_STRICT_GI
+       	list<int> int_gi_list;
+       	ITERATE ( list<TGi>, it, gi_list ) {
+       		int_gi_list.push_back(GI_TO( int, *it));
+       	}
+        x_SetOneParam(CBlast4Field::Get(eBlastOpt_GiList), & int_gi_list);
+	#else
+        x_SetOneParam(CBlast4Field::Get(eBlastOpt_NegativeGiList), & gi_list);
+	#endif
     
     m_NegativeGiList.clear();
     copy(gi_list.begin(), gi_list.end(), back_inserter(m_NegativeGiList));
@@ -1616,8 +1655,45 @@ CRemoteBlast::LoadFromArchive()
 
       m_Archive.Reset(new CBlast4_archive);
       *m_ObjectStream >> *m_Archive;
-      x_GetRequestInfoFromFile(); // update info.
 
+      m_Errs.empty();
+      if(m_Archive->IsSetMessages()) {
+    	  const list< CRef< CBlast4_error > >  & msgs = m_Archive->GetMessages();
+    	  ITERATE(list< CRef< CBlast4_error > >, itr, msgs) {
+    		  if((*itr)->IsSetMessage()) {
+    			  string msg = (*itr)->GetMessage();
+    			  if((*itr)->IsSetCode()) {
+    				  switch ((*itr)->GetCode()) {
+    			  	  case EDiagSev::eDiag_Info :
+    			  		  LOG_POST(Info << msg);
+    			  		  break;
+    			  	  case EDiagSev::eDiag_Warning :
+    			  		  LOG_POST(Warning << msg);
+    			  		  break;
+    			  	  case EDiagSev::eDiag_Error :
+    			  		  LOG_POST(Error << msg);
+    			  		  break;
+    			  	  case EDiagSev::eDiag_Critical :
+    			  		  LOG_POST(Critical << msg);
+    			  		  break;
+    				  case EDiagSev::eDiag_Fatal :
+    			  		  LOG_POST(Fatal << msg);
+    					  break;
+    				  case EDiagSev::eDiag_Trace :
+    			  		  LOG_POST(Trace << msg);
+    					  break;
+    				  default :
+    			  		  LOG_POST(Error <<"Unknown Error Code: " << msg);
+    					  break;
+    				  }
+    			  }
+    		  }
+    	  }
+      }
+
+      if (!IsErrMsgArchive()) {
+    	  x_GetRequestInfoFromFile(); // update info.
+      }
       return true;
 }
 
@@ -2588,6 +2664,17 @@ unsigned int CRemoteBlast::x_GetPsiIterationsFromServer()
      return retval;
 }
 
+bool CRemoteBlast::IsErrMsgArchive(void)
+{
+	if((m_Archive) != NULL && m_Archive->IsSetRequest() && m_Archive->GetRequest().CanGetBody() &&
+	   m_Archive->GetRequest().GetBody().IsGet_request_info() &&
+	   m_Archive->GetRequest().GetBody().GetGet_request_info().IsSetRequest_id() &&
+	   (m_Archive->GetRequest().GetBody().GetGet_request_info().GetRequest_id()  == "Error")) {
+		return true;
+	}
+	return false;
+}
+
 
 END_SCOPE(blast)
 END_NCBI_SCOPE
