@@ -1,4 +1,4 @@
-/*  $Id: deltablast_app.cpp 439436 2014-06-30 13:51:29Z fongah2 $
+/*  $Id: deltablast_app.cpp 495295 2016-03-16 14:53:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-        "$Id: deltablast_app.cpp 439436 2014-06-30 13:51:29Z fongah2 $";
+        "$Id: deltablast_app.cpp 495295 2016-03-16 14:53:41Z ivanov $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -103,6 +103,8 @@ private:
     CRef<CDeltaBlastAppArgs> m_CmdLineArgs;
 
     CRef<CBlastAncillaryData> m_AncillaryData;
+
+    CBlastAppDiagHandler m_bah;
 };
 
 void CDeltaBlastApp::Init()
@@ -195,6 +197,8 @@ int CDeltaBlastApp::Run(void)
 
         // Allow the fasta reader to complain on invalid sequence input
         SetDiagPostLevel(eDiag_Warning);
+	    SetDiagPostPrefix("deltablast");
+        SetDiagHandler(&m_bah, false);
 
         /*** Get the BLAST options ***/
         const CArgs& args = GetArgs();
@@ -220,9 +224,10 @@ int CDeltaBlastApp::Run(void)
                                      query_opts->UseLowercaseMasks(),
                                      query_opts->GetParseDeflines(),
                                      query_opts->GetRange());
-        iconfig.SetQueryLocalIdMode();
-        if(IsIStreamEmpty(m_CmdLineArgs->GetInputStream()))
-           	NCBI_THROW(CArgException, eNoValue, "Query is Empty!");
+        if(IsIStreamEmpty(m_CmdLineArgs->GetInputStream())){
+           	ERR_POST(Warning << "Query is Empty!");
+           	return BLAST_EXIT_SUCCESS;
+        }
         CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
         size_t query_batch_size = m_CmdLineArgs->GetQueryBatchSize();
         if (m_CmdLineArgs->GetNumberOfPsiBlastIterations() > 1
@@ -268,14 +273,14 @@ int CDeltaBlastApp::Run(void)
                                db_adapter->GetFilteringAlgorithm(),
                                fmt_args->GetCustomOutputFormatSpec(),
                                false, false, NULL,
-                               domain_db_ptr);
+                               domain_db_ptr,
+                               GetCmdlineArgs(GetArguments()));
         
         formatter.SetQueryRange(query_opts->GetRange());
         formatter.SetLineLength(fmt_args->GetLineLength());
-        if((fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eXml2 ||
-           fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eJson)
-           && args[kArgOutput].AsString() != "-")
+        if(UseXInclude(*fmt_args, args[kArgOutput].AsString())) {
         	formatter.SetBaseFile(args[kArgOutput].AsString());
+        }
         formatter.PrintProlog();
 
         /*** Process the input ***/
@@ -349,7 +354,8 @@ int CDeltaBlastApp::Run(void)
                 }
 
                 if (fmt_args->ArchiveFormatRequested(args)) {
-                    formatter.WriteArchive(*queries, *opts_hndl, *results);
+                    formatter.WriteArchive(*queries, *opts_hndl, *results, 0, m_bah.GetMessages());
+                    m_bah.ResetMessages();
                 } else {
                     BlastFormatter_PreFetchSequenceData(*results, scope);
                     if (m_CmdLineArgs->GetShowDomainHits()) {
@@ -368,12 +374,30 @@ int CDeltaBlastApp::Run(void)
                         formatter.PrintOneResultSet(**result, query_batch);
                     }
                 }
+
+                if (m_CmdLineArgs->GetSaveLastPssm()) {
+                    CConstRef<CBioseq> query_bioseq(&pssm->GetQuery().GetSeq());
+                    blast::CSearchResults& res = (*results)[0];
+
+                    CRef<CPSIBlastOptionsHandle> psi_opts(
+                        dynamic_cast<CPSIBlastOptionsHandle*>(&*opts_hndl));
+
+                    pssm = ComputePssmForNextPsiBlastIteration(*query_bioseq,
+                                                     res.GetSeqAlign(),
+                                                     psi_opts,
+                                                     scope,
+                                                     res.GetAncillaryData());
+
+                    SavePssmToFile(pssm);
+                }
             }
             else {
 
                 // if more than 1 iterations are requested, then
                 // do PSI-BLAST iterations, this is not allowed for remote blast
 
+                SavePssmToFile(pssm);
+
                 // print domain search results if requested
                 // query_batch_size == 1 if number of iteratins > 1
                 if (m_CmdLineArgs->GetShowDomainHits()) {
@@ -420,6 +444,10 @@ int CDeltaBlastApp::Run(void)
         }
 
     } CATCH_ALL(status)
+    if(!m_bah.GetMessages().empty()) {
+     	const CArgs & a = GetArgs();
+     	PrintErrorArchive(a, m_bah.GetMessages());
+    }
     return status;
 }
 
@@ -453,7 +481,8 @@ CDeltaBlastApp::DoPsiBlastIterations(CRef<CBlastOptionsHandle> opts_hndl,
     if (CFormattingArgs::eArchiveFormat ==
         m_CmdLineArgs->GetFormattingArgs()->GetFormattedOutputChoice()) {
         formatter.WriteArchive(*query_factory, *opts_hndl, *results,
-                               itr.GetIterationNumber());
+                               itr.GetIterationNumber(), m_bah.GetMessages());
+        m_bah.ResetMessages();
     }
     else {
         ITERATE(blast::CSearchResultSet, result, *results) {
@@ -481,7 +510,7 @@ CDeltaBlastApp::DoPsiBlastIterations(CRef<CBlastOptionsHandle> opts_hndl,
             pssm = ComputePssmForNextPsiBlastIteration(*query_bioseq, aln,
                                        psi_opts,
                                        scope,
-                                       results_1st_query.GetAncillaryData());
+                                       (*results)[0].GetAncillaryData());
 
         if (psiblast.Empty()) {
             psiblast.Reset(new CPsiBlast(pssm, db_adapter, psi_opts));
@@ -515,16 +544,26 @@ CDeltaBlastApp::DoPsiBlastIterations(CRef<CBlastOptionsHandle> opts_hndl,
             break;
         }
 
-        CConstRef<CSeq_align_set> aln(results_1st_query.GetSeqAlign());
-        CPsiBlastIterationState::TSeqIds ids;
-        CPsiBlastIterationState::GetSeqIds(aln, psi_opts, ids);
+        aln.Reset(results_1st_query.GetSeqAlign());
+        CPsiBlastIterationState::TSeqIds new_ids;
+        CPsiBlastIterationState::GetSeqIds(aln, psi_opts, new_ids);
 
-        itr.Advance(ids);
+        itr.Advance(new_ids);
     }
     if (itr.HasConverged()) {
         converged = true;
     }
 
+    if (m_CmdLineArgs->GetSaveLastPssm()) {
+        CRef<CPssmWithParameters>
+            pssm = ComputePssmForNextPsiBlastIteration(*query_bioseq, aln,
+                                       psi_opts,
+                                       scope,
+                                       results_1st_query.GetAncillaryData());
+
+        SavePssmToFile(pssm);
+    }
+
     return converged;
 }
 
@@ -547,7 +586,7 @@ CDeltaBlastApp::ComputePssmForNextPsiBlastIteration(const CBioseq& bioseq,
 #ifndef SKIP_DOXYGEN_PROCESSING
 int main(int argc, const char* argv[] /*, const char* envp[]*/)
 {
-    return CDeltaBlastApp().AppMain(argc, argv, 0, eDS_Default, 0);
+    return CDeltaBlastApp().AppMain(argc, argv, 0, eDS_Default, "");
 }
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
