@@ -1,4 +1,4 @@
-/*  $Id: user_agent.cpp 440555 2014-07-15 12:18:26Z ivanov $
+/*  $Id: user_agent.cpp 485509 2015-11-23 14:51:37Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -107,8 +107,11 @@ bool CCgiUserAgent::IsBrowser(void) const
 {
     switch ( GetEngine() ) {
         case eEngine_IE:
+        case eEngine_Edge:
         case eEngine_Gecko:
         case eEngine_KHTML:
+        case eEngine_WebKit:
+        case eEngine_Blink:
             return true;
         case eEngine_Bot:
             return false;
@@ -320,26 +323,32 @@ bool CCgiUserAgent::x_CheckPattern(int /*EPattern*/ what,
     // External patterns
     if ( !external_patterns.empty() ) {
         if (external_patterns.find_first_of(kMultiLinePatternDelimiters) != NPOS) {
-            NStr::Split(external_patterns, kMultiLinePatternDelimiters, patterns);
+            NStr::Split(external_patterns, kMultiLinePatternDelimiters, patterns,
+                        NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
         } else {
-            NStr::Split(external_patterns, kSingleLinePatternDelimiters, patterns);
+            NStr::Split(external_patterns, kSingleLinePatternDelimiters, patterns,
+                        NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
         }
     }
     // User-defined patterns
     if (current_status) {
         if ( !exclude_patterns.empty() ) {
             if (external_patterns.find_first_of(kMultiLinePatternDelimiters) != NPOS) {
-                NStr::Split(USTR(exclude_patterns), kMultiLinePatternDelimiters, patterns);
+                NStr::Split(USTR(exclude_patterns), kMultiLinePatternDelimiters, patterns,
+                            NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
             } else {
-                NStr::Split(USTR(exclude_patterns), kSingleLinePatternDelimiters, patterns);
+                NStr::Split(USTR(exclude_patterns), kSingleLinePatternDelimiters, patterns,
+                            NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
             }
         }
     } else {
         if ( !include_patterns.empty() ) {
             if (external_patterns.find_first_of(kMultiLinePatternDelimiters) != NPOS) {
-                NStr::Split(USTR(include_patterns), kMultiLinePatternDelimiters, patterns);
+                NStr::Split(USTR(include_patterns), kMultiLinePatternDelimiters, patterns,
+                            NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
             } else {
-                NStr::Split(USTR(include_patterns), kSingleLinePatternDelimiters, patterns);
+                NStr::Split(USTR(include_patterns), kSingleLinePatternDelimiters, patterns,
+                            NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
             }
         }
     }
@@ -725,6 +734,9 @@ const SBrowser s_Browsers[] = {
     { CCgiUserAgent::eVodafone,     "Vodafone Live!",           "Vodafone",                 CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_MobileDevice, fApp },
     { CCgiUserAgent::eXiino,        "Xiino",                    "Xiino",                    CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Palm,         fApp },
 
+    // ???
+
+    { CCgiUserAgent::eEdge,         "Microsoft Edge",           "Edge",                     CCgiUserAgent::eEngine_Edge,    CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
 
     // Gecko-based                                              
 
@@ -767,24 +779,24 @@ const SBrowser s_Browsers[] = {
     { CCgiUserAgent::eIE,           "Internet Explorer",        "Internet Explorer",        CCgiUserAgent::eEngine_IE,      CCgiUserAgent::ePlatform_Unknown,      fProduct },
     { CCgiUserAgent::eIE,           "Internet Explorer",        "MSIE",                     CCgiUserAgent::eEngine_IE,      CCgiUserAgent::ePlatform_Unknown,      fApp },
 
-    // AppleQWebKit/KHTML-based                                 
+    // AppleWebKit/KHTML-based (engine detected automatically)
 
-    { CCgiUserAgent::eChrome,       "Google Chrome",            "Chrome",                   CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
-    { CCgiUserAgent::eFluid,        "Fluid",                    "Fluid",                    CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
-    { CCgiUserAgent::eSafariMobile, "Mobile Safari",            "Mobile Safari",            CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_MobileDevice, fVendorProduct },
-    { CCgiUserAgent::eMidori,       "Midori",                   "Midori",                   CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fAny },
-    { CCgiUserAgent::eMidori,       "Midori",                   "midori",                   CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fAppComment },
-    { CCgiUserAgent::eNetNewsWire,  "NetNewsWire",              "NetNewsWire",              CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fAny },
-    { CCgiUserAgent::eOmniWeb,      "OmniWeb",                  "OmniWeb",                  CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
-    { CCgiUserAgent::eQtWeb,        "QtWeb",                    "QtWeb Internet Browser",   CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
-    { CCgiUserAgent::eSafari,       "Safari",                   "Safari",                   CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
-    { CCgiUserAgent::eShiira,       "Shiira",                   "Shiira",                   CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
-    { CCgiUserAgent::eStainless,    "Stainless",                "Stainless",                CCgiUserAgent::eEngine_KHTML,   CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eChrome,       "Google Chrome",            "Chrome",                   CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eFluid,        "Fluid",                    "Fluid",                    CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eSafariMobile, "Mobile Safari",            "Mobile Safari",            CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_MobileDevice, fVendorProduct },
+    { CCgiUserAgent::eMidori,       "Midori",                   "Midori",                   CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAny },
+    { CCgiUserAgent::eMidori,       "Midori",                   "midori",                   CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAppComment },
+    { CCgiUserAgent::eNetNewsWire,  "NetNewsWire",              "NetNewsWire",              CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAny },
+    { CCgiUserAgent::eOmniWeb,      "OmniWeb",                  "OmniWeb",                  CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eQtWeb,        "QtWeb",                    "QtWeb Internet Browser",   CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eSafari,       "Safari",                   "Safari",                   CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eShiira,       "Shiira",                   "Shiira",                   CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
+    { CCgiUserAgent::eStainless,    "Stainless",                "Stainless",                CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fVendorProduct },
 
     // Other                                                    
 
     { CCgiUserAgent::eiCab,         "iCab",                     "iCab",                     CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fApp },
-    { CCgiUserAgent::eKonqueror,    "Konqueror",                "Konqueror",                CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fApp },
+    { CCgiUserAgent::eKonqueror,    "Konqueror",                "Konqueror",                CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAny },
     { CCgiUserAgent::eLynx,         "Lynx",                     "Lynx",                     CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAppProduct },
     { CCgiUserAgent::eLynx,         "ELynx", /* Linx based */   "ELynx",                    CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAppProduct },
     { CCgiUserAgent::eOregano,      "Oregano",                  "Oregano2",                 CCgiUserAgent::eEngine_Unknown, CCgiUserAgent::ePlatform_Unknown,      fAppComment },
@@ -969,17 +981,22 @@ void CCgiUserAgent::x_Parse(const string& user_agent)
     pos = m_UserAgent.find(search);
     if (pos != NPOS) {
         m_Browser = eIE;
-        m_Engine  = eEngine_IE;
+        m_Engine = eEngine_IE;
         m_BrowserName = "Internet Explorer";
         m_BrowserVersion = m_EngineVersion;
         pos += search.length();
         s_ParseVersion(m_UserAgent, pos, &m_EngineVersion);
     }
 
+    // Microsoft Edge
+    // Temporary: set browser/engine version to the same value
+    if (m_Engine == eEngine_Edge) {
+        m_EngineVersion = m_BrowserVersion;
+    }
+
     // Try to get browser version for IE-based browsers or old IE
-    if ( m_Engine == eEngine_IE  &&
-         m_BrowserVersion.GetMajor() == -1 )
-    {
+    if (m_Engine == eEngine_IE  &&
+        m_BrowserVersion.GetMajor() == -1) {
         TUserAgentVersion v;
         search = USTR(" MSIE ");
         pos = m_UserAgent.find(search);
@@ -987,11 +1004,9 @@ void CCgiUserAgent::x_Parse(const string& user_agent)
             pos += search.length();
             s_ParseVersion(m_UserAgent, pos, &v);
         }
-        // Set engine version for IE-based browsers to IE version
-        if ( m_Browser == eIE ) {
+        // Set browser version for IE only, don't change for IE-based browsers
+        if (m_Browser == eIE) {
             m_BrowserVersion = v;
-        } else {
-            m_EngineVersion = v;
         }
     }
 
@@ -1029,15 +1044,40 @@ void CCgiUserAgent::x_Parse(const string& user_agent)
         }
     }
 
-    // Try to get engine version for KHTML-based browsers
-    search = USTR(" AppleWebKit/");
-    pos = m_UserAgent.find(search);
-    if (pos != NPOS) {
-        m_Engine = eEngine_KHTML;
-        pos += search.length();
-        s_ParseVersion(m_UserAgent, pos, &m_EngineVersion);
-    }
+    // Try to get engine version for KHTML/WebKit-based browsers
 
+    if (m_Engine != eEngine_IE  &&  m_Engine != eEngine_Edge) {
+        search = USTR(" AppleWebKit/");
+        pos = m_UserAgent.find(search);
+        if (pos == NPOS) {
+            search = USTR(" WebKit/");
+            pos = m_UserAgent.find(search);
+        }
+        if (pos != NPOS) {
+            m_Engine = eEngine_WebKit;
+            pos += search.length();
+            s_ParseVersion(m_UserAgent, pos, &m_EngineVersion);
+            // AppleWebKit/537.36
+            if (m_EngineVersion == CVersionInfo(537,36,-1) ) {
+                m_Engine = eEngine_Blink;
+            }
+        } else {
+            search = USTR(" KHTML/");
+            pos = m_UserAgent.find(search);
+            if (pos != NPOS) {
+                m_Engine = eEngine_KHTML;
+                pos += search.length();
+                s_ParseVersion(m_UserAgent, pos, &m_EngineVersion);
+            } else {
+                search = USTR("KHTML,");
+                pos = m_UserAgent.find(search);
+                if (pos != NPOS) {
+                    m_Engine = eEngine_KHTML;
+                    m_EngineVersion.SetVersion(-1, -1, -1);
+                }
+            }
+        }
+    }
 
     // Hack for some browsers (like Safari) that use Version/x.x.x rather than
     // Safari/x.x.x for real browser version (for Safari, numbers after browser
@@ -1055,7 +1095,8 @@ void CCgiUserAgent::x_Parse(const string& user_agent)
         } else {
             // Safari (old version) -- try to get browser version
             // depending on engine (WebKit) version (very approximately).
-            if ( m_Browser == eSafari  &&  m_Engine == eEngine_KHTML) { 
+            if ( m_Browser == eSafari  &&
+                (m_Engine == eEngine_KHTML  ||  m_Engine == eEngine_WebKit)) { 
                 // See http://www.useragentstring.com/pages/Safari/
                 int rev = m_EngineVersion.GetMajor();
                 if (rev < 85 ) {
@@ -1204,9 +1245,12 @@ string CCgiUserAgent::GetEngineName(void) const
 {
     switch ( GetEngine() ) {
         case eEngine_Unknown : return "Unknown";
-        case eEngine_IE      : return "IE";
+        case eEngine_IE      : return "Trident";
+        case eEngine_Edge    : return "Edge";
         case eEngine_Gecko   : return "Gecko";
         case eEngine_KHTML   : return "KHTML";
+        case eEngine_WebKit  : return "WebKit";
+        case eEngine_Blink   : return "Blink";
         case eEngine_Bot     : return "Bot";
     }
     _TROUBLE;
