@@ -1,4 +1,4 @@
-/* $Id: ncbi_system.cpp 355784 2012-03-08 14:27:29Z ivanov $
+/* $Id: ncbi_system.cpp 488254 2015-12-29 13:56:26Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
 #include <corelib/ncbi_system.hpp>
 #include <corelib/ncbi_safe_static.hpp>
 #include <corelib/error_codes.hpp>
+#include <corelib/ncbierror.hpp>
 #include "ncbisys.hpp"
 
 
@@ -53,10 +54,12 @@ extern "C" {
 #  include <sys/resource.h>
 #  include <sys/times.h>
 #  include <sys/types.h>
+#  include <dirent.h>
 #  include <limits.h>
 #  include <time.h>
 #  include <unistd.h>
-#  if defined(NCBI_OS_BSD) || defined(NCBI_OS_DARWIN)
+#  include <fcntl.h>
+#  if defined(NCBI_OS_BSD)  ||  defined(NCBI_OS_DARWIN)
 #    include <sys/sysctl.h>
 #  endif //NCBI_OS_BSD || NCBI_OS_DARWIN
 #  if defined(NCBI_OS_IRIX)
@@ -101,12 +104,6 @@ struct SProcessMemoryCounters
 
 #endif //NCBI_OS_MSWIN
 
-#ifdef NCBI_OS_CYGWIN
-#  define WINVER 0x0500
-#  define WIN32_LEAN_AND_MEAN
-#  include <windows.h>
-#endif //NCBI_OS_CYGWIN
-
 
 BEGIN_NCBI_SCOPE
 
@@ -129,8 +126,9 @@ extern "C" {
 DEFINE_STATIC_FAST_MUTEX(s_ExitHandler_Mutex);
 static bool                  s_ExitHandlerIsSet  = false;
 static ELimitsExitCode       s_ExitCode          = eLEC_None;
-static CSafeStaticPtr<CTime> s_TimeSet;
-static size_t                s_MemoryLimit       = 0;
+static CSafeStatic<CTime>    s_TimeSet;
+static size_t                s_MemoryLimitSoft   = 0;
+static size_t                s_MemoryLimitHard   = 0;
 static size_t                s_CpuTimeLimit      = 0;
 static char*                 s_ReserveMemory     = 0;
 static TLimitsPrintHandler   s_PrintHandler      = 0;
@@ -143,6 +141,8 @@ static TLimitsPrintParameter s_PrintHandlerParam = 0;
 
 
 // Routine to be called at the exit from application
+// It is not async-safe, so using it with SetCpuLimits() can lead to coredump
+// and program crash. Be aware.
 //
 static void s_ExitHandler(void)
 {
@@ -160,7 +160,7 @@ static void s_ExitHandler(void)
 
         switch ( s_ExitCode ) {
         case eLEC_Memory: {
-            limit_size = s_MemoryLimit;
+            limit_size = s_MemoryLimitSoft;
             break;
         }
         case eLEC_Cpu: {
@@ -182,11 +182,11 @@ static void s_ExitHandler(void)
     case eLEC_Memory:
         {
             ERR_POST_X(1, "Memory heap limit exceeded in allocating memory " \
-                          "by operator new (" << s_MemoryLimit << " bytes)");
+                          "by operator new (" << s_MemoryLimitSoft << " bytes)");
             break;
         }
         
-    case eLEC_Cpu: 
+    case eLEC_Cpu:
         {
             ERR_POST_X(2, "CPU time limit exceeded (" << s_CpuTimeLimit << " sec)");
             tms buffer;
@@ -204,13 +204,13 @@ static void s_ExitHandler(void)
 #endif
             if (tick == (clock_t)(-1))
                 tick = 0;
-            LOG_POST_X(4, "\tuser CPU time   : " << 
+            ERR_POST_X(4, Note << "\tuser CPU time   : " << 
                           buffer.tms_utime/(tick ? tick : 1) <<
                           (tick ? " sec" : " tick"));
-            LOG_POST_X(5, "\tsystem CPU time : " << 
+            ERR_POST_X(5, Note << "\tsystem CPU time : " << 
                           buffer.tms_stime/(tick ? tick : 1) <<
                           (tick ? " sec" : " tick"));
-            LOG_POST_X(6, "\ttotal CPU time  : " <<
+            ERR_POST_X(6, Note << "\ttotal CPU time  : " <<
                           (buffer.tms_stime + buffer.tms_utime)/(tick ? tick : 1) <<
                           (tick ? " sec" : " tick"));
             break;
@@ -224,11 +224,11 @@ static void s_ExitHandler(void)
     CTime ct(CTime::eCurrent);
     CTime et(2000, 1, 1);
     et.AddSecond((int) (ct.GetTimeT() - s_TimeSet->GetTimeT()));
-    LOG_POST_X(7, "Program's time: " << Endm <<
+    ERR_POST_X(7, Note << "Program's time: " << Endm <<
                   "\tstart limit - " << s_TimeSet->AsString() << Endm <<
                   "\ttermination - " << ct.AsString() << Endm);
     et.SetFormat("h:m:s");
-    LOG_POST_X(8, "\texecution   - " << et.AsString());
+    ERR_POST_X(8, Note << "\texecution   - " << et.AsString());
 }
 
 
@@ -246,12 +246,13 @@ static bool s_SetExitHandler(TLimitsPrintHandler handler,
         }
         s_ExitHandlerIsSet = true;
         s_TimeSet->SetCurrent();
-
-        // Store print handler and parameter
+        
+        // Store new print handler and its parameter
         s_PrintHandler = handler;
         s_PrintHandlerParam = parameter;
-
-        // Reserve some memory (10Kb)
+        
+        // Reserve some memory (10Kb) to allow the diagnostic API
+        // print messages on exit if memory limit is set.
         s_ReserveMemory = new char[10*1024];
     }
     return true;
@@ -280,65 +281,178 @@ bool SetMemoryLimit(size_t max_size,
                     TLimitsPrintHandler handler, 
                     TLimitsPrintParameter parameter)
 {
-    if (s_MemoryLimit == max_size) 
+    if (s_MemoryLimitSoft == max_size) {
         return true;
-    
-    if ( !s_SetExitHandler(handler, parameter) )
+    }
+    if (!s_SetExitHandler(handler, parameter)) {
         return false;
-
-    // Set new heap limit
+    }
     CFastMutexGuard LOCK(s_ExitHandler_Mutex);
     
     rlimit rl;
     if ( max_size ) {
         set_new_handler(s_NewHandler);
         rl.rlim_cur = rl.rlim_max = max_size;
-    }
-    else {
-        // Set off heap limit
+    } else {
         set_new_handler(0);
         rl.rlim_cur = rl.rlim_max = RLIM_INFINITY;
     }
-    if (setrlimit(RLIMIT_DATA, &rl) != 0) 
+    if (setrlimit(RLIMIT_DATA, &rl) != 0) {
         return false;
+    }
 #  if !defined(NCBI_OS_SOLARIS)
-    if (setrlimit(RLIMIT_AS, &rl) != 0) 
+    if (setrlimit(RLIMIT_AS, &rl) != 0) {
         return false;
+    }
 #  endif //NCBI_OS_SOLARIS
 
-    s_MemoryLimit = max_size;
+    s_MemoryLimitSoft = max_size;
+    s_MemoryLimitHard = max_size;
+    if ( max_size ) {
+        set_new_handler(s_NewHandler);
+    } else {
+        set_new_handler(0);
+    }
     return true;
 }
 
 
-// deprecated
+bool SetMemoryLimitSoft(size_t max_size,
+                        TLimitsPrintHandler handler, 
+                        TLimitsPrintParameter parameter)
+{
+    if (s_MemoryLimitSoft == max_size) {
+        return true;
+    }
+    if (!s_SetExitHandler(handler, parameter)) {
+        return false;
+    }
+    CFastMutexGuard LOCK(s_ExitHandler_Mutex);
+
+    rlimit rl;
+    if (getrlimit(RLIMIT_DATA, &rl) != 0) {
+        return false;
+    }
+    if ( max_size ) {
+        rl.rlim_cur = max_size;
+    } else {
+        rl.rlim_cur = RLIM_INFINITY;
+    }
+    if (setrlimit(RLIMIT_DATA, &rl) != 0) {
+        return false;
+    }
+#  if !defined(NCBI_OS_SOLARIS)
+    rlimit rlas;
+    if (getrlimit(RLIMIT_AS, &rlas) != 0) {
+        return false;
+    }
+    rl.rlim_max = rlas.rlim_max;
+    if (setrlimit(RLIMIT_AS, &rl) != 0) {
+        return false;
+    }
+#  endif //NCBI_OS_SOLARIS
+
+    s_MemoryLimitSoft = max_size;
+    if ( max_size ) {
+        set_new_handler(s_NewHandler);
+    } else {
+        set_new_handler(0);
+    }
+    return true;
+}
+
+
+bool SetMemoryLimitHard(size_t max_size,
+                        TLimitsPrintHandler handler, 
+                        TLimitsPrintParameter parameter)
+{
+    if (s_MemoryLimitHard == max_size) {
+        return true;
+    }
+    if (!s_SetExitHandler(handler, parameter)) {
+        return false;
+    }
+    CFastMutexGuard LOCK(s_ExitHandler_Mutex);
+
+    size_t cur_soft_limit = 0;
+    rlimit rl;
+    if (getrlimit(RLIMIT_DATA, &rl) != 0) {
+        return false;
+    }
+    if ( max_size ) {
+        rl.rlim_max = max_size;
+        if (rl.rlim_cur > max_size) {
+            rl.rlim_cur = max_size;
+        }
+        cur_soft_limit = rl.rlim_cur;
+    } else {
+        rl.rlim_max = RLIM_INFINITY;
+    }
+    if (setrlimit(RLIMIT_DATA, &rl) != 0) {
+        return false;
+    }
+#  if !defined(NCBI_OS_SOLARIS)
+    rlimit rlas;
+    if (getrlimit(RLIMIT_AS, &rlas) != 0) {
+        return false;
+    }
+    if ( max_size ) {
+        rlas.rlim_max = max_size;
+        // Descrease current soft limit of the virtual memory
+        // to the size of data segment -- min(DATA,AS). 
+        if (rlas.rlim_cur > cur_soft_limit) {
+            rlas.rlim_cur = cur_soft_limit;
+        }
+        // And use this size as current value for the soft limit
+        // in the print handler 
+        cur_soft_limit = rlas.rlim_cur;
+    } else {
+        rlas.rlim_max = RLIM_INFINITY;
+    }
+    if (setrlimit(RLIMIT_AS, &rlas) != 0) {
+        return false;
+    }
+#  endif //NCBI_OS_SOLARIS
+
+    s_MemoryLimitSoft = cur_soft_limit;
+    s_MemoryLimitHard = max_size;
+    if ( max_size ) {
+        set_new_handler(s_NewHandler);
+    } else {
+        set_new_handler(0);
+    }
+    return true;
+}
+
+
+// @deprecated
 bool SetHeapLimit(size_t max_size,
                   TLimitsPrintHandler handler, 
                   TLimitsPrintParameter parameter)
 {
-    if (s_MemoryLimit == max_size) 
+    if (s_MemoryLimitSoft == max_size) { 
         return true;
-    
-    if ( !s_SetExitHandler(handler, parameter) )
+    }
+    if (!s_SetExitHandler(handler, parameter)) {
         return false;
-
-    // Set new heap limit
+    }
     CFastMutexGuard LOCK(s_ExitHandler_Mutex);
     
     rlimit rl;
     if ( max_size ) {
-        set_new_handler(s_NewHandler);
         rl.rlim_cur = rl.rlim_max = max_size;
-    }
-    else {
-        // Set off heap limit
-        set_new_handler(0);
+    } else {
         rl.rlim_cur = rl.rlim_max = RLIM_INFINITY;
     }
-    if (setrlimit(RLIMIT_DATA, &rl) != 0) 
+    if (setrlimit(RLIMIT_DATA, &rl) != 0) {
         return false;
-
-    s_MemoryLimit = max_size;
+    }
+    s_MemoryLimitSoft = max_size;
+    if ( max_size ) {
+        set_new_handler(s_NewHandler);
+    } else {
+        set_new_handler(0);
+    }
     return true;
 }
 
@@ -352,6 +466,20 @@ bool SetMemoryLimit(size_t max_size,
   return false;
 }
 
+bool SetMemoryLimitSoft(size_t max_size, 
+                    TLimitsPrintHandler handler, 
+                    TLimitsPrintParameter parameter)
+{
+  return false;
+}
+
+bool SetMemoryLimitHard(size_t max_size, 
+                    TLimitsPrintHandler handler, 
+                    TLimitsPrintParameter parameter)
+{
+  return false;
+}
+
 bool SetHeapLimit(size_t max_size, 
                   TLimitsPrintHandler handler, 
                   TLimitsPrintParameter parameter)
@@ -375,38 +503,34 @@ static void s_SignalHandler(int _DEBUG_ARG(sig))
     _ASSERT(sig == SIGXCPU);
     _VERIFY(signal(SIGXCPU, SIG_IGN) != SIG_ERR);
     s_ExitCode = eLEC_Cpu;
-    // if (s_ExitHandlerIsSet) {
-    //     s_ExitHandler();
-    // }
-    // NB: _exit() does not go over atexit() chain
+    // _exit() does not go over atexit() chain, so just call registered
+    // handler directly. Be aware that it should be async-safe!
+    if (s_ExitHandlerIsSet) {
+        s_ExitHandler();
+    }
     _exit(-1);
 }
 
-
-bool SetCpuTimeLimit(size_t                max_cpu_time,
+bool SetCpuTimeLimit(unsigned int          max_cpu_time,
+                     unsigned int          terminate_delay_time,
                      TLimitsPrintHandler   handler, 
-                     TLimitsPrintParameter parameter,
-                     size_t                terminate_time)
+                     TLimitsPrintParameter parameter)
 {
-    if (s_CpuTimeLimit == max_cpu_time) 
+    if (s_CpuTimeLimit == max_cpu_time) {
         return true;
-    
-    if ( !s_SetExitHandler(handler, parameter) )
+    }
+    if (!s_SetExitHandler(handler, parameter)) {
         return false;
-    
-    // Set new CPU time limit
+    }
     CFastMutexGuard LOCK(s_ExitHandler_Mutex);
 
-    struct rlimit rl;
+    rlimit rl;
     if ( max_cpu_time ) {
         rl.rlim_cur = max_cpu_time;
-        rl.rlim_max = max_cpu_time + terminate_time;
-    }
-    else {
-        // Set off CPU time limit
+        rl.rlim_max = max_cpu_time + terminate_delay_time;
+    } else {
         rl.rlim_cur = rl.rlim_max = RLIM_INFINITY;
     }
-
     if (setrlimit(RLIMIT_CPU, &rl) != 0) {
         return false;
     }
@@ -422,11 +546,10 @@ bool SetCpuTimeLimit(size_t                max_cpu_time,
 
 #else
 
-bool SetCpuTimeLimit(size_t                max_cpu_time,
+bool SetCpuTimeLimit(unsigned int          max_cpu_time,
+                     unsigned int          terminate_delay_time,
                      TLimitsPrintHandler   handler, 
-                     TLimitsPrintParameter parameter,
-                     size_t                terminate_time)
-
+                     TLimitsPrintParameter parameter)
 {
     return false;
 }
@@ -434,6 +557,18 @@ bool SetCpuTimeLimit(size_t                max_cpu_time,
 #endif //USE_SETCPULIMIT
 
 
+// @deprecated
+bool SetCpuTimeLimit(size_t                max_cpu_time,
+                     TLimitsPrintHandler   handler, 
+                     TLimitsPrintParameter parameter,
+                     size_t                terminate_delay_time)
+{
+    return SetCpuTimeLimit((unsigned int)max_cpu_time,
+                           (unsigned int)terminate_delay_time,
+                           handler, parameter);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 //
 /// System information
@@ -441,7 +576,7 @@ bool SetCpuTimeLimit(size_t                max_cpu_time,
 
 unsigned int GetCpuCount(void)
 {
-#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#if defined(NCBI_OS_MSWIN)
     SYSTEM_INFO si;
     GetSystemInfo(&si);
     return (unsigned int) si.dwNumberOfProcessors;
@@ -452,7 +587,7 @@ unsigned int GetCpuCount(void)
     kern_return_t rc;
 
     rc = host_info(mach_host_self(), HOST_BASIC_INFO,
-                   (host_info_t)&hinfo, &hinfo_count);
+                  (host_info_t)&hinfo, &hinfo_count);
 
     if (rc != KERN_SUCCESS) {
         return 1;
@@ -465,12 +600,12 @@ unsigned int GetCpuCount(void)
     nproc = sysconf(_SC_NPROC_ONLN);
 # elif defined(_SC_NPROCESSORS_ONLN)
     nproc = sysconf(_SC_NPROCESSORS_ONLN);
-# elif defined(NCBI_OS_BSD) || defined(NCBI_OS_DAWRIN)
+# elif defined(NCBI_OS_BSD)  ||  defined(NCBI_OS_DAWRIN)
     size_t len = sizeof(nproc);
     int mib[2];
     mib[0] = CTL_HW;
     mib[1] = HW_NCPU;
-    if (sysctl(mib, 2, &nproc, &len, 0, 0) < 0 || len != sizeof(nproc))
+    if (sysctl(mib, 2, &nproc, &len, 0, 0) < 0  ||  len != sizeof(nproc))
         nproc = -1;
 # endif //UNIX_FLAVOR
     return nproc <= 0 ? 1 : (unsigned int) nproc;
@@ -483,18 +618,19 @@ unsigned int GetCpuCount(void)
 bool GetCurrentProcessTimes(double* user_time, double* system_time)
 {
 #if defined(NCBI_OS_MSWIN)
-    // Each FILETIME structure contains the number of 100-nanosecond time units.
+    // Each FILETIME structure contains the number of 100-nanosecond time units
     FILETIME ft_creation, ft_exit, ft_kernel, ft_user;
-    if (! ::GetProcessTimes(GetCurrentProcess(),
-                            &ft_creation, &ft_exit,
-                            &ft_kernel, &ft_user) ) {
-            return false;
+    if (!::GetProcessTimes(GetCurrentProcess(),
+                           &ft_creation, &ft_exit, &ft_kernel, &ft_user)) {
+        return false;
     }
-    if (system_time) {
-        *system_time = (ft_kernel.dwLowDateTime + ((Uint8)ft_kernel.dwHighDateTime<<32)) * 1.0e-7;
+    if ( system_time ) {
+        *system_time = (ft_kernel.dwLowDateTime +
+                        ((Uint8) ft_kernel.dwHighDateTime << 32)) * 1.0e-7;
     }
-    if (user_time) {
-        *user_time = (ft_user.dwLowDateTime + ((Uint8)ft_user.dwHighDateTime<<32)) * 1.0e-7;
+    if ( user_time ) {
+        *user_time = (ft_user.dwLowDateTime +
+                      ((Uint8) ft_user.dwHighDateTime << 32)) * 1.0e-7;
     }
 
 #elif defined(NCBI_OS_UNIX)
@@ -515,11 +651,11 @@ bool GetCurrentProcessTimes(double* user_time, double* system_time)
     if (tick == (clock_t)(-1)) {
         return false;
     }
-    if (system_time) {
-        *system_time = (double)buf.tms_stime / tick;
+    if ( system_time ) {
+        *system_time = (double)buf.tms_stime / (double)tick;
     }
-    if (user_time) {
-        *user_time = (double)buf.tms_utime / tick;
+    if ( user_time ) {
+        *user_time = (double)buf.tms_utime / (double)tick;
     }
 #endif
     return true;
@@ -537,7 +673,7 @@ unsigned long GetVirtualMemoryPageSize(void)
     static unsigned long ps = 0;
 
     if (!ps) {
-#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#if defined(NCBI_OS_MSWIN)
         SYSTEM_INFO si;
         GetSystemInfo(&si);
         ps = (unsigned long) si.dwPageSize;
@@ -574,13 +710,13 @@ unsigned long GetVirtualMemoryAllocationGranularity(void)
     static unsigned long ag = 0;
 
     if (!ag) {
-#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#if defined(NCBI_OS_MSWIN)
         SYSTEM_INFO si;
         GetSystemInfo(&si);
         ag = (unsigned long) si.dwAllocationGranularity;
 #else
         ag = GetVirtualMemoryPageSize();
-#endif //NCBI_OS_MSWIN || NCBI_OS_CYGWIN
+#endif
     }
     return ag;
 }
@@ -588,7 +724,7 @@ unsigned long GetVirtualMemoryAllocationGranularity(void)
 
 Uint8 GetPhysicalMemorySize(void)
 {
-#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#if defined(NCBI_OS_MSWIN)
 
     MEMORYSTATUSEX st;
     st.dwLength = sizeof(st);
@@ -611,10 +747,10 @@ Uint8 GetPhysicalMemorySize(void)
     uint64_t physmem;
     mib[1] = HW_MEMSIZE;
 #  else
-    /* Native BSD, may be truncated */
+    // Native BSD, may be truncated
     int      physmem;
     mib[1] = HW_PHYSMEM;
-#  endif /*HW_MEMSIZE*/
+#  endif //HW_MEMSIZE
     mib[0] = CTL_HW;
     len = sizeof(physmem);
     if (sysctl(mib, 2, &physmem, &len, 0, 0) == 0  &&  len == sizeof(physmem)){
@@ -623,7 +759,7 @@ Uint8 GetPhysicalMemorySize(void)
 
 #  ifdef NCBI_OS_DARWIN
     {
-        /* heavier fallback */
+        // heavier fallback
         struct vm_statistics vm_stat;
         mach_port_t my_host = mach_host_self();
         mach_msg_type_number_t count = HOST_VM_INFO_COUNT;
@@ -690,9 +826,9 @@ bool GetMemoryUsage(size_t* total, size_t* resident, size_t* shared)
 #elif defined(NCBI_OS_SOLARIS)
     Int8 len = CFile("/proc/self/as").GetLength();
     if (len > 0) {
-        *total    = (size_t)len;
-        *resident = (size_t)len; // conservative estimate
-        *shared   = 0;           // does this info exist anywhere?
+        *total    = (size_t) len;
+        *resident = (size_t) len; // conservative estimate
+        *shared   = 0;            // does this info exist anywhere?
         return true;
     }
 #elif defined(HAVE_GETRUSAGE)
@@ -705,27 +841,48 @@ bool GetMemoryUsage(size_t* total, size_t* resident, size_t* shared)
         memset(&t, '\0', sizeof(t));
         if (times(&t) != (clock_t)(-1)) {
             clock_t ticks = t.tms_utime + t.tms_stime;
-            *total    = _DIV0(ru.ru_ixrss + ru.ru_idrss + ru.ru_isrss, ticks);
             *resident = _DIV0(ru.ru_idrss,                             ticks);
             *shared   = _DIV0(ru.ru_ixrss,                             ticks);
-            return true;
+            *total    = _DIV0(ru.ru_ixrss + ru.ru_idrss + ru.ru_isrss, ticks);
+#  ifdef NCBI_OS_DARWIN
+            if (*total > 0)
+#  endif
+                return true;
         }
     }
 #  undef _DIV0
+#  ifdef NCBI_OS_DARWIN
+#    ifdef MACH_TASK_BASIC_INFO
+    task_flavor_t               flavor       = MACH_TASK_BASIC_INFO;
+    struct mach_task_basic_info t_info;
+    mach_msg_type_number_t      t_info_count = MACH_TASK_BASIC_INFO_COUNT;
+#    else
+    task_flavor_t          flavor       = TASK_BASIC_INFO;
+    struct task_basic_info t_info;
+    mach_msg_type_number_t t_info_count = TASK_BASIC_INFO_COUNT;
+#    endif
+    if (task_info(mach_task_self(), flavor, (task_info_t)&t_info,
+                  &t_info_count) == KERN_SUCCESS) {
+        *total = *resident = t_info.resident_size;
+        *shared = 0; // unavailable, even with mach_task_basic_info
+        return true;
+    }
+#  endif
 #endif
     return false;
 }
 
 
-#if !defined(HAVE_MADVISE)
+#ifndef HAVE_MADVISE
 bool MemoryAdvise(void*, size_t, EMemoryAdvise) {
     return false;
 }
-#else  /* HAVE_MADVISE */
+#else //HAVE_MADVISE
 bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
 {
     if ( !addr /*|| !len*/ ) {
         ERR_POST_X(11, "Memory address is not specified");
+        CNcbiError::Set(CNcbiError::eBadAddress);
         return false;
     }
     int adv;
@@ -746,6 +903,7 @@ bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
             break;
         #else
             ERR_POST_X_ONCE(12, Warning << "MADV_DONTFORK not supported");
+            CNcbiError::Set(CNcbiError::eNotSupported);
             return false;
         #endif        
     case eMADV_DoFork:
@@ -754,6 +912,7 @@ bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
             break;
         #else
             ERR_POST_X_ONCE(12, Warning << "MADV_DOTFORK not supported");
+            CNcbiError::Set(CNcbiError::eNotSupported);
             return false;
         #endif        
     case eMADV_Mergeable:
@@ -762,6 +921,7 @@ bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
             break;
         #else
             ERR_POST_X_ONCE(12, Warning << "MADV_MERGEABLE not supported");
+            CNcbiError::Set(CNcbiError::eNotSupported);
             return false;
         #endif        
     case eMADV_Unmergeable:
@@ -770,6 +930,7 @@ bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
             break;
         #else
             ERR_POST_X_ONCE(12, Warning << "MADV_UNMERGEABLE not supported");
+            CNcbiError::Set(CNcbiError::eNotSupported);
             return false;
         #endif        
     default:
@@ -781,12 +942,12 @@ bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
         int x_errno = errno; \
         ERR_POST_X(13, "madvise() failed: " << 
                    _T_STDSTRING(NcbiSys_strerror(x_errno)));
-        errno = x_errno;
+        CNcbiError::SetErrno(errno = x_errno);
         return false;
     }
     return true;
 }
-#endif  /* HAVE_MADVISE */
+#endif //HAVE_MADVISE
 
 
 
@@ -797,20 +958,24 @@ bool MemoryAdvise(void* addr, size_t len, EMemoryAdvise advise)
 
 void SleepMicroSec(unsigned long mc_sec, EInterruptOnSignal onsignal)
 {
-#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#if defined(NCBI_OS_MSWIN)
 
     // Unlike some of its (buggy) Unix counterparts, MS-Win's Sleep() is safe
     // to use with 0, which causes the current thread to sleep at most until
     // the end of the current timeslice (and only if the CPU is not idle).
-    Sleep((mc_sec + 500) / 1000);
+    static const unsigned long kMicroSecondsPerMilliSecond
+        = kMicroSecondsPerSecond / kMilliSecondsPerSecond;
+    Sleep((mc_sec + (kMicroSecondsPerMilliSecond / 2))
+          / kMicroSecondsPerMilliSecond);
 
 #elif defined(NCBI_OS_UNIX)
 
 #  if defined(HAVE_NANOSLEEP)
     struct timespec delay, unslept;
     memset(&unslept, 0, sizeof(unslept));
-    delay.tv_sec  =  mc_sec / kMicroSecondsPerSecond;
-    delay.tv_nsec = (mc_sec % kMicroSecondsPerSecond) * 1000;
+    delay.tv_sec  =   mc_sec / kMicroSecondsPerSecond;
+    delay.tv_nsec = ((mc_sec % kMicroSecondsPerSecond)
+                     * (kNanoSecondsPerSecond / kMicroSecondsPerSecond));
     while (nanosleep(&delay, &unslept) < 0) {
         if (errno != EINTR  ||  onsignal == eInterruptOnSignal)
             break;
@@ -826,6 +991,7 @@ void SleepMicroSec(unsigned long mc_sec, EInterruptOnSignal onsignal)
                 return;
         }
     }
+    // usleep() detects errors (such as EINTR) but can't tell unslept time :-/
     usleep(usec);
 #  else
     // Portable but ugly.
@@ -837,7 +1003,7 @@ void SleepMicroSec(unsigned long mc_sec, EInterruptOnSignal onsignal)
     delay.tv_sec  = mc_sec / kMicroSecondsPerSecond;
     delay.tv_usec = mc_sec % kMicroSecondsPerSecond;
     while (select(0, (fd_set*) 0, (fd_set*) 0, (fd_set*) 0, &delay) < 0) {
-#    if defined(SELECT_UPDATES_TIMEOUT)
+#    ifdef SELECT_UPDATES_TIMEOUT
         if (errno != EINTR  ||  onsignal == eInterruptOnSignal)
 #    endif
             break;
@@ -850,11 +1016,12 @@ void SleepMicroSec(unsigned long mc_sec, EInterruptOnSignal onsignal)
 
 void SleepMilliSec(unsigned long ml_sec, EInterruptOnSignal onsignal)
 {
-#if defined(NCBI_OS_MSWIN)  ||  defined(NCBI_OS_CYGWIN)
+#ifdef NCBI_OS_MSWIN
     Sleep(ml_sec);
-#elif defined(NCBI_OS_UNIX)
-    SleepMicroSec(ml_sec * 1000, onsignal);
-#endif //NCBI_OS_...
+#else
+    SleepMicroSec(ml_sec * (kMicroSecondsPerSecond / kMilliSecondsPerSecond),
+                  onsignal);
+#endif //NCBI_OS_MSWIN
 }
 
 
@@ -872,8 +1039,8 @@ void SleepSec(unsigned long sec, EInterruptOnSignal onsignal)
 
 #ifdef NCBI_OS_MSWIN
 
-static bool s_EnableSuppressSystemMessageBox = true;
-static bool s_DoneSuppressSystemMessageBox   = false;
+static bool s_EnableSuppressSystemMessageBox  = true;
+static bool s_DoneSuppressSystemMessageBox    = false;
 static bool s_SuppressedDebugSystemMessageBox = false;
 
 // Handler for "Unhandled" exceptions
@@ -892,7 +1059,7 @@ extern void SuppressSystemMessageBox(TSuppressSystemMessageBox mode)
         return;
     }
     // System errors
-    if ( (mode & fSuppress_System) == fSuppress_System ) {
+    if ((mode & fSuppress_System) == fSuppress_System ) {
        SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX |
                     SEM_NOOPENFILEERRORBOX);
     }
@@ -939,7 +1106,102 @@ extern bool IsSuppressedDebugSystemMessageBox(void)
         s_SuppressedDebugSystemMessageBox;
 #else
     return false;
-#endif
+#endif //NCBI_OS_MSWIN
+}
+
+
+
+extern int GetProcessFDCount(int* soft_limit, int* hard_limit)
+{
+#ifdef NCBI_OS_UNIX
+    int            fd_count = 0;
+    struct dirent* dp;
+
+    rlim_t  cur_limit = -1;
+    rlim_t  max_limit = -1;
+
+    struct rlimit rlim;
+    if (getrlimit(RLIMIT_NOFILE, &rlim) == 0) {
+        cur_limit = rlim.rlim_cur;
+        max_limit = rlim.rlim_max;
+    } else {
+        // fallback to sysconf
+        ERR_POST_ONCE(Warning << "getrlimit(RLIMIT_NOFILE, ...) call failed. "
+                                 "Using sysconf(_SC_OPEN_MAX) instead.");
+        cur_limit = static_cast<rlim_t>(sysconf(_SC_OPEN_MAX));
+    }
+
+    DIR* dir = opendir("/proc/self/fd/");
+    if (dir) {
+        while ((dp = readdir(dir)) != NULL)
+            ++fd_count;
+        closedir(dir);
+        fd_count -= 3;  // '.', '..' and the one for 'opendir'
+        if (fd_count < 0)
+            fd_count = -1;
+    } else {
+        // Fallback to analysis via looping over all fds
+        if (cur_limit > 0) {
+            int     max_fd = static_cast<int>(cur_limit);
+            if (cur_limit > INT_MAX)
+                max_fd = INT_MAX;
+            for (int fd = 0;  fd < max_fd;  ++fd) {
+                if (fcntl(fd, F_GETFD, 0) == -1) {
+                    if (errno == EBADF)
+                        continue;
+                }
+                ++fd_count;
+            }
+        } else {
+            fd_count = -1;
+        }
+    }
+
+    if (soft_limit  ||  hard_limit) {
+        if (soft_limit) {
+            if (cur_limit > INT_MAX)
+                *soft_limit = INT_MAX;
+            else
+                *soft_limit = static_cast<int>(cur_limit);
+        }
+        if (hard_limit) {
+            if (max_limit > INT_MAX)
+                *hard_limit = INT_MAX;
+            else
+                *hard_limit = static_cast<int>(max_limit);
+        }
+    }
+    return fd_count;
+#else
+    if (soft_limit)
+        *soft_limit = -1;
+    if (hard_limit)
+        *hard_limit = -1;
+    return -1;
+#endif //NCBI_OS_UNIX
+}
+
+
+extern int GetProcessThreadCount(void)
+{
+#ifdef NCBI_OS_LINUX
+    int            thread_count = 0;
+    struct dirent* dp;
+
+    DIR* dir = opendir("/proc/self/task/");
+    if (dir) {
+        while ((dp = readdir(dir)) != NULL)
+            ++thread_count;
+        closedir(dir);
+        thread_count -= 2;  // '.' and '..'
+        if (thread_count <= 0)
+            thread_count = -1;
+        return thread_count;
+    }
+    return -1;
+#else
+    return -1;
+#endif //NCBI_OS_LINUX
 }
 
 
