@@ -1,4 +1,4 @@
-/*  $Id: writer_cache.cpp 408805 2013-08-01 20:55:20Z rafanovi $
+/*  $Id: writer_cache.cpp 494478 2016-03-07 19:26:06Z ivanov $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -32,8 +32,8 @@
 #include <objtools/data_loaders/genbank/cache/writer_cache_entry.hpp>
 #include <objtools/data_loaders/genbank/cache/reader_cache_params.h>
 #include <objtools/data_loaders/genbank/readers.hpp> // for entry point
-#include <objtools/data_loaders/genbank/request_result.hpp>
-#include <objtools/data_loaders/genbank/dispatcher.hpp>
+#include <objtools/data_loaders/genbank/impl/request_result.hpp>
+#include <objtools/data_loaders/genbank/impl/dispatcher.hpp>
 
 #include <corelib/rwstream.hpp>
 #include <corelib/plugin_manager_store.hpp>
@@ -122,135 +122,54 @@ void CCacheWriter::ResetCache(void)
 }
 
 
-void CCacheWriter::SaveStringSeq_ids(CReaderRequestResult& result,
-                                     const string& seq_id)
+void CCacheWriter::CStoreBuffer::CheckSpace(size_t add)
 {
-    if ( !m_IdCache) {
-        return;
+    if ( m_Ptr + add > m_End ) {
+        size_t old_size = size();
+        size_t new_size = (old_size+add)*2;
+        char* new_buf = new char[new_size];
+        memcpy(new_buf, data(), old_size);
+        x_FreeBuffer();
+        m_Buffer = new_buf;
+        m_Ptr = new_buf + old_size;
+        m_End = new_buf + new_size;
     }
-
-    CLoadLockSeq_ids ids(result, seq_id);
-    WriteSeq_ids(seq_id, ids);
+    _ASSERT(m_Ptr + add <= m_End);
 }
 
 
-namespace {
-    class CStoreBuffer {
-    public:
-        CStoreBuffer(void)
-            : m_Buffer(m_Buffer0),
-              m_End(m_Buffer0+sizeof(m_Buffer0)),
-              m_Ptr(m_Buffer)
-            {
-            }
-        ~CStoreBuffer(void)
-            {
-                x_FreeBuffer();
-            }
-        
-        const char* data(void) const
-            {
-                return m_Buffer;
-            }
-        size_t size(void) const
-            {
-                return m_Ptr - m_Buffer;
-            }
-        void CheckStore(size_t size);
-        void StoreUint4(Uint4 v)
-            {
-                CheckStore(4);
-                x_StoreUint4(v);
-            }
-        void StoreInt4(Int4 v)
-            {
-                StoreUint4(v);
-            }
-        void StoreString(const string& s)
-            {
-                size_t size = s.size();
-                CheckStore(4+size);
-                x_StoreUint4(ToUint4(size));
-                memcpy(m_Ptr, s.data(), size);
-                m_Ptr += size;
-            }
-
-        static Uint4 ToUint4(size_t size)
-            {
-                Uint4 ret = Uint4(size);
-                if ( ret != size ) {
-                    NCBI_THROW(CLoaderException, eLoaderFailed,
-                               "Uint4 overflow");
-                }
-                return ret;
-            }
-    protected:
-        void x_FreeBuffer(void);
-        void x_StoreUint4(Uint4 v)
-            {
-                m_Ptr[0] = v>>24;
-                m_Ptr[1] = v>>16;
-                m_Ptr[2] = v>>8;
-                m_Ptr[3] = v;
-                m_Ptr += 4;
-            }
-
-    private:
-        CStoreBuffer(const CStoreBuffer&);
-        void operator=(const CStoreBuffer&);
-
-        char m_Buffer0[256];
-        char* m_Buffer;
-        char* m_End;
-        char* m_Ptr;
-    };
-
-    
-    void CStoreBuffer::CheckStore(size_t add)
-    {
-        if ( m_Ptr + add > m_End ) {
-            size_t old_size = size();
-            size_t new_size = (old_size+add)*2;
-            char* new_buf = new char[new_size];
-            memcpy(new_buf, data(), old_size);
-            x_FreeBuffer();
-            m_Buffer = new_buf;
-            m_Ptr = new_buf + old_size;
-            m_End = new_buf + new_size;
-        }
-        _ASSERT(m_Ptr + add <= m_End);
-    }
+void CCacheWriter::CStoreBuffer::StoreString(const string& s)
+{
+    size_t size = s.size();
+    CheckSpace(4+size);
+    x_StoreUint4(ToUint4(size));
+    memcpy(m_Ptr, s.data(), size);
+    m_Ptr += size;
+}
 
 
-    void CStoreBuffer::x_FreeBuffer(void)
-    {
-        if ( m_Buffer != m_Buffer0 ) {
-            delete[] m_Buffer;
-            m_Buffer = m_End = m_Ptr = 0;
-        }
+void CCacheWriter::CStoreBuffer::x_FreeBuffer(void)
+{
+    if ( m_Buffer != m_Buffer0 ) {
+        delete[] m_Buffer;
+        m_Buffer = m_End = m_Ptr = 0;
     }
 }
 
-void CCacheWriter::SaveStringGi(CReaderRequestResult& result,
-                                const string& seq_id)
-{
-    if( !m_IdCache) {
-        return;
-    }
 
-    CLoadLockSeq_ids ids(result, seq_id);
-    if ( ids->IsLoadedGi() ) {
-        CStoreBuffer str;
-        str.StoreInt4(GI_TO(Int4, ids->GetGi()));
-        try {
-            if ( GetDebugLevel() ) {
-                LOG_POST(Info<<"CCache:Write: "<<seq_id<<","<<GetGiSubkey());
-            }
-            m_IdCache->Store(seq_id, 0, GetGiSubkey(),
-                             str.data(), str.size());
-        }
-        catch ( exception& ) { // ignored
+void CCacheWriter::x_WriteId(const string& key,
+                             const string& subkey,
+                             const char* data,
+                             size_t size)
+{
+    try {
+        if ( GetDebugLevel() > 0 ) {
+            CReader::CDebugPrinter s("CCacheWriter");
+            s<<key<<","<<subkey<<" size="<<size;
         }
+        m_IdCache->Store(key, 0, subkey, data, size);
+    }
+    catch ( exception& ) { // ignored
     }
 }
 
@@ -262,7 +181,7 @@ void CCacheWriter::SaveSeq_idSeq_ids(CReaderRequestResult& result,
         return;
     }
 
-    CLoadLockSeq_ids ids(result, seq_id);
+    CLoadLockSeqIds ids(result, seq_id);
     WriteSeq_ids(GetIdKey(seq_id), ids);
 }
 
@@ -274,19 +193,13 @@ void CCacheWriter::SaveSeq_idGi(CReaderRequestResult& result,
         return;
     }
 
-    CLoadLockSeq_ids ids(result, seq_id);
-    if ( ids->IsLoadedGi() ) {
+    CLoadLockGi lock(result, seq_id);
+    if ( lock.IsLoadedGi() && lock.GetExpType() == GBL::eExpire_normal ) {
         CStoreBuffer str;
-        str.StoreInt4(GI_TO(Int4, ids->GetGi()));
-        try {
-            if ( GetDebugLevel() ) {
-                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetGiSubkey());
-            }
-            m_IdCache->Store(GetIdKey(seq_id), 0, GetGiSubkey(),
-                             str.data(), str.size());
-        }
-        catch ( exception& ) { // ignored
-        }
+        CReader::TSequenceGi data = lock.GetGi();
+        _ASSERT(lock.IsFound(data));
+        str.StoreInt8(lock.GetGi(data));
+        x_WriteId(GetIdKey(seq_id), GetGiSubkey(), str);
     }
 }
 
@@ -298,21 +211,15 @@ void CCacheWriter::SaveSeq_idAccVer(CReaderRequestResult& result,
         return;
     }
 
-    CLoadLockSeq_ids ids(result, seq_id);
-    if ( ids->IsLoadedAccVer() ) {
+    CLoadLockAcc lock(result, seq_id);
+    if ( lock.IsLoadedAccVer() && lock.GetExpType() == GBL::eExpire_normal ) {
         string str;
-        if ( CSeq_id_Handle acc = ids->GetAccVer() ) {
-            str = acc.AsString();
-        }
-        try {
-            if ( GetDebugLevel() ) {
-                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetAccVerSubkey());
-            }
-            m_IdCache->Store(GetIdKey(seq_id), 0, GetAccVerSubkey(),
-                             str.data(), str.size());
-        }
-        catch ( exception& ) { // ignored
+        CReader::TSequenceAcc acc = lock.GetAccVer();
+        _ASSERT(lock.IsFound(acc));
+        if ( lock.GetAcc(acc) ) {
+            str = lock.GetAcc(acc).AsString();
         }
+        x_WriteId(GetIdKey(seq_id), GetAccVerSubkey(), str);
     }
 }
 
@@ -324,18 +231,11 @@ void CCacheWriter::SaveSeq_idLabel(CReaderRequestResult& result,
         return;
     }
 
-    CLoadLockSeq_ids ids(result, seq_id);
-    if ( ids->IsLoadedLabel() ) {
-        const string& str = ids->GetLabel();
-        try {
-            if ( GetDebugLevel() ) {
-                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetLabelSubkey());
-            }
-            m_IdCache->Store(GetIdKey(seq_id), 0, GetLabelSubkey(),
-                             str.data(), str.size());
-        }
-        catch ( exception& ) { // ignored
-        }
+    CLoadLockLabel lock(result, seq_id);
+    if ( lock.IsLoadedLabel() && lock.GetExpType() == GBL::eExpire_normal ) {
+        const string& str = lock.GetLabel();
+        _ASSERT(!str.empty());
+        x_WriteId(GetIdKey(seq_id), GetLabelSubkey(), str);
     }
 }
 
@@ -347,70 +247,85 @@ void CCacheWriter::SaveSeq_idTaxId(CReaderRequestResult& result,
         return;
     }
 
-    CLoadLockSeq_ids ids(result, seq_id);
-    if ( ids->IsLoadedTaxId() ) {
+    CLoadLockTaxId lock(result, seq_id);
+    if ( lock.IsLoadedTaxId() && lock.GetExpType() == GBL::eExpire_normal ) {
         CStoreBuffer str;
-        str.StoreInt4(ids->GetTaxId());
-        try {
-            if ( GetDebugLevel() ) {
-                LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<GetTaxIdSubkey());
-            }
-            m_IdCache->Store(GetIdKey(seq_id), 0, GetTaxIdSubkey(),
-                             str.data(), str.size());
-        }
-        catch ( exception& ) { // ignored
-        }
+        _ASSERT(lock.GetTaxId() != -1);
+        str.StoreInt4(lock.GetTaxId());
+        x_WriteId(GetIdKey(seq_id), GetTaxIdSubkey(), str);
     }
 }
 
 
-namespace {
-    class CCacheDataEraser {
-        CCacheDataEraser(ICache* cache,
-                         const string& key, int version, const string& subkey)
-            : m_Cache(cache),
-              m_Key(key), m_Version(version), m_Subkey(subkey)
-            {
-            }
-        ~CCacheDataEraser(void) {
-            if ( m_Cache ) {
-                try {
-                    m_Cache->Remove(m_Key, m_Version, m_Subkey);
-                }
-                catch ( exception& ) { // ignored
-                }
-                m_Cache = 0;
-            }
-        }
-
-        void Done(void) {
-            m_Cache = 0;
-        }
-        
-    private:
-        ICache* m_Cache;
-        string m_Key;
-        int m_Version;
-        string m_Subkey;
-    };
-}
-
-
-void CCacheWriter::WriteSeq_ids(const string& key,
-                                const CLoadLockSeq_ids& ids)
+void CCacheWriter::SaveSequenceHash(CReaderRequestResult& result,
+                                    const CSeq_id_Handle& seq_id)
 {
     if( !m_IdCache) {
         return;
     }
 
-    if ( !ids.IsLoaded() ) {
+    CLoadLockHash lock(result, seq_id);
+    if ( lock.IsLoadedHash() && lock.GetExpType() == GBL::eExpire_normal ) {
+        CStoreBuffer str;
+        CReader::TSequenceHash data = lock.GetHash();
+        str.StoreInt4(data.hash);
+        str.StoreBool(data.sequence_found);
+        str.StoreBool(data.hash_known);
+        x_WriteId(GetIdKey(seq_id), GetHashSubkey(), str);
+    }
+}
+
+
+void CCacheWriter::SaveSequenceLength(CReaderRequestResult& result,
+                                      const CSeq_id_Handle& seq_id)
+{
+    if( !m_IdCache) {
+        return;
+    }
+
+    CLoadLockLength lock(result, seq_id);
+    if ( lock.IsLoadedLength() && lock.GetExpType() == GBL::eExpire_normal ) {
+        CStoreBuffer str;
+        _ASSERT(lock.GetLength() != kInvalidSeqPos);
+        str.StoreUint4(lock.GetLength());
+        x_WriteId(GetIdKey(seq_id), GetLengthSubkey(), str);
+    }
+}
+
+
+void CCacheWriter::SaveSequenceType(CReaderRequestResult& result,
+                                    const CSeq_id_Handle& seq_id)
+{
+    if( !m_IdCache) {
+        return;
+    }
+
+    CLoadLockType lock(result, seq_id);
+    if ( lock.IsLoadedType() && lock.GetExpType() == GBL::eExpire_normal ) {
+        CStoreBuffer str;
+        str.StoreInt4(lock.GetType().type);
+        x_WriteId(GetIdKey(seq_id), GetTypeSubkey(), str);
+    }
+}
+
+
+void CCacheWriter::WriteSeq_ids(const string& key,
+                                const CLoadLockSeqIds& lock)
+{
+    if( !m_IdCache) {
+        return;
+    }
+
+    if ( !lock.IsLoaded() || lock.GetExpType() != GBL::eExpire_normal ) {
         return;
     }
 
     try {
-        if ( GetDebugLevel() ) {
-            LOG_POST(Info<<"CCache:Write: "<<key<<","<<GetSeq_idsSubkey());
+        if ( GetDebugLevel() > 0 ) {
+            CReader::CDebugPrinter s("CCacheWriter");
+            s<<key<<","<<GetSeq_idsSubkey();
         }
+        _ASSERT(!lock.GetSeq_ids().empty());
         auto_ptr<IWriter> writer
             (m_IdCache->GetWriteStream(key, 0, GetSeq_idsSubkey()));
         if ( !writer.get() ) {
@@ -419,8 +334,10 @@ void CCacheWriter::WriteSeq_ids(const string& key,
 
         CWStream w_stream(writer.release(), 0, 0, CRWStreambuf::fOwnAll);
         CObjectOStreamAsnBinary obj_stream(w_stream);
-        static_cast<CObjectOStream&>(obj_stream).WriteUint4(CStoreBuffer::ToUint4(ids->size()));
-        ITERATE ( CLoadInfoSeq_ids, it, *ids ) {
+        CFixedSeq_ids seq_ids = lock.GetSeq_ids();
+        static_cast<CObjectOStream&>(obj_stream). // cast because of protected
+            WriteUint4(CStoreBuffer::ToUint4(seq_ids.size()));
+        ITERATE ( CFixedSeq_ids, it, seq_ids ) {
             obj_stream << *it->GetSeqId();
         }
     }
@@ -428,7 +345,7 @@ void CCacheWriter::WriteSeq_ids(const string& key,
         // In case of an error we need to remove incomplete data
         // from the cache.
         try {
-            m_BlobCache->Remove(key, 0, GetSeq_idsSubkey());
+            m_IdCache->Remove(key, 0, GetSeq_idsSubkey());
         }
         catch ( exception& ) { // ignored
         }
@@ -441,12 +358,12 @@ void CCacheWriter::SaveSeq_idBlob_ids(CReaderRequestResult& result,
                                       const CSeq_id_Handle& seq_id,
                                       const SAnnotSelector* sel)
 {
-    if ( !m_IdCache) {
+    if ( !m_IdCache ) {
         return;
     }
 
-    CLoadLockBlob_ids ids(result, seq_id, sel);
-    if ( !ids.IsLoaded() ) {
+    CLoadLockBlobIds ids(result, seq_id, sel);
+    if ( !ids.IsLoaded() || ids.GetExpType() != GBL::eExpire_normal ) {
         return;
     }
 
@@ -454,27 +371,37 @@ void CCacheWriter::SaveSeq_idBlob_ids(CReaderRequestResult& result,
     GetBlob_idsSubkey(sel, subkey, true_subkey);
     CStoreBuffer str;
     str.StoreInt4(BLOB_IDS_MAGIC);
-    str.StoreUint4(ids->GetState());
-    str.StoreUint4(str.ToUint4(ids->size()));
-    ITERATE ( CLoadInfoBlob_ids, it, *ids ) {
-        const CBlob_id& id = *it->first;
+    CFixedBlob_ids blob_ids = ids.GetBlob_ids();
+    str.StoreUint4(blob_ids.GetState());
+    str.StoreUint4(str.ToUint4(blob_ids.size()));
+    ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+        const CBlob_Info& info = *it;
+        const CBlob_id& id = *info.GetBlob_id();
         str.StoreUint4(id.GetSat());
         str.StoreUint4(id.GetSubSat());
         str.StoreUint4(id.GetSatKey());
-        const CBlob_Info& info = it->second;
         str.StoreUint4(info.GetContentsMask());
-        str.StoreUint4(str.ToUint4(info.GetNamedAnnotNames().size()));
-        ITERATE(CBlob_Info::TNamedAnnotNames, it2, info.GetNamedAnnotNames()) {
-            str.StoreString(*it2);
+        CConstRef<CBlob_Annot_Info> annot_info = info.GetAnnotInfo();
+        if ( annot_info ) {
+            const CBlob_Annot_Info::TNamedAnnotNames& names =
+                annot_info->GetNamedAnnotNames();
+            str.StoreUint4(str.ToUint4(names.size()));
+            ITERATE ( CBlob_Annot_Info::TNamedAnnotNames, it2, names ) {
+                str.StoreString(*it2);
+            }
         }
-        if ( info.GetAnnotInfo().empty() ) {
+        else {
+            str.StoreUint4(0);
+        }
+        if ( !annot_info || annot_info->GetAnnotInfo().empty() ) {
             str.StoreString(kEmptyStr);
         }
         else {
             CNcbiOstrstream stream_str;
             {{
                 CObjectOStreamAsnBinary stream(stream_str);
-                ITERATE( CBlob_Info::TAnnotInfo, it2, info.GetAnnotInfo() ) {
+                ITERATE( CBlob_Annot_Info::TAnnotInfo, it2,
+                         annot_info->GetAnnotInfo() ) {
                     stream << **it2;
                 }
             }}
@@ -484,14 +411,22 @@ void CCacheWriter::SaveSeq_idBlob_ids(CReaderRequestResult& result,
     if ( !true_subkey.empty() ) {
         str.StoreString(true_subkey);
     }
-    try {
-        if ( GetDebugLevel() ) {
-            LOG_POST(Info<<"CCache:Write: "<<GetIdKey(seq_id)<<","<<subkey);
-        }
-        m_IdCache->Store(GetIdKey(seq_id), 0, subkey, str.data(), str.size());
-    }
-    catch ( exception& ) { // ignored
+    x_WriteId(GetIdKey(seq_id), subkey, str);
+}
+
+
+void CCacheWriter::SaveBlobState(CReaderRequestResult& /*result*/,
+                                 const TBlobId& blob_id,
+                                 TBlobState blob_state)
+{
+    if( !m_IdCache ) {
+        return;
     }
+
+    _ASSERT(blob_state >= 0);
+    CStoreBuffer str;
+    str.StoreInt4(blob_state);
+    x_WriteId(GetBlobKey(blob_id), GetBlobStateSubkey(), str);
 }
 
 
@@ -503,18 +438,10 @@ void CCacheWriter::SaveBlobVersion(CReaderRequestResult& /*result*/,
         return;
     }
 
-    _ASSERT(version);
+    _ASSERT(version >= 0);
     CStoreBuffer str;
     str.StoreInt4(version);
-    try {
-        if ( GetDebugLevel() ) {
-            LOG_POST(Info<<"CCache:Write: "<<GetBlobKey(blob_id)<<","<<GetBlobVersionSubkey());
-        }
-        m_IdCache->Store(GetBlobKey(blob_id), 0, GetBlobVersionSubkey(),
-                         str.data(), str.size());
-    }
-    catch ( exception& ) { // ignored
-    }
+    x_WriteId(GetBlobKey(blob_id), GetBlobVersionSubkey(), str);
 }
 
 
@@ -528,12 +455,13 @@ public:
         : m_Cache(cache), m_Key(key), m_Version(version), m_Subkey(subkey),
           m_Writer(cache->GetWriteStream(key, version, subkey))
         {
-            _ASSERT(version);
-            if ( SCacheInfo::GetDebugLevel() ) {
-                LOG_POST(Info<<"CCache:Write: "<<key<<","<<subkey<<","<<version);
+            _ASSERT(version >= 0);
+            if ( SCacheInfo::GetDebugLevel() > 0 ) {
+                CReader::CDebugPrinter s("CCacheWriter");
+                s<<key<<","<<subkey<<","<<version;
             }
             if ( version == -1 ) {
-                ERR_POST("CCache:Write: "<<key<<","<<subkey<<","<<version);
+                ERR_POST("Cache:Write: "<<key<<","<<subkey<<","<<version);
             }
             if ( m_Writer.get() ) {
                 m_Stream.reset(new CWStream(m_Writer.get()));
@@ -604,11 +532,18 @@ CCacheWriter::OpenBlobStream(CReaderRequestResult& result,
     }
 
     try {
-        CLoadLockBlob blob(result, blob_id);
-        _ASSERT(blob.GetBlobVersion());
+        CLoadLockBlob blob(result, blob_id, chunk_id);
+        TBlobVersion version = blob.GetKnownBlobVersion();
+        if ( version < 0 ) {
+            CLoadLockBlobVersion version_lock(result, blob_id, eAlreadyLoaded);
+            if ( version_lock ) {
+                version = version_lock.GetBlobVersion();
+            }
+        }
+        _ASSERT(version >= 0);
         CRef<CBlobStream> stream
             (new CCacheBlobStream(m_BlobCache, GetBlobKey(blob_id),
-                                  blob.GetBlobVersion(),
+                                  version,
                                   GetBlobSubkey(blob, chunk_id)));
         if ( !stream->CanWrite() ) {
             return null;
