@@ -1,4 +1,4 @@
-/*  $Id: choice.cpp 112866 2007-10-25 14:00:30Z vasilche $
+/*  $Id: choice.cpp 497436 2016-04-06 17:56:51Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -36,22 +36,34 @@
 #include <serial/objcopy.hpp>
 #include <serial/delaybuf.hpp>
 #include <serial/serialbase.hpp>
+#include <serial/objhook.hpp>
 
 BEGIN_NCBI_SCOPE
 
 class CChoiceTypeInfoFunctions
 {
 public:
+    static void ReadChoiceSimple(CObjectIStream& in,
+                                 TTypeInfo objectType,
+                                 TObjectPtr objectPtr);
     static void ReadChoiceDefault(CObjectIStream& in,
                                   TTypeInfo objectType,
                                   TObjectPtr objectPtr);
     static void WriteChoiceDefault(CObjectOStream& out,
                                    TTypeInfo objectType,
                                    TConstObjectPtr objectPtr);
+    static void SkipChoiceSimple(CObjectIStream& in,
+                                 TTypeInfo objectType);
     static void SkipChoiceDefault(CObjectIStream& in,
                                   TTypeInfo objectType);
     static void CopyChoiceDefault(CObjectStreamCopier& copier,
                                   TTypeInfo objectType);
+    static void AssignDefault(TTypeInfo typeInfo,
+                              TObjectPtr dst, TConstObjectPtr src,
+                              ESerialRecursionMode how);
+    static void AssignSimple(TTypeInfo typeInfo,
+                             TObjectPtr dst, TConstObjectPtr src,
+                             ESerialRecursionMode how);
 };
 
 typedef CChoiceTypeInfoFunctions TFunc;
@@ -119,6 +131,20 @@ void CChoiceTypeInfo::InitChoiceTypeInfoFunctions(void)
     SetCopyFunction(&TFunc::CopyChoiceDefault);
     SetSkipFunction(&TFunc::SkipChoiceDefault);
     m_SelectDelayFunction = 0;
+    m_AllowEmpty = false;
+}
+
+void CChoiceTypeInfo::AdjustChoiceTypeInfoFunctions(void)
+{
+    m_AllowEmpty = GetVariants().Size() == 0 || (
+                   GetVariantInfo(kFirstMemberIndex)->GetId().HaveNoPrefix() &&
+                   CItemsInfo::FindNextMandatory(this) == NULL);
+    if ( !m_AllowEmpty &&
+         !GetVariantInfo(kFirstMemberIndex)->GetId().IsAttlist() ) {
+        // simple
+        SetReadFunction(&TFunc::ReadChoiceSimple);
+        SetSkipFunction(&TFunc::SkipChoiceSimple);
+    }
 }
 
 CVariantInfo* CChoiceTypeInfo::AddVariant(const char* memberId,
@@ -171,15 +197,15 @@ bool CChoiceTypeInfo::Equals(TConstObjectPtr object1, TConstObjectPtr object2,
 {
     // User defined comparison
     if ( IsCObject() ) {
-        const CSerialUserOp* op1 =
-            dynamic_cast<const CSerialUserOp*>
-            (static_cast<const CObject*>(object1));
-        const CSerialUserOp* op2 =
-            dynamic_cast<const CSerialUserOp*>
-            (static_cast<const CObject*>(object2));
-        if ( op1  &&  op2 ) {
-            if ( !op1->UserOp_Equals(*op2) )
-                return false;
+        if ( const CSerialUserOp* op1 =
+             dynamic_cast<const CSerialUserOp*>
+             (static_cast<const CObject*>(object1)) ) {
+            if ( const CSerialUserOp* op2 =
+                 dynamic_cast<const CSerialUserOp*>
+                 (static_cast<const CObject*>(object2)) ) {
+                if ( !op1->UserOp_Equals(*op2) )
+                    return false;
+            }
         }
     }
 
@@ -212,6 +238,24 @@ void CChoiceTypeInfo::SetDefault(TObjectPtr dst) const
     ResetIndex(dst);
 }
 
+inline
+void CChoiceTypeInfo::CallUserOp_Assign(TObjectPtr dst,
+                                        TConstObjectPtr src) const
+{
+    // User defined assignment
+    if ( IsCObject() ) {
+        if ( const CSerialUserOp* opsrc =
+             dynamic_cast<const CSerialUserOp*>
+             (static_cast<const CObject*>(src)) ) {
+            if ( CSerialUserOp* opdst =
+                 dynamic_cast<CSerialUserOp*>
+                 (static_cast<CObject*>(dst)) ) {
+                opdst->UserOp_Assign(*opsrc);
+            }
+        }
+    }
+}
+
 void CChoiceTypeInfo::Assign(TObjectPtr dst, TConstObjectPtr src,
                              ESerialRecursionMode how) const
 {
@@ -238,20 +282,63 @@ void CChoiceTypeInfo::Assign(TObjectPtr dst, TConstObjectPtr src,
     }
 
     // User defined assignment
-    if ( IsCObject() ) {
-        const CSerialUserOp* opsrc =
-            dynamic_cast<const CSerialUserOp*>
-            (static_cast<const CObject*>(src));
-        CSerialUserOp* opdst =
-            dynamic_cast<CSerialUserOp*>
-            (static_cast<CObject*>(dst));
-        if ( opdst  &&  opsrc ) {
-            opdst->UserOp_Assign(*opsrc);
-        }
-    }
+    CallUserOp_Assign(dst, src);
 }
 
-void CChoiceTypeInfo::SetSelectDelayFunction(TSelectDelayFunction func)
+void CChoiceTypeInfoFunctions::AssignDefault(TTypeInfo typeInfo,
+                                             TObjectPtr dst,
+                                             TConstObjectPtr src,
+                                             ESerialRecursionMode how)
+{
+    const CChoiceTypeInfo* choiceType =
+        static_cast<const CChoiceTypeInfo*>(typeInfo);
+    TMemberIndex index;
+
+    index = choiceType->GetVariants().FirstIndex();
+    const CVariantInfo* variantInfo = choiceType->GetVariantInfo(index);
+    if (variantInfo->GetId().IsAttlist()) {
+        const CMemberInfo* info =
+            static_cast<const CMemberInfo*>(choiceType->GetVariants().GetItemInfo(index));
+        info->GetTypeInfo()->Assign(GetMember(info, dst),
+                                    GetMember(info, src),how);
+    }
+
+    index = choiceType->GetIndex(src);
+    if ( index == kEmptyChoice )
+        choiceType->ResetIndex(dst);
+    else {
+        _ASSERT(index >= choiceType->GetVariants().FirstIndex() && 
+                index <= choiceType->GetVariants().LastIndex());
+        choiceType->SetIndex(dst, index);
+        choiceType->GetVariantInfo(index)->GetTypeInfo()->Assign(choiceType->GetData(dst, index),
+                                                                 choiceType->GetData(src, index), how);
+    }
+
+    choiceType->CallUserOp_Assign(dst, src);
+}
+
+
+void CChoiceTypeInfoFunctions::AssignSimple(TTypeInfo typeInfo,
+                                            TObjectPtr dst,
+                                            TConstObjectPtr src,
+                                            ESerialRecursionMode how)
+{
+    const CChoiceTypeInfo* choiceType =
+        static_cast<const CChoiceTypeInfo*>(typeInfo);
+    TMemberIndex index = choiceType->GetIndex(src);
+    if ( index == kEmptyChoice ) {
+        choiceType->ResetIndex(dst);
+        return;
+    }
+    const CVariantInfo* variantInfo = choiceType->GetVariantInfo(index);
+    choiceType->SetIndex(dst, index);
+    variantInfo->GetTypeInfo()->Assign(variantInfo->GetVariantPtr(dst),
+                                       variantInfo->GetVariantPtr(src),
+                                       how);
+}
+
+
+void CChoiceTypeInfo::SetSelectDelay(TSelectDelayFunction func)
 {
     _ASSERT(m_SelectDelayFunction == 0);
     _ASSERT(func != 0);
@@ -270,40 +357,57 @@ void CChoiceTypeInfoFunctions::ReadChoiceDefault(CObjectIStream& in,
 {
     const CChoiceTypeInfo* choiceType =
         CTypeConverter<CChoiceTypeInfo>::SafeCast(objectType);
-
-    BEGIN_OBJECT_FRAME_OF2(in, eFrameChoice, choiceType);
+    BEGIN_OBJECT_FRAME_OF3(in, eFrameChoice, choiceType, objectPtr);
     in.BeginChoice(choiceType);
     BEGIN_OBJECT_FRAME_OF(in, eFrameChoiceVariant);
     TMemberIndex index = in.BeginChoiceVariant(choiceType);
     if ( index == kInvalidMember ) {
-        if (in.CanSkipUnknownVariants()) {
+        if (choiceType->MayBeEmpty() || in.CanSkipUnknownVariants()) {
             in.SkipAnyContentVariant();
         } else {
             in.ThrowError(in.fFormatError, "choice variant id expected");
         }
     } else {
+        for (;;) {
         const CVariantInfo* variantInfo = choiceType->GetVariantInfo(index);
         if (variantInfo->GetId().IsAttlist()) {
             const CMemberInfo* memberInfo =
-                dynamic_cast<const CMemberInfo*>(
+                static_cast<const CMemberInfo*>(
                     choiceType->GetVariants().GetItemInfo(index));
             memberInfo->ReadMember(in,objectPtr);
             in.EndChoiceVariant();
             index = in.BeginChoiceVariant(choiceType);
-            if ( index == kInvalidMember )
-                in.ThrowError(in.fFormatError, "choice variant id expected");
+            if ( index == kInvalidMember ) {
+                if (in.CanSkipUnknownVariants()) {
+                    in.SkipAnyContentVariant();
+                    break;
+                } else {
+                    in.ThrowError(in.fFormatError, "choice variant id expected");
+                }
+            }
             variantInfo = choiceType->GetVariantInfo(index);
         }
         in.SetTopMemberId(variantInfo->GetId());
 
         variantInfo->ReadVariant(in, objectPtr);
         in.EndChoiceVariant();
+        break;
+        }
     }
     END_OBJECT_FRAME_OF(in);
     in.EndChoice();
     END_OBJECT_FRAME_OF(in);
 }
 
+void CChoiceTypeInfoFunctions::ReadChoiceSimple(CObjectIStream& in,
+                                                TTypeInfo objectType,
+                                                TObjectPtr objectPtr)
+{
+    const CChoiceTypeInfo* choiceType =
+        CTypeConverter<CChoiceTypeInfo>::SafeCast(objectType);
+    in.ReadChoiceSimple(choiceType, objectPtr);
+}
+
 void CChoiceTypeInfoFunctions::WriteChoiceDefault(CObjectOStream& out,
                                                   TTypeInfo objectType,
                                                   TConstObjectPtr objectPtr)
@@ -311,7 +415,7 @@ void CChoiceTypeInfoFunctions::WriteChoiceDefault(CObjectOStream& out,
     const CChoiceTypeInfo* choiceType =
         CTypeConverter<CChoiceTypeInfo>::SafeCast(objectType);
 
-    BEGIN_OBJECT_FRAME_OF2(out, eFrameChoice, choiceType);
+    BEGIN_OBJECT_FRAME_OF3(out, eFrameChoice, choiceType, objectPtr);
     out.BeginChoice(choiceType);
     TMemberIndex index = choiceType->GetVariants().FirstIndex();
     const CVariantInfo* variantInfo = choiceType->GetVariantInfo(index);
@@ -323,17 +427,21 @@ void CChoiceTypeInfoFunctions::WriteChoiceDefault(CObjectOStream& out,
     }
 
     index = choiceType->GetIndex(objectPtr);
-    if ( index == kInvalidMember )
-        out.ThrowError(out.fInvalidData, "cannot write empty choice");
+    if ( index == kInvalidMember ) {
+        if (!choiceType->MayBeEmpty()) {
+            out.ThrowError(out.fInvalidData, "cannot write empty choice");
+        }
+    } else {
+        variantInfo = choiceType->GetVariantInfo(index);
+        BEGIN_OBJECT_FRAME_OF2(out, eFrameChoiceVariant, variantInfo->GetId());
+        out.BeginChoiceVariant(choiceType, variantInfo->GetId());
 
-    variantInfo = choiceType->GetVariantInfo(index);
-    BEGIN_OBJECT_FRAME_OF2(out, eFrameChoiceVariant, variantInfo->GetId());
-    out.BeginChoiceVariant(choiceType, variantInfo->GetId());
+        variantInfo->WriteVariant(out, objectPtr);
 
-    variantInfo->WriteVariant(out, objectPtr);
+        out.EndChoiceVariant();
+        END_OBJECT_FRAME_OF(out);
+    }
 
-    out.EndChoiceVariant();
-    END_OBJECT_FRAME_OF(out);
     out.EndChoice();
     END_OBJECT_FRAME_OF(out);
 }
@@ -349,34 +457,78 @@ void CChoiceTypeInfoFunctions::SkipChoiceDefault(CObjectIStream& in,
 {
     const CChoiceTypeInfo* choiceType =
         CTypeConverter<CChoiceTypeInfo>::SafeCast(objectType);
-
     BEGIN_OBJECT_FRAME_OF2(in, eFrameChoice, choiceType);
     in.BeginChoice(choiceType);
     BEGIN_OBJECT_FRAME_OF(in, eFrameChoiceVariant);
     TMemberIndex index = in.BeginChoiceVariant(choiceType);
-    if ( index == kInvalidMember )
-        in.ThrowError(in.fFormatError,"choice variant id expected");
-    const CVariantInfo* variantInfo = choiceType->GetVariantInfo(index);
-    if (variantInfo->GetId().IsAttlist()) {
-        const CMemberInfo* memberInfo =
-            dynamic_cast<const CMemberInfo*>(
-                choiceType->GetVariants().GetItemInfo(index));
-        memberInfo->SkipMember(in);
+    if ( index == kInvalidMember ) {
+        if (choiceType->MayBeEmpty() || in.CanSkipUnknownVariants()) {
+            in.SkipAnyContentVariant();
+        } else {
+            in.ThrowError(in.fFormatError, "choice variant id expected");
+        }
+    } else {
+        for (;;) {
+        const CVariantInfo* variantInfo = choiceType->GetVariantInfo(index);
+        if (variantInfo->GetId().IsAttlist()) {
+            const CMemberInfo* memberInfo =
+                dynamic_cast<const CMemberInfo*>(
+                    choiceType->GetVariants().GetItemInfo(index));
+            memberInfo->SkipMember(in);
+            in.EndChoiceVariant();
+            index = in.BeginChoiceVariant(choiceType);
+            if ( index == kInvalidMember ) {
+                if (in.CanSkipUnknownVariants()) {
+                    in.SkipAnyContentVariant();
+                    break;
+                } else {
+                    in.ThrowError(in.fFormatError, "choice variant id expected");
+                }
+            }
+            variantInfo = choiceType->GetVariantInfo(index);
+        }
+        in.SetTopMemberId(variantInfo->GetId());
+
+        variantInfo->SkipVariant(in);
         in.EndChoiceVariant();
-        index = in.BeginChoiceVariant(choiceType);
-        if ( index == kInvalidMember )
-            in.ThrowError(in.fFormatError,"choice variant id expected");
-        variantInfo = choiceType->GetVariantInfo(index);
+        break;
+        }
     }
 
-    in.SetTopMemberId(variantInfo->GetId());
-
-    variantInfo->SkipVariant(in);
-
-    in.EndChoiceVariant();
     END_OBJECT_FRAME_OF(in);
     in.EndChoice();
     END_OBJECT_FRAME_OF(in);
 }
 
+
+void CChoiceTypeInfoFunctions::SkipChoiceSimple(CObjectIStream& in,
+                                                 TTypeInfo objectType)
+{
+    const CChoiceTypeInfo* choiceType =
+        CTypeConverter<CChoiceTypeInfo>::SafeCast(objectType);
+    in.SkipChoiceSimple(choiceType);
+}
+
+
+void CChoiceTypeInfo::SetGlobalHook(const CTempString& variants,
+                                    CReadChoiceVariantHook* hook_ptr)
+{
+    CRef<CReadChoiceVariantHook> hook(hook_ptr);
+    if ( variants == "*" ) {
+        for ( CIterator i(this); i.Valid(); ++i ) {
+            const_cast<CVariantInfo*>(GetVariantInfo(i))->
+                SetGlobalReadHook(hook);
+        }
+    }
+    else {
+        vector<CTempString> tokens;
+        NStr::Split(variants, ",", tokens, NStr::fSplit_NoMergeDelims);
+        ITERATE ( vector<CTempString>, it, tokens ) {
+            const_cast<CVariantInfo*>(GetVariantInfo(*it))->
+                SetGlobalReadHook(hook);
+        }
+    }
+}
+
+
 END_NCBI_SCOPE
