@@ -1,4 +1,4 @@
-/*  $Id: streambuf.cpp 207374 2010-10-05 15:20:21Z ivanov $
+/* $Id: streambuf.cpp 489095 2016-01-08 13:02:41Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -23,7 +23,7 @@
  *
  * ===========================================================================
  *
- * Authors:  Vladimir Ivanov
+ * Author:  Vladimir Ivanov
  *
  * File Description:  CCompression based C++ streambuf
  *
@@ -74,12 +74,11 @@ CCompressionStreambuf::CCompressionStreambuf(
     }
 
     // Allocate memory for all buffers at one time
-    auto_ptr<CT_CHAR_TYPE> bp(new CT_CHAR_TYPE[read_bufsize + write_bufsize]);
+    AutoArray<CT_CHAR_TYPE> bp(new CT_CHAR_TYPE[size_t(read_bufsize + write_bufsize)]);
     m_Buf = bp.get();
     if ( !m_Buf ) {
         return;
     }
-
     // Init processors and set the buffer pointers
     if ( m_Reader ) {
         m_Reader->Init();
@@ -118,30 +117,40 @@ CCompressionStreambuf::~CCompressionStreambuf()
                          "through call Finalize()"
     #define msg_error    "Finalize() failed"
 
-    // Read processor
-    sp = GetStreamProcessor(CCompressionStream::eRead);
-    if ( sp ) {
-        sp->m_Processor->End();
-        sp->m_State = CSP::eDone;
-    }
-    // Write processor
-    sp = GetStreamProcessor(CCompressionStream::eWrite);
-    if ( sp ) {
-        if ( sp->m_State == CSP::eInit  ||
-             sp->m_State == CSP::eActive ) {
-            Finalize(CCompressionStream::eWrite);
-            if ( sp->m_LastStatus == CP::eStatus_Overflow ) {
-                ERR_COMPRESS(72, msg_where << msg_overflow);
+    try {
+        // Read processor
+        sp = GetStreamProcessor(CCompressionStream::eRead);
+        if ( sp ) {
+            bool abandon = IsStreamProcessorHaveData(CCompressionStream::eRead);
+            sp->m_Processor->End(abandon);
+            sp->m_State = CSP::eDone;
+        }
+        // Write processor
+        sp = GetStreamProcessor(CCompressionStream::eWrite);
+        if ( sp ) {
+            if ( sp->m_State == CSP::eInit  ||
+                 sp->m_State == CSP::eActive ) {
+                Finalize(CCompressionStream::eWrite);
+                if ( sp->m_LastStatus == CP::eStatus_Overflow ) {
+                    ERR_COMPRESS(72, msg_where << msg_overflow);
+                }
+                if ( sp->m_LastStatus == CP::eStatus_Error ) {
+                    ERR_COMPRESS(73, msg_where << msg_error);
+                }
             }
-            if ( sp->m_LastStatus == CP::eStatus_Error ) {
-                ERR_COMPRESS(73, msg_where << msg_error);
+            if (IsStreamProcessorHaveData(CCompressionStream::eWrite)) {
+                sp->m_Processor->End(); 
+                sp->m_State = CSP::eDone;
+                // Write remaining data from buffers to underlying stream
+                WriteOutBufToStream(true /*force write*/);
+            } else {
+                sp->m_Processor->End(1 /*abandon state*/); 
+                sp->m_State = CSP::eDone;
             }
         }
-        sp->m_Processor->End();
-        sp->m_State = CSP::eDone;
-        // Write remaining data from buffers to underlying stream
-        WriteOutBufToStream(true /*force write*/);
     }
+    COMPRESS_HANDLE_EXCEPTIONS(86, msg_where);
+
     // Delete buffers
     delete[] m_Buf;
 }
@@ -152,7 +161,7 @@ int CCompressionStreambuf::sync()
     if ( !IsOkay() ) {
         return -1;
     }
-	int status = 0;
+    int status = 0;
     // Sync write processor buffers
     CSP* sp = GetStreamProcessor(CCompressionStream::eWrite);
     if ( sp  &&  
@@ -171,21 +180,21 @@ int CCompressionStreambuf::sync()
 
 int CCompressionStreambuf::Sync(CCompressionStream::EDirection dir)
 {
-    // Check processor status
+    // Check processor's state
     if ( !IsStreamProcessorOkay(dir) ) {
         return -1;
     }
+    CSP* sp = GetStreamProcessor(dir);
+    // Check processor's status
+    if ( sp->m_LastStatus == CP::eStatus_Error ) {
+        return -1;
+    }
     // Check that we have some data to process, before calling 
     // any compression/decompression methods, or we can get 
     // a garbage on the output.
-    if ( !IsStreamProcessorHaveData(dir) ) {
+    if ( !IsStreamProcessorHaveData(dir)  &&  sp->m_LastStatus != CP::eStatus_Overflow ) {
         return 0;
     }
-    CSP* sp = GetStreamProcessor(dir);
-    // Check processor status
-    if ( sp->m_LastStatus == CP::eStatus_Error) {
-        return -1;
-    }
     // Process remaining data in the preprocessing buffer
     if ( !Process(dir) ) {
         return -1;
@@ -197,12 +206,11 @@ int CCompressionStreambuf::Sync(CCompressionStream::EDirection dir)
 
 int CCompressionStreambuf::Finish(CCompressionStream::EDirection dir)
 {
-    // Check processor status
+    // Check processor's state
     if ( !IsStreamProcessorOkay(dir) ) {
         return -1;
     }
     CSP* sp = GetStreamProcessor(dir);
-    // Check processor status
     if ( sp->m_LastStatus == CP::eStatus_Error ) {
         return -1;
     }
@@ -210,10 +218,15 @@ int CCompressionStreambuf::Finish(CCompressionStream::EDirection dir)
         // Already finalized
         return 0;
     }
-    // Process remaining data in the preprocessing buffer
-    Process(dir);
-    if ( sp->m_LastStatus == CP::eStatus_Error ) {
-        return -1;
+    // Check that we have some data to process, before calling 
+    // any compression/decompression methods, or we can get 
+    // a garbage in the output.
+    if ( IsStreamProcessorHaveData(dir) ) {
+        // Process remaining data in the preprocessing buffer
+        Process(dir);
+        if ( sp->m_LastStatus == CP::eStatus_Error ) {
+            return -1;
+        }
     }
     // Finish. Change state to 'finalized'.
     sp->m_State = CSP::eFinalize;
@@ -225,7 +238,7 @@ int CCompressionStreambuf::Flush(CCompressionStream::EDirection dir)
 {
     CSP* sp = GetStreamProcessor(dir);
 
-    // Check processor status
+    // Check processor's status
     if ( sp->m_LastStatus == CP::eStatus_Error ) {
         return -1;
     }
@@ -262,7 +275,7 @@ int CCompressionStreambuf::Flush(CCompressionStream::EDirection dir)
             _VERIFY(sp->m_State == CSP::eActive);
             sp->m_LastStatus = 
                 sp->m_Processor->Flush(buf, out_size, &out_avail);
-            // No more data -- automaticaly finalize stream
+            // No more data -- automatically finalize stream
             if ( sp->m_LastStatus == CP::eStatus_EndOfData ) {
                 sp->m_State = CSP::eFinalize;
             }
@@ -284,12 +297,14 @@ int CCompressionStreambuf::Flush(CCompressionStream::EDirection dir)
             }
         }
     } while (sp->m_LastStatus == CP::eStatus_Repeat  ||
-            (out_avail  &&  sp->m_LastStatus == CP::eStatus_Overflow));
+            (out_avail  &&  (sp->m_LastStatus == CP::eStatus_Success || 
+                             sp->m_LastStatus == CP::eStatus_Overflow))
+            );
 
     // Flush underlying stream (on write)
     if (dir == CCompressionStream::eWrite) {
         if ( sp->m_LastStatus == CP::eStatus_EndOfData  ||
-            (sp->m_State == CSP::eFinalize && !out_avail)) {
+             sp->m_State == CSP::eFinalize) {
             if ( !WriteOutBufToStream(true /*force write*/) ) {
                 return -1;
             }
@@ -301,7 +316,7 @@ int CCompressionStreambuf::Flush(CCompressionStream::EDirection dir)
 
 CT_INT_TYPE CCompressionStreambuf::overflow(CT_INT_TYPE c)
 {
-    // Check processor status
+    // Check processor's state
     if ( !IsStreamProcessorOkay(CCompressionStream::eWrite) ) {
         return CT_EOF;
     }
@@ -313,7 +328,7 @@ CT_INT_TYPE CCompressionStreambuf::overflow(CT_INT_TYPE c)
         // (this function is called when pptr() == eptr() but we
         // have reserved one byte more in the constructor, thus
         // *epptr() and now *pptr() point to valid positions)
-        *pptr() = c;
+        *pptr() = (CT_CHAR_TYPE)c;
         // Increment put pointer
         pbump(1);
     }
@@ -326,11 +341,10 @@ CT_INT_TYPE CCompressionStreambuf::overflow(CT_INT_TYPE c)
 
 CT_INT_TYPE CCompressionStreambuf::underflow(void)
 {
-    // Check processor status
+    // Check processor's state
     if ( !IsStreamProcessorOkay(CCompressionStream::eRead) ) {
         return CT_EOF;
     }
-
     // Reset pointer to the processed data
     setg(m_Reader->m_OutBuf, m_Reader->m_OutBuf, m_Reader->m_OutBuf);
 
@@ -371,9 +385,14 @@ bool CCompressionStreambuf::ProcessStreamRead()
             if ( m_Reader->m_Begin == m_Reader->m_End ) {
                 n_read = m_Stream->rdbuf()->sgetn(m_Reader->m_InBuf,
                                                   m_Reader->m_InBufSize);
+#ifdef NCBI_COMPILER_WORKSHOP
+                if (n_read < 0) {
+                    n_read = 0; // WS6 is known to return -1 from sgetn() :-/
+                }
+#endif //NCBI_COMPILER_WORKSHOP
                 if ( !n_read ) {
                     // We can't read more of data.
-                    // Automaticaly 'finalize' (de)compressor.
+                    // Automatically 'finalize' (de)compressor.
                     m_Reader->m_State = CSP::eFinalize;
                     return Flush(CCompressionStream::eRead) == 0;
                 }
@@ -403,7 +422,7 @@ bool CCompressionStreambuf::ProcessStreamRead()
         if ( m_Reader->m_LastStatus == CP::eStatus_Error ) {
             return false;
         }
-        // No more data -- automaticaly finalize stream
+        // No more data -- automatically finalize stream
         if ( m_Reader->m_LastStatus == CP::eStatus_EndOfData ) {
             m_Reader->m_State = CSP::eFinalize;
         }
@@ -425,15 +444,15 @@ bool CCompressionStreambuf::ProcessStreamRead()
 
 bool CCompressionStreambuf::ProcessStreamWrite()
 {
-    const char*      in_buf    = pbase();
-    const streamsize count     = pptr() - pbase();
-    size_t           in_avail  = count;
+    const char*  in_buf    = pbase();
+    const size_t count     = pptr() - pbase();
+    size_t       in_avail  = count;
 
     // Nothing was written into processor yet
     if ( m_Writer->m_State == CSP::eInit ) {
         if ( !count )
             return false;
-        // Reset stato to eActive
+        // Reset state to eActive
         m_Writer->m_State = CSP::eActive;
     }
     // End of stream has been detected
@@ -449,8 +468,8 @@ bool CCompressionStreambuf::ProcessStreamWrite()
     while ( in_avail ) {
         // Process next data portion
         size_t out_avail = 0;
-        streamsize out_size = m_Writer->m_OutBuf + 
-                              m_Writer->m_OutBufSize - m_Writer->m_End;
+        size_t out_size = m_Writer->m_OutBuf + 
+                          m_Writer->m_OutBufSize - m_Writer->m_End;
         m_Writer->m_LastStatus = m_Writer->m_Processor->Process(
             in_buf + count - in_avail, in_avail, m_Writer->m_End, out_size,
             &in_avail, &out_avail);
@@ -459,7 +478,7 @@ bool CCompressionStreambuf::ProcessStreamWrite()
         if ( m_Writer->m_LastStatus == CP::eStatus_Error ) {
             return false;
         }
-        // No more data -- automaticaly finalize stream
+        // No more data -- automatically finalize stream
         if ( m_Writer->m_LastStatus == CP::eStatus_EndOfData ) {
             m_Writer->m_State = CSP::eFinalize;
         }
@@ -489,6 +508,9 @@ bool CCompressionStreambuf::WriteOutBufToStream(bool force_write)
 
         streamsize to_write = m_Writer->m_End - m_Writer->m_Begin;
         if ( to_write ) {
+            if ( !m_Stream->good() ) {
+                return false;
+            }
             streamsize n_write = m_Stream->rdbuf()->sputn(m_Writer->m_Begin, to_write);
             if ( n_write != to_write ) {
                 m_Writer->m_Begin += n_write;
@@ -506,7 +528,7 @@ bool CCompressionStreambuf::WriteOutBufToStream(bool force_write)
 streamsize CCompressionStreambuf::xsputn(const CT_CHAR_TYPE* buf,
                                          streamsize count)
 {
-    // Check processor status
+    // Check processor's state
     if ( !IsStreamProcessorOkay(CCompressionStream::eWrite) ) {
         return CT_EOF;
     }
