@@ -1,4 +1,4 @@
-/*  $Id: format_guess.cpp 411734 2013-08-30 18:18:01Z rafanovi $
+/*  $Id: format_guess.cpp 497436 2016-04-06 17:56:51Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -65,8 +65,24 @@ static unsigned char symbol_type_table[256];
 //  ----------------------------------------------------------------------------
 static bool s_IsTokenPosInt(
     const string& strToken )
+    //  ----------------------------------------------------------------------------
 {
-    return ( -1 != NStr::StringToNonNegativeInt( strToken ) );
+    size_t tokenSize = strToken.size();
+    if (tokenSize == 0) {
+        return false;
+    }
+    if (tokenSize == 1  &&  strToken[0] == '0') {
+        return true;
+    }
+    if (strToken[0] < '1'  ||  '9' < strToken[0]) {
+        return false;
+    }
+    for (size_t i=1; i<tokenSize; ++i) {
+        if (strToken[i] < '0'  ||  '9' < strToken[i]) {
+            return false;
+        }
+    }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
@@ -74,7 +90,7 @@ static bool s_IsTokenInteger(
     const string& strToken )
 //  ----------------------------------------------------------------------------
 {
-    if ( ! strToken.empty() && strToken[0] == '-' ) {
+    if ( ! strToken.empty() && (strToken[0] == '-'  ||  strToken[0] == '+')) {
         return s_IsTokenPosInt( strToken.substr( 1 ) );
     }
     return s_IsTokenPosInt( strToken );
@@ -89,7 +105,10 @@ static bool s_IsTokenDouble(
     if ( token.size() > 1 && token[0] == '-' ) {
         token[0] = '1';
     }
-    return s_IsTokenPosInt( token );
+    if (token.size() > 1 && token[0] == '0') {
+        token[0] = '1';
+    }
+    return s_IsTokenPosInt(token);
 }
 
 //  ----------------------------------------------------------------------------
@@ -97,50 +116,46 @@ static void init_symbol_type_table(void)
 {
     if ( symbol_type_table[0] == 0 ) {
         for ( const char* s = "ACGNTU"; *s; ++s ) {
-            unsigned char c = *s;
+            int c = *s;
             symbol_type_table[c] |= fDNA_Main_Alphabet;
             c = tolower(c);
             symbol_type_table[c] |= fDNA_Main_Alphabet;
         }
         for ( const char* s = "BDHKMRSVWY"; *s; ++s ) {
-            unsigned char c = *s;
+            int c = *s;
             symbol_type_table[c] |= fDNA_Ambig_Alphabet;
             c = tolower(c);
             symbol_type_table[c] |= fDNA_Ambig_Alphabet;
         }
         for ( const char* s = "ACDEFGHIKLMNPQRSTVWYBZX"; *s; ++s ) {
-            unsigned char c = *s;
+            int c = *s;
             symbol_type_table[c] |= fProtein_Alphabet;
             c = tolower(c);
             symbol_type_table[c] |= fProtein_Alphabet;
         }
-        symbol_type_table[(unsigned char)'-']
-            |= fDNA_Main_Alphabet | fProtein_Alphabet;
-        symbol_type_table[(unsigned char)'*'] |= fProtein_Alphabet;
+        symbol_type_table[(int)'-'] |= fDNA_Main_Alphabet | fProtein_Alphabet;
+        symbol_type_table[(int)'*'] |= fProtein_Alphabet;
         for ( const char* s = "\r\n"; *s; ++s ) {
-            unsigned char c = *s;
+            int c = *s;
             symbol_type_table[c] |= fLineEnd;
         }
         for ( int c = 1; c < 256; ++c ) {
-            if ( isalpha(c) )
+            if ( isalpha((unsigned char)c) )
                 symbol_type_table[c] |= fAlpha;
-            if ( isdigit(c) )
+            if ( isdigit((unsigned char)c) )
                 symbol_type_table[c] |= fDigit;
-            if ( isspace(c) )
+            if ( isspace((unsigned char)c) )
                 symbol_type_table[c] |= fSpace;
         }
         symbol_type_table[0] |= fInvalid;
     }
 }
 
-//  ----------------------------------------------------------------------------
-int
-CFormatGuess::s_CheckOrder[] =
-//  ----------------------------------------------------------------------------
+
+// Must list all EFormats except eUnknown and eFormat_max. 
+// Will cause assertion if violated!
+int CFormatGuess::s_CheckOrder[] =
 {
-    //  must list all EFormats except eUnknown and eFormat_max. Will cause
-    //  assertion if violated!
-    //
     eBam, // must precede eGZip!
     eZip,
     eGZip,
@@ -149,17 +164,17 @@ CFormatGuess::s_CheckOrder[] =
     eSra,
     eRmo,
     eVcf,
-    eGtf,
     eGvf,
     eGff3,
+    eGtf,
     eGff2,
     eGlimmer3,
     eAgp,
     eXml,
     eWiggle,
+    eNewick,
     eBed,
     eBed15,
-    eNewick,
     eHgvs,
     eAlignment,
     eDistanceMatrix,
@@ -172,13 +187,15 @@ CFormatGuess::s_CheckOrder[] =
     ePhrapAce,
     eTable,
     eBinaryASN,
+    eUCSCRegion,
 };
 
 
 // This array must stay in sync with enum EFormat, but that's not
 // supposed to change in the middle anyway, so the explicit size
 // should suffice to avoid accidental skew.
-const char* const CFormatGuess::sm_FormatNames[CFormatGuess::eFormat_max] = {
+const char* const CFormatGuess::sm_FormatNames[CFormatGuess::eFormat_max] =
+{
     "unknown",
     "binary ASN.1",
     "RepeatMasker",
@@ -212,6 +229,7 @@ const char* const CFormatGuess::sm_FormatNames[CFormatGuess::eFormat_max] = {
     "SRA",
     "BAM",
     "VCF",
+    "UCSC Region"
 };
 
 const char*
@@ -372,12 +390,12 @@ CFormatGuess::GuessFormat(
         return eUnknown;
     }
     EMode mode = eQuick;
-    unsigned int uFormatCount = sizeof( s_CheckOrder ) / sizeof( int );
+    size_t uFormatCount = ArraySize(s_CheckOrder);
 
     // First, try to use hints
     if ( !m_Hints.IsEmpty() ) {
-        for (unsigned int f = 0; f < uFormatCount; ++f) {
-            EFormat fmt = EFormat( s_CheckOrder[ f ] );
+        for (size_t f = 0; f < uFormatCount; ++f) {
+            EFormat fmt = EFormat( s_CheckOrder[f] );
             if (m_Hints.IsPreferred(fmt)  &&  x_TestFormat(fmt, mode)) {
                 return fmt;
             }
@@ -385,8 +403,8 @@ CFormatGuess::GuessFormat(
     }
 
     // Check other formats, skip the ones that are disabled through hints
-    for (unsigned int f = 0; f < uFormatCount; ++f) {
-        EFormat fmt = EFormat( s_CheckOrder[ f ] );
+    for (size_t f = 0; f < uFormatCount; ++f) {
+        EFormat fmt = EFormat( s_CheckOrder[f] );
         if ( ! m_Hints.IsDisabled(fmt)  &&  x_TestFormat(fmt, mode) ) {
             return fmt;
         }
@@ -402,8 +420,7 @@ CFormatGuess::TestFormat( EFormat format, EMode )
 }
 
 //  ----------------------------------------------------------------------------
-bool
-CFormatGuess::TestFormat(
+bool CFormatGuess::TestFormat(
     EFormat format,
     EOnError onerror )
 {
@@ -442,14 +459,14 @@ bool CFormatGuess::x_TestFormat(EFormat format, EMode mode)
         return TestFormatAgp( mode );
     case eXml:
         return TestFormatXml( mode );
+    case eNewick:
+        return TestFormatNewick( mode );
     case eWiggle:
         return TestFormatWiggle( mode );
     case eBed:
         return TestFormatBed( mode );
     case eBed15:
         return TestFormatBed15( mode );
-    case eNewick:
-        return TestFormatNewick( mode );
     case eAlignment:
         return TestFormatAlignment( mode );
     case eDistanceMatrix:
@@ -486,9 +503,12 @@ bool CFormatGuess::x_TestFormat(EFormat format, EMode mode)
         return TestFormatBam( mode );
     case eVcf:
         return TestFormatVcf( mode );
+    case eUCSCRegion:
+        return false;
     default:
         NCBI_THROW( CCoreException, eInvalidArg,
-            "CFormatGuess::x_TestFormat(): Unsupported format ID." );
+            "CFormatGuess::x_TestFormat(): Unsupported format ID (" +
+            NStr::NumericToString((int)format) + ")." );
     }
 }
 
@@ -511,6 +531,7 @@ CFormatGuess::Initialize()
     m_iStatsCountAlNumChars = 0;
     m_iStatsCountDnaChars = 0;
     m_iStatsCountAaChars = 0;
+    m_iStatsCountBraces = 0;
 }
 
 //  ----------------------------------------------------------------------------
@@ -583,6 +604,7 @@ CFormatGuess::EnsureStats()
     //     from the DNA alphabet
     //   m_iStatsCountAaChars: number of characters counted in m_iStatsCountData
     //     from the AA alphabet
+    //  m_iStatsCountBraces: Opening { and closing } braces
     //
     while ( ! TestBuffer.fail() ) {
         NcbiGetlineEOL( TestBuffer, strLine );
@@ -600,6 +622,9 @@ CFormatGuess::EnsureStats()
             if ( type & (fAlpha | fDigit | fSpace) ) {
                 ++m_iStatsCountAlNumChars;
             }
+            else if (c == '{'  ||  c == '}') {
+                ++m_iStatsCountBraces;
+            }
             if ( !is_header ) {
                 if ( !(type & fSpace) ) {
                     ++m_iStatsCountData;
@@ -635,8 +660,7 @@ bool CFormatGuess::x_TestInput( CNcbiIstream& input, EOnError onerror )
 }
 
 //  ----------------------------------------------------------------------------
-bool
-CFormatGuess::TestFormatRepeatMasker(
+bool CFormatGuess::TestFormatRepeatMasker(
     EMode /* not used */ )
 {
     if ( ! EnsureStats() || ! EnsureSplitLines() ) {
@@ -752,6 +776,9 @@ CFormatGuess::TestFormatGff3(
         //  Make sure to ignore any UCSC track and browser lines prior to the
         //  start of data
         //
+        if (!uGffLineCount && NStr::StartsWith(*it, "##gff-version 3")) {
+            return true;
+        }
         if ( it->empty() || (*it)[0] == '#' ) {
             continue;
         }
@@ -843,7 +870,11 @@ CFormatGuess::TestFormatAgp(
         return false;
     }
     ITERATE( list<string>, it, m_TestLines ) {
-        if ( !IsLineAgp( *it ) ) {
+        try {
+            if ( !IsLineAgp( *it ) ) {
+                return false;
+            }
+        } catch(...) {
             return false;
         }
     }
@@ -856,6 +887,74 @@ CFormatGuess::TestFormatNewick(
     EMode /* not used */ )
 {
 //  -----------------------------------------------------------------------------
+    // newick trees can be found in nexus files. check for that first as a special case
+    if ( ! EnsureTestBuffer() || ! EnsureSplitLines() ) {
+        const int BUFFSIZE = 8096;
+        m_pTestBuffer = new char[BUFFSIZE+1];
+        m_Stream.read( m_pTestBuffer, BUFFSIZE );
+        m_iTestDataSize = m_Stream.gcount();
+        m_pTestBuffer[m_iTestDataSize] = 0;
+        m_Stream.clear();  // in case we reached eof
+        CStreamUtils::Stepback( m_Stream, m_pTestBuffer, m_iTestDataSize );
+        m_TestLines.push_back(m_pTestBuffer);
+    }
+
+    // Note: We can live with false negatives. Avoid false positives
+    // at all cost.
+
+    bool is_nexus = false;
+    bool has_trees = false;
+    const size_t check_size = 12;
+
+    ITERATE( list<string>, it, m_TestLines ) {
+        if ( NPOS != it->find( "#NEXUS" ) ) {
+            is_nexus = true;
+        }
+    }
+
+    // Trees can be anywhere in a nexus file.  If nexus is true, 
+    // try to read the whole file to see if there is a tree.
+    if (is_nexus) {
+        // Read in file one chunk at a time.  Readline would be better
+        // but is not avialable for this stream.  Since the text we 
+        // are looking for "begin trees;" may span two chunks, we
+        // copy the last 12 characters of the previous chunk to
+        // the front of the new one.
+        const size_t read_size = 16384;
+        char test_buf[read_size + check_size + 1];
+        memset(test_buf, ' ', check_size); // "previous chunk" initially blank.
+
+        size_t max_reads = 32768;  // max read to locate tree: 512 MB
+        for (size_t i = 0; i < max_reads; ++i) {
+            m_Stream.read(test_buf+check_size, read_size);
+            size_t num_read = m_Stream.gcount();
+            if (num_read > 0) {
+                test_buf[num_read + check_size] = 0; // null terminator
+                if (NPOS != NStr::FindNoCase(CTempString(test_buf), "begin trees;")) {
+                    has_trees = true;
+                    m_Stream.clear();  // in case we reached eof
+                    break;
+                }
+                // copy end of buffer to beginning in case string
+                // spans two buffers:
+                strncpy(test_buf, test_buf + num_read, check_size);
+            }
+
+            if (m_Stream.eof() || m_Stream.fail()) {
+                m_Stream.clear();  // clear eof
+                break;
+            }
+        }
+    }
+
+    // In a nexus file with a tree, we will just read in the tree (ignoring for now
+    // the alignment)
+    if (is_nexus ) {
+        if (has_trees)
+            return true;
+        return false;
+    }
+
     //  special newick consideration:
     //  newick files may come with all data cramped into a single run-on line,
     //  that single oversized line may not have a line terminator
@@ -923,7 +1022,7 @@ CFormatGuess::TestFormatDistanceMatrix(
     list<string> toks;
 
     /// first line: one token, one number
-    NStr::Split(*iter++, "\t ", toks);
+    NStr::Split(*iter++, "\t ", toks, NStr::fSplit_Tokenize);
     if (toks.size() != 1  ||
         toks.front().find_first_not_of("0123456789") != string::npos) {
         return false;
@@ -934,7 +1033,7 @@ CFormatGuess::TestFormatDistanceMatrix(
     // line, the number of values should increase monotonically
     for (size_t i = 1;  iter != m_TestLines.end();  ++i, ++iter) {
         toks.clear();
-        NStr::Split(*iter, "\t ", toks);
+        NStr::Split(*iter, "\t ", toks, NStr::fSplit_Tokenize);
         if (toks.size() != i) {
             /// we can ignore the last line ; it may be truncated
             list<string>::const_iterator it = iter;
@@ -1028,9 +1127,7 @@ CFormatGuess::TestFormatXml(
     static const char* known_types[] = {
         "<Blast4-request>"
     };
-    const int num_types = sizeof( known_types ) / sizeof( const char* );
-
-    for ( int i=0; i < num_types; ++i ) {
+    for ( size_t i=0; i < ArraySize(known_types); ++i ) {
         if ( NStr::StartsWith( input, known_types[i], NStr::eCase ) ) {
             return true;
         }
@@ -1076,18 +1173,12 @@ CFormatGuess::TestFormatAlignment(
     list<string>::const_iterator iter = m_TestLines.begin();
     list<string> toks;
 
-    // Merge delims if > 1.  Do not merge single delims (since they could 
-    // more easily represent blank fields
-    NStr::EMergeDelims  merge_delims = NStr::eMergeDelims;
-    if (delims.size() == 1)
-        merge_delims = NStr::eNoMergeDelims;
-
-
     // Skip initial lines since not all headers start with comments like # or ;:
     // Don't skip though if file is very short - add up to 3, 1 for each line 
     // over 5:
     for (size_t i=5; i<7; ++i)
-        if (m_TestLines.size() > i) ++iter;
+        if (m_TestLines.size() > i) 
+            ++iter;
 
     /// determine the number of observed columns
     size_t ncols = 0;
@@ -1098,7 +1189,7 @@ CFormatGuess::TestFormatAlignment(
         }
 
         toks.clear();
-        NStr::Split(*iter, delims, toks);
+        NStr::Split(*iter, delims, toks, NStr::fSplit_Tokenize);
         ncols = toks.size();
         found = true;
     }
@@ -1115,7 +1206,7 @@ CFormatGuess::TestFormatAlignment(
         } 
 
         toks.clear();
-        NStr::Split(*iter, delims, toks);
+        NStr::Split(*iter, delims, toks, NStr::fSplit_Tokenize);
         if (toks.size() != ncols) {
             list<string>::const_iterator it = iter;
             ++it;
@@ -1126,7 +1217,7 @@ CFormatGuess::TestFormatAlignment(
             ++nlines;
         }
     }
-    return ( nlines >= 2 );
+    return ( nlines >= 3 );
  }
 
 bool
@@ -1136,6 +1227,9 @@ CFormatGuess::TestFormatTable(
     if ( ! EnsureTestBuffer() || ! EnsureSplitLines() ) {
         return false;
     }
+    if ( ! IsAsciiText()) {//gp-13007:  "table" means "ascii table"
+        return false;
+    }
 
     //
     //  NOTE 1:
@@ -1171,16 +1265,32 @@ CFormatGuess::TestFormatTable(
 }
 
 //  -----------------------------------------------------------------------------
+void SkipCommentAndBlank(CTempString &text)
+{
+	const CTempString COMMENT_SYMBOLS(";#!");
+	const CTempString NEW_LINE_SYMBOLS("\r\n");
+	while (true) {
+		text = NStr::TruncateSpaces_Unsafe(text, NStr::eTrunc_Begin);
+		if ( COMMENT_SYMBOLS.find(text[0]) != CTempString::npos ) {
+			CTempString::size_type pos = text.find_first_of(NEW_LINE_SYMBOLS, 1);
+			text = text.substr(pos);
+		} else {
+			break;
+		}
+	}
+}
+
 bool
-CFormatGuess::TestFormatFasta(
-    EMode /* not used */ )
+CFormatGuess::TestFormatFasta( EMode /* not used */ )
 {
     if ( ! EnsureStats() ) {
         return false;
     }
 
     // reject obvious misfits:
-    if ( m_iTestDataSize == 0 || m_pTestBuffer[0] != '>' ) {
+	CTempString header(m_pTestBuffer, m_iTestDataSize);
+	SkipCommentAndBlank(header);
+	if ( m_iTestDataSize == 0 || header.length() == 0 || header[0] != '>' ) {
         return false;
     }
     if ( m_iStatsCountData == 0 ) {
@@ -1191,9 +1301,9 @@ CFormatGuess::TestFormatFasta(
     }
 
     // remaining decision based on text stats:
-    double dAlNumFraction =  (double)m_iStatsCountAlNumChars / m_iTestDataSize;
-    double dDnaFraction = (double)m_iStatsCountDnaChars / m_iStatsCountData;
-    double dAaFraction = (double)m_iStatsCountAaChars / m_iStatsCountData;
+    double dAlNumFraction = (double)m_iStatsCountAlNumChars / (double)m_iTestDataSize;
+    double dDnaFraction   = (double)m_iStatsCountDnaChars / (double)m_iStatsCountData;
+    double dAaFraction    = (double)m_iStatsCountAaChars / (double)m_iStatsCountData;
 
     // want at least 80% text-ish overall:
     if ( dAlNumFraction < 0.8 ) {
@@ -1226,7 +1336,8 @@ CFormatGuess::TestFormatTextAsn(
     // at least 80% text-ish,
     // "::=" as the 2nd field of the first non-blank non comment line.
     //
-    double dAlNumFraction =  (double)m_iStatsCountAlNumChars / m_iTestDataSize;
+    double dAlNumFraction = (double)(m_iStatsCountAlNumChars+m_iStatsCountBraces) /
+                            (double)m_iTestDataSize;
     if ( dAlNumFraction < 0.80 ) {
         return false;
     }
@@ -1237,8 +1348,8 @@ CFormatGuess::TestFormatTextAsn(
 
     while ( ! TestBuffer.fail() ) {
         vector<string> Fields;
-        NcbiGetline( TestBuffer, strLine, "\n\r" );
-        NStr::Tokenize( strLine, " \t", Fields, NStr::eMergeDelims );
+        NcbiGetline(TestBuffer, strLine, "\n\r");
+        NStr::Split(strLine, " \t", Fields, NStr::fSplit_Tokenize);
         if ( IsAsnComment( Fields  ) ) {
             continue;
         }
@@ -1315,7 +1426,7 @@ CFormatGuess::TestFormatBed(
         }
 
         vector<string> columns;
-        NStr::Tokenize( str, " \t", columns, NStr::eMergeDelims );
+        NStr::Split(str, " \t", columns, NStr::fSplit_Tokenize);
         if (columns.size() < 3 || columns.size() > 12) {
             return false;
         }
@@ -1368,7 +1479,7 @@ CFormatGuess::TestFormatBed15(
         }
 
         vector<string> columns;
-        NStr::Tokenize( *it, " \t", columns, NStr::eMergeDelims );
+        NStr::Split(*it, " \t", columns, NStr::fSplit_Tokenize);
         if ( columns.size() != columncount ) {
             return false;
         } else {
@@ -1426,8 +1537,15 @@ bool
 CFormatGuess::TestFormatHgvs(
     EMode /* not used */ )
 {
-    if ( ! EnsureTestBuffer() || ! EnsureSplitLines() ) {
-        return false;
+    if ( ! EnsureStats() || ! EnsureSplitLines() ) {
+        const int BUFFSIZE = 1024;
+        m_pTestBuffer = new char[BUFFSIZE+1];
+        m_Stream.read( m_pTestBuffer, BUFFSIZE );
+        m_iTestDataSize = m_Stream.gcount();
+        m_pTestBuffer[m_iTestDataSize] = 0;
+        m_Stream.clear();  // in case we reached eof
+        CStreamUtils::Stepback( m_Stream, m_pTestBuffer, m_iTestDataSize );
+        m_TestLines.push_back(m_pTestBuffer);
     }
 
     unsigned int uHgvsLineCount = 0;
@@ -1897,7 +2015,7 @@ bool CFormatGuess::IsLineAgp(
     }
 
     vector<string> tokens;
-    if ( NStr::Tokenize( line, " \t", tokens, NStr::eMergeDelims ).size() < 8 ) {
+    if ( NStr::Split(line, " \t", tokens, NStr::fSplit_Tokenize).size() < 8 ) {
         return false;
     }
 
@@ -1954,7 +2072,7 @@ bool CFormatGuess::IsLineGlimmer3(
     const string& line )
 {
     list<string> toks;
-    NStr::Split(line, "\t ", toks);
+    NStr::Split(line, "\t ", toks, NStr::fSplit_Tokenize);
     if (toks.size() != 5) {
         return false;
     }
@@ -1995,7 +2113,7 @@ bool CFormatGuess::IsLineGtf(
     const string& line )
 {
     vector<string> tokens;
-    if ( NStr::Tokenize( line, " \t", tokens, NStr::eMergeDelims ).size() < 8 ) {
+    if ( NStr::Split(line, " \t", tokens, NStr::fSplit_Tokenize).size() < 8 ) {
         return false;
     }
     if ( ! s_IsTokenPosInt( tokens[3] ) ) {
@@ -2024,9 +2142,10 @@ bool CFormatGuess::IsLineGtf(
 //  ----------------------------------------------------------------------------
 bool CFormatGuess::IsLineGvf(
     const string& line )
+//  ----------------------------------------------------------------------------
 {
     vector<string> tokens;
-    if ( NStr::Tokenize( line, " \t", tokens, NStr::eMergeDelims ).size() < 8 ) {
+    if ( NStr::Split(line, " \t", tokens, NStr::fSplit_Tokenize).size() < 8 ) {
         return false;
     }
     if ( ! s_IsTokenPosInt( tokens[3] ) ) {
@@ -2035,7 +2154,10 @@ bool CFormatGuess::IsLineGvf(
     if ( ! s_IsTokenPosInt( tokens[4] ) ) {
         return false;
     }
+
+    //make sure that "type" is a GVF admissible value:
 	{{
+        bool typeOk = false;
 		list<string> terms;
 		terms.push_back("snv");
 		terms.push_back("cnv");
@@ -2063,9 +2185,14 @@ bool CFormatGuess::IsLineGvf(
 		terms.push_back("structural_variation");
 		terms.push_back("sequence_alteration");
 		ITERATE(list<string>, termiter, terms) {
-			if(NStr::EqualNocase(*termiter, tokens[2]))
-				return true;
+			if(NStr::EqualNocase(*termiter, tokens[2])) {
+				typeOk = true;
+                break;
+            }
 		}
+        if (!typeOk) {
+            return false;
+        }
 	}}
 	if ( ! s_IsTokenDouble( tokens[5] ) ) {
         return false;
@@ -2076,19 +2203,15 @@ bool CFormatGuess::IsLineGvf(
     if ( tokens[7].size() != 1 || NPOS == tokens[7].find_first_of( ".0123" ) ) {
         return false;
     }
-	if(tokens.size() >= 9) {
-		list<string> terms;
-		terms.push_back("start_range");
-		terms.push_back("end_range");
-		terms.push_back("variant_seq");
-		terms.push_back("genotype");
-		ITERATE(list<string>, termiter, terms) {
-			if(NStr::EqualNocase(*termiter, tokens[8]))
-				return true;
-		}
-	}
 
-    return false;
+    //make sure all the mandatory attributes are present:
+    string attrs = tokens[8];
+    if (string::npos == attrs.find("ID="))
+        return false;
+    if (string::npos == attrs.find("Variant_seq=")) {
+        return false;
+    }
+    return true;
 }
 
 
@@ -2097,7 +2220,7 @@ bool CFormatGuess::IsLineGff3(
     const string& line )
 {
     vector<string> tokens;
-    if ( NStr::Tokenize( line, " \t", tokens, NStr::eMergeDelims ).size() < 8 ) {
+    if ( NStr::Split(line, " \t", tokens, NStr::fSplit_Tokenize).size() < 8 ) {
         return false;
     }
     if ( ! s_IsTokenPosInt( tokens[3] ) ) {
@@ -2109,7 +2232,7 @@ bool CFormatGuess::IsLineGff3(
     if ( ! s_IsTokenDouble( tokens[5] ) ) {
         return false;
     }
-    if ( tokens[6].size() != 1 || NPOS == tokens[6].find_first_of( ".+-" ) ) {
+    if ( tokens[6].size() != 1 || NPOS == tokens[6].find_first_of( ".+-?" ) ) {
         return false;
     }
     if ( tokens[7].size() != 1 || NPOS == tokens[7].find_first_of( ".0123" ) ) {
@@ -2120,14 +2243,14 @@ bool CFormatGuess::IsLineGff3(
     }
     if ( tokens.size() >= 9 && tokens[8].size() > 1) {
         const string& col9 = tokens[8];
-        if ( NPOS == NStr::FindNoCase(col9, "ID") &&
-             NPOS == NStr::FindNoCase(col9, "Parent") &&
-             NPOS == NStr::FindNoCase(col9, "Target") &&
-             NPOS == NStr::FindNoCase(col9, "Name") &&
-             NPOS == NStr::FindNoCase(col9, "Alias") &&
-             NPOS == NStr::FindNoCase(col9, "Note") &&
-             NPOS == NStr::FindNoCase(col9, "Dbxref") &&
-             NPOS == NStr::FindNoCase(col9, "Xref") ) {
+        if ( NPOS == NStr::Find(col9, "ID") &&
+             NPOS == NStr::Find(col9, "Parent") &&
+             NPOS == NStr::Find(col9, "Target") &&
+             NPOS == NStr::Find(col9, "Name") &&
+             NPOS == NStr::Find(col9, "Alias") &&
+             NPOS == NStr::Find(col9, "Note") &&
+             NPOS == NStr::Find(col9, "Dbxref") &&
+             NPOS == NStr::Find(col9, "Xref") ) {
             return false;
         }
     }
@@ -2141,7 +2264,7 @@ bool CFormatGuess::IsLineGff2(
     const string& line )
 {
     vector<string> tokens;
-    if ( NStr::Tokenize( line, " \t", tokens, NStr::eMergeDelims ).size() < 8 ) {
+    if ( NStr::Split(line, " \t", tokens, NStr::fSplit_Tokenize).size() < 8 ) {
         return false;
     }
     if ( ! s_IsTokenPosInt( tokens[3] ) ) {
@@ -2168,7 +2291,7 @@ bool CFormatGuess::IsLinePhrapId(
     const string& line )
 {
     vector<string> values;
-    if ( NStr::Tokenize( line, " \t", values, NStr::eMergeDelims ).empty() ) {
+    if ( NStr::Split(line, " \t", values, NStr::fSplit_Tokenize).empty() ) {
         return false;
     }
 
@@ -2201,7 +2324,7 @@ bool CFormatGuess::IsLineRmo(
     //  Make sure there is enough stuff on that line:
     //
     list<string> values;
-    if ( NStr::Split( line, " \t", values ).size() < MIN_VALUES_PER_RECORD ) {
+    if ( NStr::Split(line, " \t", values, NStr::fSplit_Tokenize).size() < MIN_VALUES_PER_RECORD ) {
         return false;
     }
 
@@ -2305,14 +2428,14 @@ CFormatGuess::EnsureSplitLines()
     string data( m_pTestBuffer, (size_t)m_iTestDataSize );
     m_TestLines.clear();
 
-    if ( string::npos != data.find( "\r\n" ) ) {
-        NStr::Split( data, "\r\n", m_TestLines );
+    if ( string::npos != data.find("\r\n") ) {
+        NStr::Split(data, "\r\n", m_TestLines, NStr::fSplit_Tokenize);
     }
-    else if ( string::npos != data.find( "\n" ) ) {
-        NStr::Split( data, "\n", m_TestLines );
+    else if ( string::npos != data.find("\n") ) {
+        NStr::Split(data, "\n", m_TestLines, NStr::fSplit_Tokenize);
     }
-    else if ( string::npos != data.find( "\r" ) ) {
-        NStr::Split( data, "\r", m_TestLines );
+    else if ( string::npos != data.find("\r") ) {
+        NStr::Split(data, "\r", m_TestLines, NStr::fSplit_Tokenize);
     }
     else {
         //single truncated line
@@ -2339,7 +2462,7 @@ CFormatGuess::IsAsciiText()
             ++count_print;
         }
     }
-    if (count_print < count * REQUIRED_ASCII_RATIO) {
+    if (count_print < (double)count * REQUIRED_ASCII_RATIO) {
 		return false;
     }
     return true;
@@ -2380,7 +2503,7 @@ bool CFormatGuess::IsLineHgvs(
     const string& line )
 {
     // This simple check can mistake Newwick, so Newwick is checked first
-    //  /:(g|c|r|p|m|mt|n)\./  as in NC_000001.9:g.1234567C>T
+    //  /[:alnum:]+:(g|c|r|p|m|mt|n)\.[:alnum:]+/  as in NC_000001.9:g.1234567C>T
     int State = 0;
     ITERATE(string, Iter, line) {
         char Char = *Iter;
@@ -2391,27 +2514,37 @@ bool CFormatGuess::IsLineHgvs(
             Next = *NextI;
         
         if(State == 0) {
-            if(Char == ':')
-                State = 1;
+            if( isalnum(Char) )
+                State++;
         } else if(State == 1) {
+            if(Char == ':')
+                State++;
+        } else if(State == 2) {
             if (Char == 'g' ||
                 Char == 'c' ||
                 Char == 'r' ||
                 Char == 'p' ||
                 Char == 'n' ||
                 Char == 'm' ) {
-                State = 2;
+                State++;
                 if (Char=='m' && Next == 't') {
                     ++Iter;
                 }
+            } else {
+                return false;
             }
-        } else if(State == 2) {
+        } else if(State == 3) {
             if(Char == '.') 
-                State = 3;
+                State++;
+            else
+                return false;
+        } else if(State == 4) {
+            if( isalnum(Char) )
+                State++;
         }
     }
     
-    return (State == 3);    
+    return (State == 5);    
 }
 
 
