@@ -1,4 +1,4 @@
-/*  $Id: sequtil_convert_imp.cpp 446394 2014-09-15 17:57:40Z ivanov $
+/*  $Id: sequtil_convert_imp.cpp 495186 2016-03-15 16:38:09Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -281,7 +281,7 @@ SIZE_TYPE CSeqConvert_imp::x_ConvertIupacnaTo2na
     if ( length % 4 != 0 ) {
         *dst = 0x0;
         for( size_t i = 0; i < (length % 4); ++i, ++src_i ) {
-            *dst |= table[static_cast<Uint1>(*src_i) * 4 + i];
+            *dst |= (char)table[static_cast<Uint1>(*src_i) * 4 + i];
         }
     }
     
@@ -429,7 +429,7 @@ SIZE_TYPE CSeqConvert_imp::x_Convert2naTo4na
         {{
             *dst = table[static_cast<Uint1>(*iter) * 2 + 1];
             if ( length == 1 ) {
-                *dst &= 0xf0;
+                *dst &= (char)0xf0;
                 return length;
             }
             size -= 2;
@@ -562,23 +562,23 @@ SIZE_TYPE CSeqConvert_imp::x_Convert2naExpandTo2na
     // main loop. pack 4 ncbi2na_expand bytes into a single bye and add it
     // to the output container
     for ( size_t i = length / 4; i; --i, ++dst ) {
-        *dst = (*iter << 6) | (*(iter + 1) << 4) | 
-               (*(iter + 2) << 2) | (*(iter + 3));
+        *dst = char((*iter << 6) | (*(iter + 1) << 4) | 
+                    (*(iter + 2) << 2) | (*(iter + 3)));
         iter += 4;
     }
 
     switch ( length % 4 ) {
     case 1:
-        *dst = (*iter << 6);
+        *dst = char(*iter << 6);
         break;
     case 2:
-        *dst = (*iter << 6) | (*(iter + 1) << 4);
+        *dst = char((*iter << 6) | (*(iter + 1) << 4));
         break;
     case 3:
-        *dst = (*iter << 6) | (*(iter + 1) << 4) | (*(iter + 2) << 2);
+        *dst = char((*iter << 6) | (*(iter + 1) << 4) | (*(iter + 2) << 2));
         break;
     }
-    
+   
     return length;
 }
 
@@ -696,9 +696,9 @@ SIZE_TYPE CSeqConvert_imp::x_Convert4naTo2na
                     *dst = (table[static_cast<Uint1>(*iter) * 2]) & 0xF0;
                     break;
                 case 3:
-                    *dst = 
-                        table[static_cast<Uint1>(*iter) * 2]           |
-                        (table[static_cast<Uint1>(*(iter + 1)) * 2 + 1] & 0xFC);
+                    *dst = (char)(
+                        table[static_cast<Uint1>(*iter) * 2] |
+                        (table[static_cast<Uint1>(*(iter + 1)) * 2 + 1] & 0xFC));
                     break;
                 }
             }
@@ -723,14 +723,14 @@ SIZE_TYPE CSeqConvert_imp::x_Convert4naTo2na
                     *dst = table[static_cast<Uint1>(*iter) * 3] & 0xC0;
                     break;
                 case 2:
-                    *dst = 
+                    *dst = (char)(
                         table[static_cast<Uint1>(*iter) * 3]           |
-                        (table[static_cast<Uint1>(*(iter + 1)) * 3 + 1] & 0xF0);
+                        (table[static_cast<Uint1>(*(iter + 1)) * 3 + 1] & 0xF0));
                     break;
                 case 3:
-                    *dst = 
+                    *dst = (char)(
                         table[static_cast<Uint1>(*iter) * 3]           |
-                        (table[static_cast<Uint1>(*(iter + 1)) * 3 + 1] & 0xFC);
+                        (table[static_cast<Uint1>(*(iter + 1)) * 3 + 1] & 0xFC));
                     break;
                 }
             }
@@ -809,7 +809,7 @@ SIZE_TYPE CSeqConvert_imp::x_Convert8naTo2na
     if ( (length % 4) != 0 ) {
         *dst = 0;
         for( size_t i = 0; i < (length % 4); ++i, ++iter ) {
-            *dst |= table[static_cast<Uint1>(*iter) * 4 + i];
+            *dst |= (char)table[static_cast<Uint1>(*iter)*4 + i];
         }
     }
     
@@ -862,12 +862,12 @@ SIZE_TYPE CSeqConvert_imp::x_Convert8naTo4na
     const char* iter = src + pos;
 
     for ( size_t i = length / 2; i; --i, ++dst ) {
-        *dst = (*iter << 4) | (*(iter + 1));
+        *dst = char((*iter << 4) | (*(iter + 1)));
         iter += 2;
     }
 
     if ( (length % 2) != 0 ) {
-        *dst = (*iter << 4) & 0xf0;
+        *dst = char((*iter << 4) & 0xf0);
     }
     
     return length;
@@ -1012,7 +1012,7 @@ SIZE_TYPE CSeqConvert_imp::Subseq
                 const char* end  = iter + length;
                 
                 for ( ; iter != end; ++iter, ++dst ) {
-                    *dst = toupper((unsigned char)(*iter));
+                    *dst = (char)toupper((unsigned char)(*iter));
                 }
                 converted = length;
             }}
@@ -1115,6 +1115,12 @@ SIZE_TYPE CSeqConvert_imp::Pack(const char* src, TSeqPos length,
 const CSeqUtil::ECoding CSeqConvert_imp::CPacker::kNoCoding
     = CSeqUtil::e_Ncbi2na_expand; // never used by best_coding
 
+CSeqConvert_imp::CPacker::~CPacker()
+{
+    m_EndingNarrow.Reset();
+    m_EndingWide.shared_codings = NULL;
+}
+
 SIZE_TYPE CSeqConvert_imp::CPacker::Pack(const char* src, TSeqPos length)
 {
     const char* src_end   = src + GetBytesNeeded(m_SrcCoding, length);
@@ -1132,13 +1138,13 @@ SIZE_TYPE CSeqConvert_imp::CPacker::Pack(const char* src, TSeqPos length)
             TCoding type1 = m_BestCoding[(residue >> 4)   * 0x11],
                     type2 = m_BestCoding[(residue & 0x0F) * 0x11];
             if (type1 != prev_type) {
-                x_AddBoundary((p - src) * 2, type1);
+                x_AddBoundary(TSeqPos((p - src)*2), type1);
             }
-            x_AddBoundary((p - src) * 2 + 1, type2);
+            x_AddBoundary(TSeqPos((p - src)*2 + 1), type2);
             prev_type = type2;
         } else if (p != src_end) {
             _ASSERT(curr_type != kNoCoding);
-            x_AddBoundary((p - src) * m_SrcDensity, curr_type);
+            x_AddBoundary(TSeqPos((p - src) * m_SrcDensity), curr_type);
             prev_type = curr_type;
         }
     }
@@ -1152,14 +1158,22 @@ SIZE_TYPE CSeqConvert_imp::CPacker::Pack(const char* src, TSeqPos length)
 
     // XXX - fine-tune 4na/2na boundaries to minimize wastage?
 
-    size_t n = best_arrangement->codings.size();
-    _ASSERT(n == m_Boundaries.size() - 1);
+    size_t n = m_Boundaries.size() - 1;
+    vector<TCoding> coding_vec(n);
+    SCodings*       codings = best_arrangement->codings;
+    for (size_t i = 0;  i < n;
+         i += codings->current_used, codings = codings->previous) {
+        _ASSERT(codings != NULL);
+        memcpy(coding_vec.data() + (n - i - codings->current_used),
+               codings->current, codings->current_used * sizeof(TCoding));
+    }
+    _ASSERT(codings == NULL);
 
     SIZE_TYPE result = 0;
     for (size_t i = 0;  i < n;  ++i) {
-        TCoding coding  = best_arrangement->codings[i];
+        TCoding coding  = coding_vec[i];
         TSeqPos start   = m_Boundaries[i];
-        while (i < n - 1  &&  best_arrangement->codings[i + 1] == coding) {
+        while (i < n - 1  &&  coding_vec[i + 1] == coding) {
             ++i; // merge when possible
         }
 
@@ -1182,8 +1196,8 @@ void CSeqConvert_imp::CPacker::x_AddBoundary(TSeqPos pos, TCoding new_coding)
     if (m_Boundaries.empty()) {
         _ASSERT(pos == 0);
         m_Boundaries.push_back(pos);
-        m_EndingNarrow.codings.push_back(new_coding);
-        m_EndingWide.codings.push_back(m_WideCoding);
+        m_EndingNarrow.AddCoding(new_coding);
+        m_EndingWide.AddCoding(m_WideCoding);
         m_EndingWide.cost   = m_Target.GetOverhead(m_WideCoding);
         m_EndingNarrow.cost = m_Target.GetOverhead(new_coding);
         return;
@@ -1193,7 +1207,7 @@ void CSeqConvert_imp::CPacker::x_AddBoundary(TSeqPos pos, TCoding new_coding)
     _ASSERT(last_length > 0);
     m_Boundaries.push_back(pos);
 
-    TCoding last_narrow = m_EndingNarrow.codings.back();
+    TCoding last_narrow = m_EndingNarrow.codings->GetLast();
     // This always rounds up to full bytes, and as such can slightly
     // overestimate the total cost of ncbi4na.
     m_EndingNarrow.cost += GetBytesNeeded(last_narrow,  last_length);
@@ -1230,10 +1244,46 @@ void CSeqConvert_imp::CPacker::x_AddBoundary(TSeqPos pos, TCoding new_coding)
     }
     // _TRACE('(' << m_EndingNarrow.cost << ", " << m_EndingWide.cost << ')');
 
-    m_EndingNarrow.codings.push_back(new_coding);
-    m_EndingWide.codings.push_back(m_WideCoding);
-    _ASSERT(m_EndingNarrow.codings.size() == m_Boundaries.size());
-    _ASSERT(m_EndingWide.codings.size() == m_Boundaries.size());    
+    m_EndingNarrow.AddCoding(new_coding);
+    m_EndingWide.AddCoding(m_WideCoding);
+}
+
+CSeqConvert_imp::CPacker::SArrangement&
+CSeqConvert_imp::CPacker::SArrangement::operator= (SArrangement& arr)
+{
+    _ASSERT(arr.shared_codings == shared_codings);
+    if (codings->previous == shared_codings) {
+        _ASSERT(arr.codings->previous == shared_codings);
+        codings->current_used = arr.codings->current_used;
+        memcpy(codings->current, arr.codings->current,
+               codings->current_used * sizeof(TCoding));
+    } else {
+        Reset();
+        codings = new SCodings(*arr.codings);
+        shared_codings = arr.shared_codings = codings->previous;
+    }
+    cost = arr.cost;
+    return *this;
+}
+
+void CSeqConvert_imp::CPacker::SArrangement::Reset(void)
+{
+    while (codings != NULL  &&  codings != shared_codings) {
+        SCodings* previous = codings->previous;
+        delete codings;
+        codings = previous;
+    }
+}
+
+void CSeqConvert_imp::CPacker::SArrangement::AddCoding(TCoding coding)
+{
+    if (codings == NULL  ||  codings->current_used == SCodings::kBlockSize) {
+        SCodings* new_codings = new SCodings;
+        new_codings->previous = codings;
+        new_codings->current_used = 0;
+        codings = new_codings;
+    }
+    codings->current[codings->current_used++] = coding;
 }
 
 CSeqUtil::ECoding
